@@ -1,26 +1,24 @@
 /****************************************************************************
-** $Id: pref.cpp,v 1.16 1997/09/29 19:48:15 eiriken Exp $
+** $Id: pref.cpp,v 1.21 1998/06/22 15:22:08 warwick Exp $
 **
-** Silly little preferences example
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** Copyright (C) 1996 by Troll Tech AS.  All rights reserved.
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
 **
 *****************************************************************************/
 
 #include "pref.h"
-#include <qtabdlg.h>
-#include <qmlined.h>
-#include <qbttngrp.h>
-#include <qradiobt.h>
+#include <qtabdialog.h>
+#include <qmultilinedit.h>
+#include <qbuttongroup.h>
+#include <qradiobutton.h>
 #include <qlabel.h>
 #include <qslider.h>
 #include <qlayout.h>
 #include <stdio.h>
 #include <qstring.h>
 
-RCSTAG("$Id: pref.cpp,v 1.16 1997/09/29 19:48:15 eiriken Exp $");
-
-
 Preferences::Preferences( QWidget * parent , const char * name )
     : QLabel( parent, name )
 {
@@ -195,5 +193,3 @@ void Preferences::apply()
 
     repaint();
 }
-
-
