@@ -1,14 +1,16 @@
-//
-// Qt Example Application: timestmp
-//
-// Demonstrates QDir, QFile, QFileInfo, QDataStream and QDateTime
-//
-// See the README file for more information.
-//
+/****************************************************************************
+** $Id: timestmp.cpp,v 2.4 1998/06/16 11:39:35 warwick Exp $
+**
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
 
 #include <qdir.h>
-#include <qdstream.h>
-#include <qdatetm.h>
+#include <qdatastream.h>
+#include <qdatetime.h>
 #include <qstring.h>
 
 const char *dName = "time.qt";             // dir to make under home dir
