@@ -1,11 +1,23 @@
 /****************************************************************************
-** $Id: qgl.cpp,v 1.15.2.2 1997/12/18 19:10:59 hanord Exp $
+** $Id: qgl.cpp,v 1.24 1998/07/03 00:09:27 hanord Exp $
 **
 ** Implementation of OpenGL classes for Qt
 **
 ** Created : 970112
 **
-** Copyright (C) 1997 by Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+**
+** This file is part of Qt Free Edition, version 1.40.
+**
+** See the file LICENSE included in the distribution for the usage
+** and distribution terms, or http://www.troll.no/free-license.html.
+**
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
@@ -26,9 +38,6 @@
 #include <X11/Xmu/StdCmap.h>
 #endif
 
-RCSTAG("$Id: qgl.cpp,v 1.15.2.2 1997/12/18 19:10:59 hanord Exp $");
-
-
 #if defined(_CC_MSVC_)
 #pragma warning(disable:4355) // 'this' : used in base member initializer list
 #endif
@@ -56,6 +65,8 @@ const char *qGLVersion()
   \brief The QGLFormat class specifies the display format of an OpenGL
   rendering context.
 
+  \extension OpenGL
+
   A display format has several characteristics:
   <ul>
   <li> \link setDoubleBuffer() Double or single buffering.\endlink
@@ -95,9 +106,10 @@ const char *qGLVersion()
     f.setAlpha( TRUE );
     f.setStereo( TRUE );
     w->setFormat( f );
-    if ( !w->create() ) {
+    if ( !w->isValid() ) {
         f.setStereo( FALSE );	// ok, goggles off
-        if ( !w->create() ) {
+        w->setFormat( f );
+        if ( !w->isValid() ) {
             fatal( "Cool hardware wanted" );
         }
     }
@@ -131,7 +143,7 @@ const char *qGLVersion()
 
 QGLFormat::QGLFormat( bool doubleBuffer )
 {
-    data = new Internal;
+    data = new FormatFlags;
     CHECK_PTR( data );
     data->doubleBuffer = doubleBuffer;
     data->depth	       = TRUE;
@@ -461,6 +473,8 @@ void QGLFormat::setDefaultFormat( const QGLFormat &f )
   \class QGLContext qgl.h
   \brief The QGLContext class encapsulates an OpenGL rendering context.
 
+  \extension OpenGL
+
   An OpenGL rendering context is a complete set of OpenGL state
   variables.
 
@@ -470,7 +484,7 @@ void QGLFormat::setDefaultFormat( const QGLFormat &f )
 /*!
   Constructs an OpenGL context for the paint device \a device,
   which must be a widget or a pixmap.
-  The \a format specify several display options for this context.
+  The \a format specifies several display options for this context.
 
   The context will be \link isValid() invalid\endlink if the \a format
   settings cannot be satified by the underlying OpenGL/Window system.
@@ -550,6 +564,9 @@ void QGLContext::setFormat( const QGLFormat &format )
   a context that satisfies the requested \link setFormat() format\endlink,
   otherwise FALSE is returned (the context is invalid).
 
+  If successful and the \a shareContext points to a valid QGLContext,
+  this context will share OpenGL display lists with \a shareContext.
+
   <strong>Implementation note:</strong> Initialization of C++ class members
   usually takes place in the class constructor. QGLContext is an exception
   because it must be simple to customize. The virtual functions
@@ -561,17 +578,17 @@ void QGLContext::setFormat( const QGLFormat &format )
   \sa chooseContext(), isValid()
 */
 
-bool QGLContext::create()
+bool QGLContext::create( const QGLContext* shareContext )
 {
     reset();
-    valid = chooseContext();
+    valid = chooseContext( shareContext );
     return valid;
 }
 
 
 
 /*!
-  \fn bool QGLContext::chooseContext()
+  \fn bool QGLContext::chooseContext( const QGLContext* shareContext = 0 )
 
   This semi-internal function is called by create(). It creates a
   system-dependent OpenGL handle that matches the specified \link
@@ -638,7 +655,7 @@ bool QGLContext::create()
  *****************************************************************************/
 
 
-bool QGLContext::chooseContext()
+bool QGLContext::chooseContext( const QGLContext* shareContext )
 {
     bool success = FALSE;
     if ( paintDevice->devType() == PDT_WIDGET ) {
@@ -668,6 +685,8 @@ bool QGLContext::chooseContext()
 #endif
 	}
     }
+    if ( success && shareContext && shareContext->isValid() )
+	success = wglShareLists( shareContext->rc, rc );
     if ( win ) {
 	ReleaseDC( win, dc );
 	dc = 0;
@@ -801,13 +820,17 @@ void QGLContext::swapBuffers()
   QGLContext UNIX/GLX-specific code
  *****************************************************************************/
 
-bool QGLContext::chooseContext()
+bool QGLContext::chooseContext( const QGLContext* shareContext )
 {
     vi = chooseVisual();
     if ( vi == 0 )
 	return FALSE;
-    cx = glXCreateContext( paintDevice->x11Display(), (XVisualInfo *)vi,
-			   None, TRUE );
+    if ( shareContext && shareContext->cx )
+	cx = glXCreateContext( paintDevice->x11Display(), (XVisualInfo *)vi,
+			       (GLXContext)shareContext->cx, TRUE );
+    else
+	cx = glXCreateContext( paintDevice->x11Display(), (XVisualInfo *)vi,
+			       None, TRUE );
     return cx != 0;
 }
 
@@ -926,6 +949,8 @@ void QGLContext::swapBuffers()
   \class QGLWidget qgl.h
   \brief The QGLWidget class is a widget for rendering OpenGL graphics.
 
+  \extension OpenGL
+
   It is easy to render OpenGL graphics in Qt applications. You can
   create a subclass of QGLWidget and reimplement two functions:
   resizeGL() and paintGL(). The resizeGL() method is called whenever
@@ -979,19 +1004,28 @@ void QGLContext::swapBuffers()
   The \e parent, \e name and \e f arguments are passed to the QWidget
   constructor.
 
-  Note: If the constructor of your QGLWidget subclass calls any OpenGL
-  functions, it must first call makeCurrent() to set the correct
-  rendering context.
+  If the \a shareWidget parameter points to a valid QGLWidget, this
+  widget will share OpenGL display lists with \a shareWidget.
+
+  Note: Initialization of OpenGL rendering state etc. should be done
+  by overriding the initializeGL() function, not in the constructor of
+  your QGLWidget subclass.
 
   \sa QGLFormat::defaultFormat()
 */
 
-QGLWidget::QGLWidget( QWidget *parent, const char *name, WFlags f )
+QGLWidget::QGLWidget( QWidget *parent, const char *name,
+		      const QGLWidget* shareWidget, WFlags f )
     : QWidget(parent, name, f)
 {
-    setBackgroundColor( black );
+    setBackgroundMode( NoBackground );
+    initDone = FALSE;
     glcx = 0;
-    setContext( new QGLContext(QGLFormat::defaultFormat(),this) );
+    if ( shareWidget )
+	setContext( new QGLContext( QGLFormat::defaultFormat(), this ),
+		    shareWidget->context() );
+    else
+	setContext( new QGLContext( QGLFormat::defaultFormat(), this ) );
 }
 
 
@@ -1005,20 +1039,28 @@ QGLWidget::QGLWidget( QWidget *parent, const char *name, WFlags f )
   The \e parent, \e name and \e f arguments are passed to the QWidget
   constructor.
 
-  Note: If the constructor of your QGLWidget subclass calls any OpenGL
-  functions, it must first call makeCurrent() to set the correct
-  rendering context.
+  If the \a shareWidget parameter points to a valid QGLWidget, this
+  widget will share OpenGL display lists with \a shareWidget.
+
+  Note: Initialization of OpenGL rendering state etc. should be done
+  by overriding the initializeGL() function, not in the constructor of
+  your QGLWidget subclass.
 
   \sa QGLFormat::defaultFormat(), isValid()
 */
 
 QGLWidget::QGLWidget( const QGLFormat &format, QWidget *parent,
-		      const char *name, WFlags f )
+		      const char *name, const QGLWidget* shareWidget,
+		      WFlags f )
     : QWidget(parent, name, f)
 {
-    setBackgroundColor( black );
+    setBackgroundMode( NoBackground );
+    initDone = FALSE;
     glcx = 0;
-    setContext( new QGLContext(format,this) );
+    if ( shareWidget )
+	setContext( new QGLContext( format, this ), shareWidget->context() );
+    else
+	setContext( new QGLContext( format, this ) );
 }
 
 
@@ -1071,6 +1113,10 @@ QGLWidget::~QGLWidget()
 /*!
   Sets a new format for this widget. The widget becomes invalid if the
   requested format cannot be satisfied.
+
+  Calling setFormat() will not alter any display list sharing this
+  widget currently has.
+
   \sa format(), setContext(), isValid()
 */
 
@@ -1193,10 +1239,17 @@ static Colormap choose_cmap( Display *dpy, XVisualInfo *vi )
   Sets a new context for this widget. The context must be created using
   \e new.  QGLWidget will delete the context when another context is set
   or when the widget is destroyed.
+
+  Calling setContext() will not alter any display list sharing this
+  widget currently has, unless you explicitly request sharing with a
+  different context by giving a \a shareContext parameter that points
+  to a valid context.
+
   \sa context(), setFormat()
 */
 
-void QGLWidget::setContext( QGLContext *context )
+void QGLWidget::setContext( QGLContext *context,
+			    const QGLContext* shareContext )
 {
     if ( context == 0 ) {
 #if defined(CHECK_NULL)
@@ -1209,15 +1262,15 @@ void QGLWidget::setContext( QGLContext *context )
 #endif
     }
 
-#if defined(Q_WGL)
-    bool has_cx = glcx != 0;
-#endif
-    delete glcx;
+    QGLContext* oldcx = glcx;
+
     glcx = context;
+    initDone = FALSE;
+    bool createFailed = FALSE;
 
 #if defined(Q_WGL)
 
-    if ( has_cx ) {
+    if ( oldcx ) {
 	// We already have a context and must therefore create a new
 	// window since Windows does not permit setting a new OpenGL
 	// context for a window that already has one set.
@@ -1225,18 +1278,18 @@ void QGLWidget::setContext( QGLContext *context )
 	create( 0, TRUE, TRUE );
     }
 
-    if ( !glcx->isValid() ) {
-	if ( !glcx->create() )
-	    return;
-    }
-
 #endif
 
-#if defined(Q_GLX)
     if ( !glcx->isValid() ) {
-	if ( !glcx->create() )
-	    return;
+	if ( !glcx->create( shareContext ? shareContext : oldcx ) )
+	    createFailed = TRUE;
     }
+    delete oldcx;
+    if ( createFailed )
+	return;
+
+#if defined(Q_GLX)
+
     bool visible = isVisible();
     if ( visible )
 	hide();
@@ -1280,7 +1333,7 @@ void QGLWidget::setContext( QGLContext *context )
     glXReleaseBuffersMESA( dpy, winId() );
 #endif
     create( w );
-    clearWFlags( WState_Visible );		// workaround for Qt 1.30 bug
+    clearWFlags( WState_Visible );   //## workaround for Qt 1.30 bug
 
     XSetWMColormapWindows( dpy, topLevelWidget()->winId(), cmw, count );
     delete [] cmw;
@@ -1298,12 +1351,29 @@ void QGLWidget::setContext( QGLContext *context )
 */
 
 
+/*!
+  This virtual function is called one time before the first call to
+  paintGL() or resizeGL(), and then one time whenever the widget has
+  been assigned a new OpenGL context.  Reimplement it in a subclass.
+
+  This function should take care of setting OpenGL rendering flags,
+  defining display lists, etc.
+
+  There is no need to call makeCurrent() because this has already been
+  done when this function is called.
+*/
+
+void QGLWidget::initializeGL()
+{
+}
+
+
 /*!
   This virtual function is called whenever the widget needs to be painted.
   Reimplement it in a subclass.
 
   There is no need to call makeCurrent() because this has already been
-  done when the function is called.
+  done when this function is called.
 */
 
 void QGLWidget::paintGL()
@@ -1317,7 +1387,7 @@ void QGLWidget::paintGL()
   Reimplement it in a subclass.
 
   There is no need to call makeCurrent() because this has already been
-  done when the function is called.
+  done when this function is called.
 */
 
 void QGLWidget::resizeGL( int, int )
@@ -1325,6 +1395,8 @@ void QGLWidget::resizeGL( int, int )
 }
 
 
+
+
 /*!
   Handles paint events. Calls the virtual function paintGL().
 */
@@ -1332,6 +1404,10 @@ void QGLWidget::resizeGL( int, int )
 void QGLWidget::paintEvent( QPaintEvent * )
 {
     makeCurrent();
+    if ( !initDone ) {
+	gl_init();
+	resizeGL( width(), height() );	// New context needs this "resize"
+    }
     paintGL();
     if ( doubleBuffer() )
 	swapBuffers();
@@ -1347,6 +1423,8 @@ void QGLWidget::paintEvent( QPaintEvent * )
 void QGLWidget::resizeEvent( QResizeEvent * )
 {
     makeCurrent();
+    if ( !initDone )
+	gl_init();
 #if defined(Q_GLX)
     glXWaitX();
 #endif
@@ -1354,12 +1432,32 @@ void QGLWidget::resizeEvent( QResizeEvent * )
 }
 
 
+/*!
+  Initializes OpenGL for this widget's context. Calls the virtual
+  function initializeGL().
+*/
+
+void QGLWidget::gl_init()
+{
+    /*
+    if ( isValid() ) {
+	glClearColor( 0.0, 0.0, 0.0, 0.0 ); // Let OpenGL clear to black
+	QGLFormat* fmt = format();
+	if ( fmt->depth() )
+	    glEnable( GL_DEPTH_TEST );
+	if ( fmt->stencil() )
+	    glEnable( GL_STENCIL_TEST );
+    }
+    */
+    initializeGL();
+    initDone = TRUE;
+}
+
 /*****************************************************************************
   QGL classes overview documentation.
  *****************************************************************************/
 
-/*!
-\page qgl.html
+/*! \page qgl.html
 
 <title>Qt OpenGL Classes</title>
 </head><body bgcolor="#ffffff">
