@@ -1,27 +1,33 @@
 /****************************************************************************
-** $Id: qbrush.h,v 2.1 1996/11/22 18:31:52 agulbra Exp $
+** $Id: qbrush.h,v 2.4 1998/07/03 00:09:30 hanord Exp $
 **
 ** Definition of QBrush class
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QBRUSH_H
 #define QBRUSH_H
 
+#ifndef QT_H
 #include "qcolor.h"
 #include "qshared.h"
+#endif // QT_H
 
 
 enum BrushStyle					// brush style
