@@ -1,32 +1,35 @@
 /****************************************************************************
-** $Id: qbutton.h,v 2.11 1997/08/06 15:05:56 hanord Exp $
+** $Id: qbutton.h,v 2.18 1998/07/03 00:09:47 hanord Exp $
 **
 ** Definition of QButton widget class
 **
 ** Created : 940206
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QBUTTON_H
 #define QBUTTON_H
 
+#ifndef QT_H
 #include "qwidget.h"
+#endif // QT_H
 
 
-class QButtonGroup;				// button group (qbttngrp.h)
+class QButtonGroup;				// button group (qbuttongroup.h)
 struct QButtonData;
 
 
@@ -47,7 +50,9 @@ public:
 
     bool	isToggleButton() const;
 
+    void	setDown( bool );
     bool	isDown() const;
+
     bool	isOn() const;
 
     bool	autoResize() const;
@@ -58,6 +63,7 @@ public:
 
 public slots:
     void	animateClick();
+    void	toggle();
 
 signals:
     void	pressed();
@@ -68,7 +74,6 @@ signals:
 protected:
     void	setToggleButton( bool );
     void	setOn( bool );
-    void	setDown( bool );
 
     virtual bool hitButton( const QPoint &pos ) const;
     virtual void drawButton( QPainter * );
@@ -105,10 +110,9 @@ private:
     QButtonGroup *group() const;
     void	  setGroup( QButtonGroup* );
     QTimer	 *timer();
-
 private:	// Disabled copy constructor and operator=
-    QButton( const QButton & ) {}
-    QButton &operator=( const QButton & ) { return *this; }
+    QButton( const QButton & );
+    QButton &operator=( const QButton & );
 };
 
 
