@@ -1,29 +1,32 @@
 /****************************************************************************
-** $Id: qintdict.h,v 2.2 1997/08/06 15:05:53 hanord Exp $
+** $Id: qintdict.h,v 2.5 1998/07/03 00:09:46 hanord Exp $
 **
 ** Definition of QIntDict template/macro class
 **
 ** Created : 940624
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QINTDICT_H
 #define QINTDICT_H
 
+#ifndef QT_H
 #include "qgdict.h"
+#endif // QT_H
 
 
 #if defined(USE_MACROCLASS)
@@ -60,6 +63,7 @@ public:									      \
     bool  remove( long k )	{ return QGDict::remove((const char*)k); }    \
     type *take( long k )	{ return (type*)QGDict::take((const char*)k);}\
     void  clear()		{ QGDict::clear(); }			      \
+    void  resize( uint n )	{ QGDict::resize(n); }			      \
     type *find( long k )	const					      \
 	{ return (type *)((QGDict*)this)->QGDict::look((const char*)k,0,0);}  \
     type *operator[]( long k ) const					      \
@@ -122,6 +126,7 @@ public:
     bool  remove( long k )	{ return QGDict::remove((const char*)k); }
     type *take( long k )	{ return (type*)QGDict::take((const char*)k); }
     void  clear()		{ QGDict::clear(); }
+    void  resize( uint n )	{ QGDict::resize(n); }
     type *find( long k )	const
 	{ return (type *)((QGDict*)this)->QGDict::look((const char*)k,0,0); }
     type *operator[]( long k ) const
