@@ -1,29 +1,32 @@
 /****************************************************************************
-** $Id: qmenubar.h,v 2.7 1997/09/05 20:00:43 agulbra Exp $
+** $Id: qmenubar.h,v 2.12 1998/07/03 00:09:51 hanord Exp $
 **
 ** Definition of QMenuBar class
 **
 ** Created : 941209
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QMENUBAR_H
 #define QMENUBAR_H
 
-#include "qpopmenu.h"
+#ifndef QT_H
+#include "qpopupmenu.h"
+#endif // QT_H
 
 
 class QMenuBar : public QFrame, public QMenuData
@@ -87,12 +90,13 @@ private:
     int		itemAtPos( const QPoint & );
     QRect	itemRect( int item );
 
+    void	setupAccelerators();
     QAccel     *autoaccel;
     QRect      *irects;
 
 private:	// Disabled copy constructor and operator=
-    QMenuBar( const QMenuBar & ) {}
-    QMenuBar &operator=( const QMenuBar & ) { return *this; }
+    QMenuBar( const QMenuBar & );
+    QMenuBar &operator=( const QMenuBar & );
 };
 
 
