@@ -1,29 +1,32 @@
 /****************************************************************************
-** $Id: qsemimodal.h,v 2.1 1997/06/27 11:35:16 warwick Exp $
+** $Id: qsemimodal.h,v 2.4 1998/07/03 00:09:40 hanord Exp $
 **
 ** Definition of QSemiModal class
 **
 ** Created : 970627
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QSEMIMODAL_H
 #define QSEMIMODAL_H
 
+#ifndef QT_H
 #include "qwidget.h"
+#endif // QT_H
 
 
 class QSemiModal : public QWidget
@@ -47,8 +50,8 @@ private:
     uint	did_resize : 1;
 
 private:	// Disabled copy constructor and operator=
-    QSemiModal( const QSemiModal & ) {}
-    QSemiModal &operator=( const QSemiModal & ) { return *this; }
+    QSemiModal( const QSemiModal & );
+    QSemiModal &operator=( const QSemiModal & );
 };
 
 
