@@ -1,29 +1,32 @@
 /****************************************************************************
-** $Id: qsize.h,v 2.3 1997/08/27 13:37:46 agulbra Exp $
+** $Id: qsize.h,v 2.7 1998/07/03 00:09:41 hanord Exp $
 **
 ** Definition of QSize class
 **
 ** Created : 931028
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QSIZE_H
 #define QSIZE_H
 
+#ifndef QT_H
 #include "qpoint.h"
+#endif // QT_H
 
 #if defined(QSIZE_C) || defined(DEBUG)
 #define QSIZE_DEBUG
@@ -46,6 +49,9 @@ public:
     void   setHeight( int h );
     void   transpose();
 
+    QSize expandedTo( const QSize & ) const;
+    QSize boundedTo( const QSize & ) const;
+
     QCOORD &rwidth();
     QCOORD &rheight();
 
@@ -173,4 +179,14 @@ inline QSize operator/( const QSize &s, float c )
 #endif // no-debug functions
 
 
+inline QSize QSize::expandedTo( const QSize & otherSize ) const
+{
+    return QSize( QMAX( wd, otherSize.wd ), QMAX( ht, otherSize.ht ) );
+}
+
+inline QSize QSize::boundedTo( const QSize & otherSize ) const
+{
+    return QSize( QMIN( wd, otherSize.wd ), QMIN( ht, otherSize.ht ) );
+}
+
 #endif // QSIZE_H
