@@ -1,30 +1,33 @@
 /****************************************************************************
-** $Id: qslider.h,v 2.20 1997/09/26 10:57:41 eiriken Exp $
+** $Id: qslider.h,v 2.24 1998/07/03 00:09:53 hanord Exp $
 **
 ** Definition of QSlider class
 **
 ** Created : 961019
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QSLIDER_H
 #define QSLIDER_H
 
+#ifndef QT_H
 #include "qwidget.h"
-#include "qrangect.h"
+#include "qrangecontrol.h"
+#endif // QT_H
 
 
 class QTimer;
@@ -119,8 +122,8 @@ private:
     Orientation orient;
 
 private:	// Disabled copy constructor and operator=
-    QSlider( const QSlider & ) {}
-    QSlider &operator=( const QSlider & ) { return *this; }
+    QSlider( const QSlider & );
+    QSlider &operator=( const QSlider & );
 };
 
 inline bool QSlider::tracking() const
