@@ -1,37 +1,38 @@
 /****************************************************************************
-** $Id: qaccel.cpp,v 2.2 1996/10/07 08:23:18 hanord Exp $
+** $Id: qaccel.cpp,v 2.14 1998/07/03 00:09:29 hanord Exp $
 **
 ** Implementation of QAccel class
 **
 ** Created : 950419
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #define QAccelList QListM_QAccelItem
 #include "qaccel.h"
-#include "qapp.h"
+#include "qapplication.h"
 #include "qwidget.h"
 #include "qlist.h"
 #include "qsignal.h"
 
-RCSTAG("$Id: qaccel.cpp,v 2.2 1996/10/07 08:23:18 hanord Exp $");
-
-
 /*!
   \class QAccel qaccel.h
   \brief The QAccel class handles keyboard accelerator keys.
 
-  \ingroup uiclasses
+  \ingroup kernel
 
   A QAccel contains a list of accelerator items. Each accelerator item
   consists of an identifier and a keyboard code combined with modifiers
@@ -45,8 +46,11 @@ RCSTAG("$Id: qaccel.cpp,v 2.2 1996/10/07 08:23:18 hanord Exp $");
   items can also be individually connected, so that two different keys
   will activate two different slots (see connectItem()).
 
-  A QAccel object handles key events to its parent widget and all children
-  of this parent widget.
+  A QAccel object handles key events to the
+  \link QWidget::topLevelWidget() top level window\endlink
+  containing \a parent, and hence to any child widgets of that window.
+  Note that the accelerator will be deleted only when the \a parent
+  is deleted, and will consume relevant key events until then.
 
   Example:
   \code
@@ -55,6 +59,10 @@ RCSTAG("$Id: qaccel.cpp,v 2.2 1996/10/07 08:23:18 hanord Exp $");
 		     myWindow,			// connected to myWindow's
 		     SLOT(printDoc()) );	// printDoc() slot
   \endcode
+
+  \sa QKeyEvent QWidget::keyPressEvent() QMenuData::setAccel()
+  QButton::setAccel()
+  <a href="guibooks.html#fowler">GUI Design Handbook: Keyboard Shortcuts,</a>
 */
 
 
@@ -106,7 +114,10 @@ QAccel::QAccel( QWidget *parent, const char *name )
     CHECK_PTR( aitems );
     aitems->setAutoDelete( TRUE );
     enabled = TRUE;
-    if ( parent && parent->isWidgetType() ) {	// install event filter
+    if ( parent ) {				// install event filter
+#if defined(CHECK_RANGE)
+	ASSERT( parent->isWidgetType() );
+#endif
 	tlw = parent->topLevelWidget();
 	tlw->installEventFilter( this );
 	connect( tlw, SIGNAL(destroyed()), SLOT(tlwDestroyed()) );
@@ -130,7 +141,6 @@ QAccel::~QAccel()
 }
 
 
-
 /*!
   \fn void QAccel::activated( int id )
   This signal is emitted when an accelerator key is pressed. \e id is
@@ -182,8 +192,8 @@ uint QAccel::count() const
     a->insertItem( Key_P + CTRL, 200 );		// Ctrl+P to print document
     a->insertItem( Key_X + ALT , 201 );		// Alt+X  to quit
     a->insertItem( ASCII_ACCEL + 'q', 202 );	// ASCII 'q' to quit
-    a->insertItem( Key_D );			// gets id 2
-    a->insertItem( Key_P + CTRL + SHIFT );	// gets id 3
+    a->insertItem( Key_D );			// gets id 3
+    a->insertItem( Key_P + CTRL + SHIFT );	// gets id 4
   \endcode
 */
 
@@ -227,6 +237,7 @@ int QAccel::key( int id )
     return item ? item->key : 0;
 }
 
+
 /*!
   Returns the identifier of the accelerator item with the key code \e key, or
   -1 if the item cannot be found.
@@ -251,6 +262,7 @@ bool QAccel::isItemEnabled( int id ) const
     return item ? item->enabled : FALSE;
 }
 
+
 /*!
   Enables or disables an accelerator item.
   \arg \e id is the item identifier.
@@ -310,13 +322,45 @@ bool QAccel::disconnectItem( int id, const QObject *receiver,
 }
 
 
+/*!
+  Make sure that the accelerator is watching the correct event
+  filter.  Used by QWidget::recreate().
+*/
+
+void QAccel::repairEventFilter()
+{
+    QWidget *ntlw;
+    if ( parent() ) {
+#if defined(CHECK_RANGE)
+	ASSERT( parent()->isWidgetType() );
+#endif
+	ntlw = ((QWidget*)parent())->topLevelWidget();
+    } else {
+	ntlw = 0;
+    }
+    if ( tlw != ntlw ) {
+	if ( tlw ) {
+	    tlw->removeEventFilter( this );
+	    disconnect( tlw, SIGNAL(destroyed()), this, SLOT(tlwDestroyed()) );
+	}
+	tlw = ntlw;
+	if ( tlw ) {
+	    tlw->installEventFilter( this );
+	    connect( tlw, SIGNAL(destroyed()), this, SLOT(tlwDestroyed()) );
+	}
+    }
+}
+
+
 /*!
   Processes accelerator events intended for the top level widget.
 */
 
 bool QAccel::eventFilter( QObject *, QEvent *e )
 {
-    if ( enabled && e->type() == Event_Accel ) {
+    if ( enabled && e->type() == Event_Accel &&
+	 parent() && parent()->isWidgetType() &&
+	 ((QWidget *)parent())->isVisibleToTLW() ) {
 	QKeyEvent *k = (QKeyEvent *)e;
 	int key = k->key();
 	if ( k->state() & ShiftButton )
@@ -349,3 +393,71 @@ void QAccel::tlwDestroyed()
 {
     tlw = 0;
 }
+
+
+/* 62 defined shortcuts from the microsoft book:
+
+<b><u>A</u></b>bout
+Insert <b><u>O</u></b>bject
+<b><u>Q</u></b>uick View
+Always on <b><u>T</u></b>op
+<b><u>L</u></b>ink Here
+<b><u>R</u></b>edo
+<b><u>A</u></b>pply
+Ma<b><u>x</u></b>imize
+<b><u>R</u></b>epeat
+<b><u>B</u></b>ack
+Mi<b><u>n</u></b>imize
+<b><u>R</u></b>estore
+<b><u>B</u></b>rowse
+<b><u>M</u></b>ove
+<b><u>R</u></b>esume
+<b><u>C</u></b>lose
+<b><u>M</u></b>ove Here
+<b><u>R</u></b>etry
+<b><u>C</u></b>opy
+<b><u>N</u></b>ew
+<b><u>R</u></b>un
+<b><u>C</u></b>opy Here
+<b><u>N</u></b>ext
+<b><u>S</u></b>ave
+Create <b><u>S</u></b>hortcut
+<b><u>N</u></b>o
+Save <b><u>A</u></b>s
+Create <b><u>S</u></b>hortcut Here
+<b><u>O</u></b>pen
+Select <b><u>A</u></b>ll
+Cu<b><u>t</u></b>
+Open <b><u>W</u></b>ith
+Se<b><u>n</u></b>d To
+<b><u>D</u></b>elete
+<b><u>P</u></b>aste
+<b><u>S</u></b>how
+<b><u>E</u></b>dit
+Paste <b><u>L</u></b>ink
+<b><u>S</u></b>ize
+<b><u>E</u></b>xit
+Paste <b><u>S</u></b>hortcut
+S<b><u>p</u></b>lit
+<b><u>E</u></b>xplore
+Page Set<b><u>u</u></b>p
+<b><u>S</u></b>top
+<b><u>F</u></b>ile
+Paste <b><u>S</u></b>pecial
+<b><u>U</u></b>ndo
+<b><u>F</u></b>ind
+<b><u>P</u></b>ause
+<b><u>V</u></b>iew
+<b><u>H</u></b>elp
+<b><u>P</u></b>lay
+<b><u>W</u></b>hat's This?
+Help <b><u>T</u></b>opics
+<b><u>P</u></b>rint
+<b><u>W</u></b>indow
+<b><u>H</u></b>ide
+<b><u>P</u></b>rint Here
+<b><u>Y</u></b>es
+<b><u>I</u></b>nsert
+P<b><u>r</u></b>operties
+
+*/
