@@ -1,27 +1,28 @@
 /****************************************************************************
-** $Id: qevent.cpp,v 2.2 1997/03/12 09:06:47 aavit Exp $
+** $Id: qevent.cpp,v 2.18 1998/07/03 00:09:32 hanord Exp $
 **
 ** Implementation of event classes
 **
 ** Created : 931029
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qevent.h"
 
-RCSTAG("$Id: qevent.cpp,v 2.2 1997/03/12 09:06:47 aavit Exp $");
-
-
 void qRemovePostedEvent( QEvent * );		// defined in qapp_xxx.cpp
 
 
@@ -102,33 +103,35 @@ void QEvent::peErrMsg()				// posted event error message
   event classes. Event objects contain event parameters.
 
   \ingroup event
+  \ingroup kernel
 
   The \link QApplication::exec() main event loop\endlink of Qt fetches
   native window system events from the event queue, translates them
-  to Qt events and sends the translated events to application objects.
+  into QEvent and sends the translated events to QObjects.
 
-  Generally, events come from the underlying window system, but it is also
-  possible to manually send events through the QApplication class using
-  QApplication::sendEvent() and QApplication::postEvent().
+  Generally, events come from the underlying window system, but it is
+  also possible to manually send events through the QApplication class
+  using QApplication::sendEvent() and QApplication::postEvent().
 
-  Only classes that inherit QObject and reimplement the virtual
-  QObject::event() function may receive events.
-
-  The QWidget class reimplements the event() function to
-  dispatch an event to an appropriate event handler (virtual function) on
-  basis of the event type.
-
-  QWidget::keyPressEvent() and QWidget::mouseMoveEvent() are examples of
-  widget event handlers.
+  QObject recevied events by having its QObject::event() function
+  called.  The default implementation simply calls
+  QObject::timerEvent() for timer events and ignores all other events.
+  QWidget reimplements \link QWidget::event() event() \endlink and
+  dispatches incoming events to various event handles such as
+  QWidget::keyPressEvent() and QWidget::mouseMoveEvent().
 
   The basic QEvent only contains an event type parameter.  Subclasses
   of QEvent contain additional parameters that descripe the particular
   event.
+
+  \sa QObject::event() QObject::installEventFilter() QWidget::event()
+  QApplication::sendEvent() QApplcation::postEvent()
+  QApplication::processEvents()
 */
 
 /*!
   \fn QEvent::QEvent( int type )
-  Contructs an event object with a \e type. The file qevent.h lists
+  Contructs an event object with a \a type. The file qevent.h lists
   all event types.
 */
 
@@ -151,6 +154,7 @@ void QEvent::peErrMsg()				// posted event error message
 
   \ingroup event
 
+
   Timer events are sent at regular intervals to objects that have
   started one or more timers.  Each timer has a unique identifier.
 
@@ -168,7 +172,7 @@ void QEvent::peErrMsg()				// posted event error message
 
 /*!
   \fn QTimerEvent::QTimerEvent( int timerId )
-  Constructs a timer event object with the timer identifier set to \e timerId.
+  Constructs a timer event object with the timer identifier set to \a timerId.
 */
 
 /*!
@@ -185,6 +189,7 @@ void QEvent::peErrMsg()				// posted event error message
 
   \ingroup event
 
+
   Mouse events occur when a mouse button is pressed or released inside a
   widget, or when the mouse cursor is moved.
 
@@ -192,7 +197,7 @@ void QEvent::peErrMsg()				// posted event error message
   unless \link QWidget::setMouseTracking() mouse tracking\endlink has been
   enabled.
 
-  Qt make an automatic mouse grab when a mouse button is pressed inside a
+  Qt makes an automatic mouse grab when a mouse button is pressed inside a
   widget, and the widget will continue to receive mouse events until the
   last mouse button is released.
 
@@ -206,7 +211,8 @@ void QEvent::peErrMsg()				// posted event error message
   QWidget::mouseDoubleClickEvent() and QWidget::mouseMoveEvent() receive
   mouse events.
 
-  \sa QWidget::setMouseTracking(), QWidget::grabMouse() */
+  \sa QWidget::setMouseTracking(), QWidget::grabMouse()
+*/
 
 /*!
   \fn QMouseEvent::QMouseEvent( int type, const QPoint &pos, int button, int state )
@@ -220,8 +226,23 @@ void QEvent::peErrMsg()				// posted event error message
 
 /*!
   \fn const QPoint &QMouseEvent::pos() const
-  Returns the position of the mouse relative to the widget that received the
-  event.
+  Returns the position of the mouse pointer, relative to the widget that
+  received the event.
+  \sa x(), y()
+*/
+
+/*!
+  \fn int QMouseEvent::x() const
+  Returns the X position of the mouse pointer, relative to the widget that
+  received the event.
+  \sa y(), pos()
+*/
+
+/*!
+  \fn int QMouseEvent::y() const
+  Returns the Y position of the mouse pointer, relative to the widget that
+  received the event.
+  \sa x(), pos()
 */
 
 /*!
@@ -231,8 +252,8 @@ void QEvent::peErrMsg()				// posted event error message
   Possible return values are \c LeftButton, \c RightButton, \c MidButton and
   \c NoButton.
 
-  Note that the returned value is always \c NoButton (0) when a mouse move
-  event is received.
+  Note that the returned value is always \c NoButton (0) for mouse move
+  events.
 
   \sa state()
 */
@@ -240,8 +261,11 @@ void QEvent::peErrMsg()				// posted event error message
 
 /*!
   \fn int QMouseEvent::state() const
-  Returns the current button state (a combination of mouse buttons and keyboard
-  modifiers).
+
+  Returns the current button state (a combination of mouse buttons and
+  keyboard modifiers), i.e. what buttons were depressed when the event
+  was generated. This does not include the event-causing button
+  itself.
 
   The returned value is \c LeftButton, \c RightButton, \c MidButton,
   \c ShiftButton, \c ControlButton and \c AltButton OR'ed together.
@@ -256,6 +280,7 @@ void QEvent::peErrMsg()				// posted event error message
 
   \ingroup event
 
+
   Key events occur when a key is pressed or released when a widget has
   keyboard input focus.
 
@@ -276,9 +301,9 @@ void QEvent::peErrMsg()				// posted event error message
   \fn QKeyEvent::QKeyEvent( int type, int key, int ascii, int state )
   Constructs a key event object.
 
-  The \e type parameter must be \c Event_KeyPress or \c Event_KeyRelease.
+  The \a type parameter must be \c Event_KeyPress or \c Event_KeyRelease.
 
-  If \e key is 0, the event is not a result of a known key (e.g. it
+  If \a key is 0, the event is not a result of a known key (e.g. it
   may be the result of a compose sequence or keyboard macro).
 
   The accept flag is set to TRUE.
@@ -286,7 +311,7 @@ void QEvent::peErrMsg()				// posted event error message
 
 /*!
   \fn int QKeyEvent::key() const
-  Returns the code if the key that was pressed or released.
+  Returns the code of the key that was pressed or released.
 
   The header file qkeycode.h lists the possible keyboard codes.	 These codes
   are independent of the underlying window system.
@@ -345,6 +370,7 @@ void QEvent::peErrMsg()				// posted event error message
 
   \ingroup event
 
+
   Focus events are sent to widgets when the keyboard input focus changes.
 
   The event handlers QWidget::focusInEvent() and QWidget::focusOutEvent()
@@ -357,7 +383,7 @@ void QEvent::peErrMsg()				// posted event error message
   \fn QFocusEvent::QFocusEvent( int type )
   Constructs a focus event object.
 
-  The \e type parameter must be either \e Event_FocusIn or \e Event_FocusOut.
+  The \a type parameter must be either \a Event_FocusIn or \a Event_FocusOut.
 */
 
 /*!
@@ -377,6 +403,7 @@ void QEvent::peErrMsg()				// posted event error message
 
   \ingroup event
 
+
   Paint events are sent to widgets that need to update themselves, for instance
   when a part of a widget is exposed because an overlying widget is moved away.
 
@@ -402,6 +429,7 @@ void QEvent::peErrMsg()				// posted event error message
 
   \ingroup event
 
+
   Move events are sent to widgets that have been moved to a new position
   relative to their parent.
 
@@ -433,6 +461,7 @@ void QEvent::peErrMsg()				// posted event error message
 
   \ingroup event
 
+
   Resize events are sent to widgets that have been resized.
 
   The event handler QWidget::resizeEvent() receives resize events.
@@ -463,6 +492,7 @@ void QEvent::peErrMsg()				// posted event error message
 
   \ingroup event
 
+
   Close events are sent to widgets that the user wants to close, usually
   by choosing "Close" from the window menu. They are also sent when you
   call QWidget::close() to close a widget from inside the program.
@@ -503,6 +533,11 @@ void QEvent::peErrMsg()				// posted event error message
   it in the close event. In this case, calling QCloseEvent::accept() or
   QCloseEvent::ignore() makes no difference.
 
+  \warning Be careful.  The code below assumes that the widget was created
+  on the heap using the \c new operator. Even when the widget has been
+  created by new doing this is a tricky opreation. Be sure that you cannot
+  have any other pointers to the widget hanging around.
+
   \code
     void MyWidget::closeEvent( QCloseEvent * )
     {
@@ -510,8 +545,9 @@ void QEvent::peErrMsg()				// posted event error message
     }
   \endcode
 
-  QObject emits the \link QObject::destroyed()\endlink signal when it is
-  deleted.
+  QObject emits the \link QObject::destroyed() destroyed()\endlink signal
+  when it is deleted.  This is a useful signal if a widget needs to know
+  when another widget is deleted.
 
   \sa QWidget::close(), QWidget::hide(), QObject::destroyed(),
   QApplication::setMainWidget(), QApplication::quit()
@@ -556,12 +592,59 @@ void QEvent::peErrMsg()				// posted event error message
 */
 
 
+/*!
+  \class QChildEvent qevent.h
+  \brief The QChildEvent class contains event parameters for child widget
+  events.
+
+  \ingroup event
+
+
+  Child events are sent to widgets when children are inserted or removed.
+
+  In this release of Qt, no event handlers are defined to receive
+  child events. Reimplement QObject::event() or install an
+  QObject::eventFilter() to handle these events.
+
+*/
+
+#if QT_VERSION >= 200
+#error "Implement child event handlers"
+#endif
+
+/*!
+  \fn QChildEvent::QChildEvent( int type, QWidget *child )
+  Constructs a child event object.
+
+  The \a type parameter must be either \a Event_ChildInserted
+  or \a Event_ChildRemoved.
+*/
+
+/*!
+  \fn QWidget *QChildEvent::child() const
+  Returns the child widget inserted or removed.
+*/
+
+/*!
+  \fn bool QChildEvent::inserted() const
+  Returns TRUE if the widget received a new child.
+*/
+
+/*!
+  \fn bool QChildEvent::removed() const
+  Returns TRUE if the widget lost a child.
+*/
+
+
+
+
 /*!
   \class QCustomEvent qevent.h
   \brief The QCustomEvent class provides support for custom events.
 
   \ingroup event
 
+
   QCustomEvent is a user-defined event type which contains a \c void*.
 
   \warning
@@ -572,8 +655,8 @@ void QEvent::peErrMsg()				// posted event error message
 
 /*!
   \fn QCustomEvent::QCustomEvent( int type, void *data )
-  Constructs a custom event object with the event type \e type and a
-  pointer to \e data.
+  Constructs a custom event object with the event type \a type and a
+  pointer to \a data.
 */
 
 /*!
