@@ -1,38 +1,37 @@
 /****************************************************************************
-** $Id: qfont.cpp,v 2.17 1997/08/06 15:05:36 hanord Exp $
+** $Id: qfont.cpp,v 2.37 1998/07/03 00:09:33 hanord Exp $
 **
 ** Implementation of QFont, QFontMetrics and QFontInfo classes
 **
 ** Created : 941207
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qfont.h"
-#include "qfontdta.h"
-#include "qfontmet.h"
-#include "qfontinf.h"
+#include "qfontdata.h"
+#include "qfontmetrics.h"
+#include "qfontinfo.h"
 #include "qwidget.h"
 #include "qpainter.h"
 #include "qdict.h"
 #include "qstrlist.h"
-#include "qdstream.h"
+#include "qdatastream.h"
 #include <ctype.h>
-
-RCSTAG("$Id: qfont.cpp,v 2.17 1997/08/06 15:05:36 hanord Exp $");
-
+#include <limits.h>
 
 /*!
   \class QFont qfont.h
@@ -42,60 +41,96 @@ RCSTAG("$Id: qfont.cpp,v 2.17 1997/08/06 15:05:36 hanord Exp $");
   \ingroup fonts
   \ingroup drawing
 
-  A QFont has a series of attributes that can be set to specify an abstract
-  font. When actual drawing of text is done Qt will select a font in the
-  underlying window system that \link fontmatch.html matches\endlink
-  the abstract font as close as possible. The most important attributes
-  are
-  \link setFamily() family\endlink,
-  \link setPointSize() point size\endlink,
-  \link setWeight() weight\endlink and
-  \link setItalic() italic\endlink.
+  QFont, more precisely, is a collection of attributes of a font.
+  When Qt needs to draw text, it will look up and load the \link
+  fontmatch.html closest matching installed font \endlink and draw
+  using that.
 
-  One of the QFont constructors take exactly these attributes as
-  arguments:
+  The most important attributes of a QFont are \link setFamily()
+  family, \endlink \link setPointSize() point size, \endlink \link
+  setWeight() weight \endlink and \link setItalic() italic. \endlink
+  There are QFont constructors that take these attributes as
+  arguments, as shown in this example:
 
   \code
     void MyWidget::paintEvent( QPaintEvent * )
     {
-	QPainter p;
-	p.begin( this );
-					// times, 12pt, normal
+	QPainter p( this );
+
+	// times, 12pt, normal
 	p.setFont( QFont( "times" ) );
 	p.drawText( 10, 20, "Text1" );
-					// helvetica, 18pt, normal
+
+	// helvetica, 18pt, normal
 	p.setFont( QFont( "helvetica", 18 ) );
 	p.drawText( 10, 120, "Text2" );
-					// courier, 24pt, bold
+
+	// courier, 24pt, bold
 	p.setFont( QFont( "courier", 24, QFont::Bold ) );
 	p.drawText( 10, 220, "Text3" );
-					// lucida, 36pt, bold, italic
+
+	// lucida, 36pt, bold, italic
 	p.setFont( QFont( "lucida", 36, QFont::Bold, TRUE ) );
 	p.drawText( 10, 320, "Text4" );
-
-	p.end();
     }
   \endcode
 
-  In general font handling and loading are costly operations that put
-  a heavy load on the window system, this is especially true for
-  X11. The QFont class has an internal sharing and reference
-  count mechanism in order to speed up copies so QFonts may be passed
-  around as arguments, and it has a lazy loading mechanism and does
-  not load a font until it \e really has to. It also caches previously
-  loaded fonts and under X it caches previously matched font attribute
-  combinations.
+  The default QFont constructor makes a copy of the \link
+  setDefaultFont() default font. \endlink
 
-  Note that the functions returning attribute values in QFont return
-  the values previously set, \e not the attributes of the actual
-  window system font used for drawing. To get information about the
-  actual font use QFontInfo.
+  You can also change these attributes of an existing QFont object
+  using functions such as setFamily(), setPointSize(), setWeight() and
+  setItalic().
 
-  To get font size information use the class QFontMetrics.
+  There are also some less-used attributes.  setUnderline() decides
+  whether the font is underlined or not; setStrikeOut() can be used to
+  get overstrike (a horizontal line through the middle of the
+  characters); setFixedPitch() determines whether Qt should give
+  preference to fixed-pitch or variable-pitch fonts when it needs to
+  choose an installed font; setStyleHint() can be used to offer more
+  general help to the font matching algorithm, and on X11 setRawMode()
+  can be used to bypass the entire font matching and use an X11 XLFD.
 
-  QFont objects make use of implicit sharing.
+  Of course there is also a reader function for each of these set*()
+  functions.  Note that the reader functions return the values
+  previously set, \e not the attributes of the actual window system
+  font that will be used for drawing.  You can get information about
+  the font that will be used for drawing by using QFontInfo, but be
+  aware that QFontInfo may be slow and that its results depend on what
+  fonts are installed.
 
-  \sa QApplication::setFont(), QWidget::setFont(), QPainter::setFont()
+  In general font handling and loading are costly operations,
+  especially on X11.  The QFont class contains extensive optimizations
+  to make copying of QFont objects fast, and to cache the results of
+  the slow window system functions it uses.
+
+  QFont also offers a few static functions, mostly to tune the \link
+  fontmatch.html font matching algorithm: \endlink You can control
+  what happens if a font's family isn't installed using
+  insertSubstitution() and removeSubstitution(), ask what happens for
+  a single family uing substitute() and you can get a complete list of
+  the fallback families using listSubstitutions().
+
+  cacheStatistics() offers cache effectiveness information; this is
+  useful mostly for debugging.
+
+  Finally, setDefaultFont() allows you to set the default font.  The
+  default default font is chosen at application startup from a set of
+  common installed fonts that support the correct \link QFont::CharSet
+  character set. \endlink Of course, the initialization algorithm has
+  a default, too: The default default default font!
+
+  For more general information on fonts, see the
+  <a href="http://www.oreilly.com/homepages/comp.fonts/ifa/">Internet Font
+  Archive</a> and in particular its extensive FAQs: <ul>
+  <li> <a href="http://www.oreilly.com/homepages/comp.fonts/FAQ/">Questions
+  about fonts in general</a> (from <a href="news:comp.fonts">comp.fonts</a>)
+  <li> <a href="http://www.oreilly.com/homepages/comp.fonts/ifa/faq.htm">
+  Questions about the IFA and fonts on the Internet.</a></ul>
+
+  \sa QFontMetrics QFontInfo QApplication::setFont()
+  QWidget::setFont() QPainter::setFont() QFont::StyleHint
+  QFont::CharSet QFont::Weight
 */
 
 
@@ -110,7 +145,7 @@ void QFont::init()
     CHECK_PTR( d );
     d->req.pointSize	 = 0;
     d->req.styleHint	 = AnyStyle;
-    d->req.charSet	 = Latin1;
+    d->req.charSet	 = defFont?(CharSet)(defFont->d->req.charSet):Latin1;
     d->req.weight	 = 0;
     d->req.italic	 = FALSE;
     d->req.underline	 = FALSE;
@@ -119,13 +154,15 @@ void QFont::init()
     d->req.hintSetByUser = FALSE;
     d->req.rawMode	 = FALSE;
     d->req.dirty	 = TRUE;
+    d->req.lbearing	 = SHRT_MIN;
+    d->req.rbearing	 = SHRT_MIN;
     d->exactMatch	 = FALSE;
 }
 
 /*!
   \internal
-  Constructs a font that gets a 
-  \link shclass.html deep copy\endlink of \e data.
+  Constructs a font that gets a
+  \link shclass.html deep copy \endlink of \e data.
 */
 
 QFont::QFont( QFontData *data )
@@ -161,8 +198,11 @@ QFont::QFont()
 
 /*!
   Constructs a font object with the specified \e family, \e pointSize,
-  \e weight and \e italic settings. If \e pointSize is less than or
-  equal to 0 it is set to 1.
+  \e weight and \e italic settings.  The \link charSet() character set
+  \endlink is copied from the \link defaultFont() default font
+  \endlink and the rest of the settings ser set reasonably.
+
+  If \e pointSize is less than or equal to 0 it is set to 1.
 
   \sa setFamily(), setPointSize(), setWeight(), setItalic()
 */
@@ -242,8 +282,8 @@ QFont &QFont::operator=( const QFont &font )
 /*!
   Returns the family name set by setFamily().
 
-  Use QFontInfo to find the family name of the window system font actually
-  used.
+  Use QFontInfo to find the family name of the window system font that
+  is actually used for drawing.
 
   Example:
   \code
@@ -265,7 +305,7 @@ const char *QFont::family() const
 
   The family name is case insensitive.
 
-  If the family is not available a default family will be used instead.
+  If the family is not available a default family is used.
 
   \sa family(), setStyleHint(), QFontInfo,
   \link fontmatch.html font matching\endlink
@@ -295,7 +335,8 @@ int QFont::deciPointSize() const
 /*!
   Returns the point size set by setPointSize().
 
-  Use QFontInfo to find the point size of the window system font actually used.
+  Use QFontInfo to find the point size of the window system font
+  actually used.
 
   Example of use:
   \code
@@ -306,7 +347,7 @@ int QFont::deciPointSize() const
     debug( "Font size actually used is: %d", info.pointSize() );
   \endcode
 
-  \sa setPointSize()
+  \sa setPointSize() deciPointSize()
 */
 
 int QFont::pointSize() const
@@ -317,15 +358,13 @@ int QFont::pointSize() const
 /*!
   Sets the point size to \a pointSize. The point size must be greater
   than zero.
-  
+
   Example:
   \code
     QFont font( "courier" );
     font.setPointSize( 18 );
   \endcode
 
-  If the point size is not available the closest available will be used.
-
   \sa pointSize(), QFontInfo, \link fontmatch.html font matching\endlink
 */
 
@@ -363,8 +402,6 @@ bool QFont::italic() const
 /*!
   Sets italic on or off.
 
-  If the mode selected is not available the other will be used.
-
   \sa italic(), QFontInfo, \link fontmatch.html font matching\endlink
 */
 
@@ -393,6 +430,8 @@ int QFont::weight() const
 /*!
   Sets the weight (or boldness).
 
+  \define QFont::Weight
+
   The enum \c QFont::Weight contains the predefined font weights:<ul>
   <li> \c QFont::Light (25)
   <li> \c QFont::Normal (50)
@@ -457,7 +496,7 @@ void QFont::setWeight( int weight )
   Returns the value set by setUnderline().
 
   Use QFontInfo to find the underline value of the window system font
-  actually used.
+  actually used for drawing.
 
   \sa setUnderline(), QFontInfo::underline()
 */
@@ -470,8 +509,6 @@ bool QFont::underline() const
 /*!
   Sets underline on or off.
 
-  If the mode selected is not available the other will be used.
-
   \sa underline(), QFontInfo, \link fontmatch.html font matching.\endlink
 */
 
@@ -502,8 +539,6 @@ bool QFont::strikeOut() const
 /*!
   Sets strike out on or off.
 
-  If the mode selected is not available the other will be used.
-
   \sa strikeOut(), QFontInfo, \link fontmatch.html font matching\endlink
 */
 
@@ -533,9 +568,8 @@ bool QFont::fixedPitch() const
 
 /*!
   Sets fixed pitch on or off.
-  
-  A fixed pitch font is a font that has constant character pixel width.  
-  If the mode selected is not available the other will be used.
+
+  A fixed pitch font is a font where all characters have the same width.
 
   \sa fixedPitch(), QFontInfo, \link fontmatch.html font matching\endlink
 */
@@ -564,20 +598,23 @@ QFont::StyleHint QFont::styleHint() const
 /*!
   Sets the style hint.
 
-  The style hint is used by the \link fontmatch.html font matching\endlink
-  algorithm when a selected font family cannot be found and is used to
-  find an appropriate default family.
+  \define QFont::StyleHint
 
-  The style hint has a default value of \c AnyStyle which leaves the task of
-  finding a good default family to the font matching algorithm.
+  The style hint is used by the \link fontmatch.html font
+  matching algorithm \endlink when a selected font family cannot be
+  found and is used to find an appropriate default family.
 
-  In this example (which is a complete program) the push button
-  will display its text label with the Bavaria font family if this family
-  is available, if not it will display its text label with the Times font
-  family:
+  The style hint has a default value of \c AnyStyle which leaves the
+  task of finding a good default family to the font matching
+  algorithm.
+
+  In this example (which is a complete program) the push button will
+  display its text label with the Bavaria font family if this family
+  is available, if not it will display its text label with another
+  serif font:
   \code
-    #include <qapp.h>
-    #include <qpushbt.h>
+    #include <qapplication.h>
+    #include <qpushbutton.h>
     #include <qfont.h>
 
     int main( int argc, char **argv )
@@ -586,13 +623,16 @@ QFont::StyleHint QFont::styleHint() const
 	QPushButton  push("Push me");
 
 	QFont font( "Bavaria", 18 );	    // preferrred family is Bavaria
-	font.setStyleHint( QFont::Times );  // use Times if Bavaria isn't here
+	font.setStyleHint( QFont::Serif );  // can also use any serif font
 
 	push.setFont( font );
 	return app.exec( &push );
     }
   \endcode
 
+  The other available style hints are \c QFont::SansSerif, \c
+  QFont::TypeWriter, \c QFont::OldEnglish, \c QFont::System
+
   \sa styleHint(), QFontInfo, \link fontmatch.html font matching\endlink
 */
 
@@ -620,15 +660,18 @@ QFont::CharSet QFont::charSet() const
 }
 
 /*!
-  Sets the character set (e.g. \c Latin1).
+  Sets the character set encoding (e.g. \c Latin1).
 
-  If the character set is not available another will be used, for most
-  non-trivial applications you will probably not want this to happen since
-  it can totally obscure the text shown to the user when the font is
-  used. This is why the \link fontmatch.html font matching\endlink
-  algorithm gives high priority to finding the correct character set.
+  \define QFont::CharSet
 
-  To ensure that the character set is correct you can use the QFontInfo
+  If the character set encoding is not available another will be used
+  for drawing.  for most non-trivial applications you will probably
+  not want this to happen since it can totally obscure the text shown
+  to the user.  This is why the \link fontmatch.html font
+  matching algorithm \endlink gives high priority to finding the
+  correct character set.
+
+  You can test that the character set is correct using the QFontInfo
   class.
 
   Example:
@@ -639,6 +682,21 @@ QFont::CharSet QFont::charSet() const
 	fatal( "Cannot find a Latin 1 Times font" );
   \endcode
 
+  In Qt 1.40, the following character set encodings are available: <ul>
+  <li> \c QFont::Latin1 - ISO 8859-1, common in much of Europe
+  <li> \c QFont::Latin2 - ISO 8859-2, less common European character set
+  <li> \c QFont::Latin3 - ISO 8859-3, less common European character set
+  <li> \c QFont::Latin4 - ISO 8859-4, less common European character set
+  <li> \c QFont::Latin5 - ISO 8859-5, Cyrillic
+  <li> \c QFont::Latin6 - ISO 8859-6, Arabic
+  <li> \c QFont::Latin7 - ISO 8859-7, Greek
+  <li> \c QFont::Latin8 - ISO 8859-8, Hebrew
+  <li> \c QFont::Latin9 - ISO 8859-9, less common European character set
+  <li> \c QFont::KOI8R - KOI8-R, Cyrillic, defined in
+       <a href="http://ds.internic.net/rfc/rfc1489.txt">RFC 1489.</a>
+  <li> \c QFont::AnyCharSet - whatever is handiest.
+  </ul>
+
   \sa charSet(), QFontInfo, \link fontmatch.html font matching\endlink
 */
 
@@ -776,9 +834,17 @@ typedef Q_DECLARE(QDictM,char)	       QFontSubst;
 typedef Q_DECLARE(QDictIteratorM,char) QFontSubstIt;
 static QFontSubst *fontSubst = 0;
 
-static void cleanupFontSubst()			// cleanup substitution dict
+#if (QT_VERSION == 200)
+#error "fixme"
+#endif
+static QFont * staticFont = 0;
+
+static void cleanup()
 {
     delete fontSubst;
+    fontSubst = 0;
+    delete staticFont;
+    staticFont = 0;
 }
 
 static void initFontSubst()			// create substitution dict
@@ -800,7 +866,7 @@ static void initFontSubst()			// create substitution dict
     fontSubst->setAutoDelete( TRUE );
     for ( int i=0; initTbl[i] != 0; i += 2 )
 	fontSubst->insert( initTbl[i],	qstrdup(initTbl[i+1]) );
-    qAddPostRoutine( cleanupFontSubst );
+    qAddPostRoutine( cleanup );
 }
 
 
@@ -1095,7 +1161,7 @@ void QFontMetrics::reset( const QPainter *p )
 
 
 /*!
-  \class QFontMetrics qfontmet.h
+  \class QFontMetrics qfontmetrics.h
   \brief The QFontMetrics class provides font metrics information about fonts.
 
   \ingroup fonts
@@ -1126,7 +1192,7 @@ void QFontMetrics::reset( const QPainter *p )
     int h = fm.height();
   \endcode
 
-  <strong>In Qt 2.0 the font metrics object will no longer be
+  <strong>NOTE: In Qt 2.0 the font metrics object will no longer be
   automatically updated when the widget or painter gets a new font.</strong>
   Make sure you program does not depend on this feature.
 
@@ -1241,19 +1307,6 @@ QFontMetrics &QFontMetrics::operator=( const QFontMetrics &fm )
 }
 
 
-/*!
-  Returns the pixel width of a \e ch.
-  \sa boundingRect()
-*/
-
-int QFontMetrics::width( char ch ) const
-{
-    char tmp[2];
-    tmp[1] = '\0';
-    tmp[0] = ch;
-    return width( tmp, 1 );
-}
-
 /*!
   Returns the bounding rectangle of \e ch relative to the leftmost
   point on the base line.
@@ -1297,8 +1350,10 @@ const QFont &QFontMetrics::font() const
 		 "future version of Qt" );
 #endif
     }
-    static QFont f;
-    return f;
+    // ### uglehack wrt naming
+    if ( !staticFont )
+	initFontSubst();
+    return *staticFont;
 }
 
 
@@ -1369,7 +1424,7 @@ void QFontInfo::reset( const QPainter *p )
 
 
 /*!
-  \class QFontInfo qfontinf.h
+  \class QFontInfo qfontinfo.h
 
   \brief The QFontInfo class provides general information about fonts.
 
@@ -1621,7 +1676,7 @@ bool QFontInfo::strikeOut() const
 
 /*!
   Returns the fixed pitch value of the matched window system font.
-  A fixed pitch font is a font that has constant character pixel width.  
+  A fixed pitch font is a font that has constant character pixel width.
   \sa QFont::fixedPitch()
 */
 
@@ -1714,6 +1769,136 @@ const QFont &QFontInfo::font() const
 		 "future version of Qt" );
 #endif
     }
-    static QFont f;
-    return f;
+    // ### uglehack wrt naming
+    if ( !staticFont )
+	initFontSubst();
+    return *staticFont;
+}
+
+// From qpainter.cpp
+void qt_format_text( const QFontMetrics& fm, int x, int y, int w, int h,
+		     int tf, const char *str, int len, QRect *brect,
+		     int tabstops, int* tabarray, int tabarraylen,
+		     char **internal, QPainter* painter );
+
+/*!
+  Returns the bounding rectangle of the first \e len characters of \e str,
+  which is the set of pixels the text would cover if drawn at (0,0).
+
+  If \a len is negative (default value), the whole string is used.
+
+  The \a flags argument is
+  the bitwise OR of the following flags:  <ul>
+  <li> \c AlignLeft aligns to the left border.
+  <li> \c AlignRight aligns to the right border.
+  <li> \c AlignHCenter aligns horizontally centered.
+  <li> \c AlignTop aligns to the top border.
+  <li> \c AlignBottom aligns to the bottom border.
+  <li> \c AlignVCenter aligns vertically centered
+  <li> \c AlignCenter (= \c AlignHCenter | AlignVCenter)
+  <li> \c SingleLine ignores newline characters in the text.
+  <li> \c ExpandTabs expands tabulators.
+  <li> \c ShowPrefix interprets "&x" as "x" underlined.
+  <li> \c WordBreak breaks the text to fit the rectangle.
+  </ul>
+
+  Horizontal alignment defaults to AlignLeft and vertical alignment
+  defaults to AlignTop.
+
+  If several of the horizontal or several of the vertical alignment flags
+  are set, the resulting alignment is undefined.
+
+  These flags are defined in qwindowdefs.h.
+
+  If \c ExpandTabs is set in \a flags, then:
+  if \a tabarray is non.zero, it specifies a 0-terminated sequence
+  of pixel-positions for tabs; otherwise
+  if \a tabstops is non-zero, it is used as the tab spacing (in pixels).
+
+  Note that the bounding rectangle may extend to the left of (0,0),
+  e.g. for italicized fonts, and that the text output may cover \e all
+  pixels in the bounding rectangle.
+
+  Newline characters are processed as linebreaks.
+
+  Despite the different actual character heights, the heights of the
+  bounding rectangles of "Yes" and "yes" are the same.
+
+  The bounding rectangle given by this function is somewhat larger
+  than that calculated by the simpler boundingRect() function.  This
+  function uses the \link minLeftBearing() maximum left \endlink and
+  \link minRightBearing() right \endlink font bearings as is necessary
+  for multi-line text to align correctly.  Also, fontHeight() and
+  lineSpacing() are used to calculate the height, rather than
+  individual character heights.
+
+  The \a internal argument is for internal purposes.
+
+  \sa width(), QPainter::boundingRect() */
+
+QRect QFontMetrics::boundingRect( int x, int y, int w, int h, int flags,
+				  const char *str, int len, int tabstops,
+				  int *tabarray, char **intern ) const
+{
+    if ( len < 0 )
+	len = strlen( str );
+
+    int tabarraylen=0;
+    if (tabarray)
+	while (tabarray[tabarraylen])
+	    tabarraylen++;
+
+    QRect r;
+    qt_format_text( *this, x, y, w, h, flags, str, len, &r,
+		    tabstops, tabarray, tabarraylen, intern, 0 );
+
+    return r;
+}
+
+/*!
+  Returns the size in pixels of the first \e len characters of \e str.
+
+  If \a len is negative (default value), the whole string is used.
+
+  The \a flags argument is
+  the bitwise OR of the following flags:  <ul>
+  <li> \c SingleLine ignores newline characters in the text.
+  <li> \c ExpandTabs expands tabulators.
+  <li> \c ShowPrefix interprets "&x" as "x" underlined.
+  <li> \c WordBreak breaks the text to fit the rectangle.
+  </ul>
+
+  These flags are defined in qwindowdefs.h.
+
+  If \c ExpandTabs is set in \a flags, then:
+  if \a tabarray is non.zero, it specifies a 0-terminated sequence
+  of pixel-positions for tabs; otherwise
+  if \a tabstops is non-zero, it is used as the tab spacing (in pixels).
+
+  Newline characters are processed as linebreaks.
+
+  Despite the different actual character heights, the heights of the
+  bounding rectangles of "Yes" and "yes" are the same.
+
+  The \a internal argument is for internal purposes.
+
+  \sa boundingRect() */
+
+QSize QFontMetrics::size( int flags, const char *str, int len, int tabstops,
+			  int *tabarray, char **intern ) const
+{
+    return boundingRect(0,0,1,1,flags,str,len,tabstops,tabarray,intern).size();
+}
+
+
+/*!  Returns TRUE if this font and \a f are copies of each other,
+  ie. one of them was created as a copy of the other and neither was
+  subsequently modified.  This is much stricter than equality.
+
+  \sa operator=, operator==
+*/
+
+bool QFont::isCopyOf( const QFont & f ) const
+{
+    return d && d == f.d;
 }
