@@ -1,29 +1,32 @@
 /****************************************************************************
-** $Id: qkeycode.h,v 2.4 1997/09/22 21:07:53 agulbra Exp $
+** $Id: qkeycode.h,v 2.7 1998/07/03 00:09:34 hanord Exp $
 **
 ** Definition of keyboard codes
 **
 ** Created : 931030
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QKEYCODE_H
 #define QKEYCODE_H
 
+#ifndef QT_H
 #include "qglobal.h"
+#endif // QT_H
 
 
 const uint SHIFT	= 0x00002000;		// accelerator modifiers
@@ -61,7 +64,7 @@ const uint ASCII_ACCEL	= 0x10000000;
 #define Key_Alt			0x1023
 #define Key_CapsLock		0x1024
 #define Key_NumLock		0x1025
-#define Key_ScrollLock		0x1026	// see translateKeyEvent()
+#define Key_ScrollLock		0x1026		// see translateKeyEvent()
 
 #define Key_F1			0x1030		// function keys
 #define Key_F2			0x1031
@@ -87,6 +90,17 @@ const uint ASCII_ACCEL	= 0x10000000;
 #define Key_F22			0x1045
 #define Key_F23			0x1046
 #define Key_F24			0x1047
+#define Key_F25			0x1048		// F25 .. F35 only on X11
+#define Key_F26			0x1049
+#define Key_F27			0x104a
+#define Key_F28			0x104b
+#define Key_F29			0x104c
+#define Key_F30			0x104d
+#define Key_F31			0x104e
+#define Key_F32			0x104f
+#define Key_F33			0x1050
+#define Key_F34			0x1051
+#define Key_F35			0x1052
 
 #define Key_Space		0x20		// 7 bit printable ASCII
 #define Key_Exclam		0x21
