@@ -1,22 +1,23 @@
 /****************************************************************************
-** $Id: qmovie.cpp,v 1.26 1997/09/30 11:18:37 hanord Exp $
+** $Id: qmovie.cpp,v 1.39 1998/07/03 00:09:35 hanord Exp $
 **
 ** Implementation of movie classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
@@ -38,6 +39,8 @@
   \class QMovie qmovie.h
   \brief Incrementally loads an animation or image, signalling as it progresses.
 
+  \ingroup images
+
   A QMovie provides a QPixmap as the framePixmap(), and connections can
   be made via connectResize() and connectUpdate() to receive notification
   of size and pixmap changes.  All decoding is driven by
@@ -64,11 +67,25 @@
   disposal, local color maps, and background colors.  The Netscape looping
   extension is obeyed.
 
+  Archives of animated GIFs and tools for building them can be found
+  at <a href=http://www.yahoo.com/Arts/Visual_Arts/Animation/Computer_Animation/Animated_GIFs/>
+  Yahoo!</a>.
+
   We are required to state: The Graphics Interchange Format(c) is the
   Copyright property of CompuServe Incorporated. GIF(sm) is a Service
   Mark property of CompuServe Incorporated.
 
-  <img src=qmovie.gif>
+  \warning Unisys has changed its position regarding GIF.  If you are
+  in a country where Unisys holds a patent on LZW compression and/or
+  decompression, Unisys may require a license from you.  These
+  countries include Canada, Japan, the USA, France, Germany, Italy and
+  the UK.  There is more information on Unisys web site: <a
+  href="http://corp2.unisys.com/LeadStory/lzwfaq.html">Overview of
+  Unisys' position.</a> GIF support may be removed in a future version
+  of Qt.  We recommend using the PNG format, which is available in the
+  <a href="imageio.html">Qt Image IO Extension</a> package.
+
+  <img src="qmovie.gif">
 
   \sa QLabel::setMovie()
 */
@@ -97,13 +114,13 @@ public: // for QMovie
     void showChanges();
 
     // This as QImageConsumer
-    bool changed(const QRect& rect);
-    bool end();
-    bool frameDone();
+    void changed(const QRect& rect);
+    void end();
+    void frameDone();
     void restartTimer();
-    bool setLooping(int l);
-    bool setFramePeriod(int milliseconds);
-    bool setSize(int w, int h);
+    void setLooping(int l);
+    void setFramePeriod(int milliseconds);
+    void setSize(int w, int h);
 
     // This as QDataSink
     int readyToReceive();
@@ -176,7 +193,8 @@ QMoviePrivate::QMoviePrivate(QDataSource* src, QMovie* movie, int bufsize) :
 
 QMoviePrivate::~QMoviePrivate()
 {
-    delete buffer;
+    if ( buffer )				// Avoid purify complaint
+	delete [] buffer;
     delete pump;
     delete decoder;
     delete source;
@@ -192,12 +210,18 @@ bool QMoviePrivate::isNull() const
 void QMoviePrivate::init(bool fully)
 {
     buf_usage = buf_r = buf_w = 0;
-    delete buffer;
+    if ( buffer )				// Avoid purify complaint
+	delete [] buffer;
     buffer = fully ? new uchar[buf_size] : 0;
 
     delete decoder;
     decoder = fully ? new QImageDecoder(this) : 0;
 
+#ifdef AVOID_OPEN_FDS
+    if ( source && !source->isOpen() )
+	source->open(IO_ReadOnly);
+#endif
+
     waitingForFrameTick = FALSE;
     stepping = -1;
     framenumber = 0;
@@ -294,19 +318,17 @@ void QMoviePrivate::showChanges()
 }
 
 // QMoviePrivate as QImageConsumer
-bool QMoviePrivate::changed(const QRect& rect)
+void QMoviePrivate::changed(const QRect& rect)
 {
     if (!frametimer->isActive()) frametimer->start(0);
     changed_area = changed_area.unite(rect);
-    return TRUE;
 }
 
-bool QMoviePrivate::end()
+void QMoviePrivate::end()
 {
-    return TRUE; // Not that it will do anything.
 }
 
-bool QMoviePrivate::frameDone()
+void QMoviePrivate::frameDone()
 {
     if (stepping > 0) {
 	stepping--;
@@ -321,7 +343,6 @@ bool QMoviePrivate::frameDone()
     showChanges();
     emit dataStatus(QMovie::EndOfFrame);
     framenumber++;
-    return FALSE;
 }
 
 void QMoviePrivate::restartTimer()
@@ -337,7 +358,7 @@ void QMoviePrivate::restartTimer()
     }
 }
 
-bool QMoviePrivate::setLooping(int nloops)
+void QMoviePrivate::setLooping(int nloops)
 {
     if (loop == -1) { // Only if we don't already know how many loops!
 	if (source->rewindable()) {
@@ -348,25 +369,21 @@ bool QMoviePrivate::setLooping(int nloops)
 	    loop = -2;
 	}
     }
-
-    return TRUE;
 }
 
-bool QMoviePrivate::setFramePeriod(int milliseconds)
+void QMoviePrivate::setFramePeriod(int milliseconds)
 {
     // Animation:  only show complete frame
     frameperiod = milliseconds;
     if (stepping<0 && frameperiod >= 0) restartTimer();
-    return TRUE;
 }
 
-bool QMoviePrivate::setSize(int w, int h)
+void QMoviePrivate::setSize(int w, int h)
 {
     if (mypixmap.width() != w || mypixmap.height() != h) {
 	mypixmap.resize(w, h);
 	emit sizeChanged(QSize(w, h));
     }
-    return TRUE;
 }
 
 
@@ -374,6 +391,8 @@ bool QMoviePrivate::setSize(int w, int h)
 
 int QMoviePrivate::readyToReceive()
 {
+    // Could pre-fill buffer, but more efficient to just leave the
+    // data back at the source.
     return (waitingForFrameTick || !stepping || buf_usage || error)
 	? 0 : buf_size;
 }
@@ -420,9 +439,14 @@ void QMoviePrivate::eof()
     } else {
 	delete decoder;
 	decoder = 0;
-	delete buffer;
+	if ( buffer )				// Avoid purify complaint
+	    delete [] buffer;
 	buffer = 0;
 	emit dataStatus(QMovie::EndOfMovie);
+#ifdef AVOID_OPEN_FDS
+	if ( source )
+	    source->close();
+#endif
     }
 }
 
@@ -437,6 +461,11 @@ void QMoviePrivate::pause()
 
 void QMoviePrivate::refresh()
 {
+    if (!decoder) {
+	frametimer->stop();
+	return;
+    }
+
     if (frameperiod < 0 && loop == -1) {
 	// Only show changes if probably not an animation
 	showChanges();
@@ -467,12 +496,9 @@ QMovie::QMovie()
     d = new QMoviePrivate();
 }
 
-#if 0
-// Removed - QDataSource should be the interface.  QIODevice is not useful,
-// as it doesn't emit signals upon data availability.
-/*
+/*!
   Creates a QMovie which reads an image sequence from the given
-  QIODevice.  The device must be allocated dynamically,
+  QDataSource.  The source must be allocated dynamically,
   as it becomes owned by the QMovie, and will be destroyed
   when the movie is destroyed.
   The movie starts playing as soon as event processing continues.
@@ -482,11 +508,10 @@ QMovie::QMovie()
   value, the better interleaved the movie playback will be with other
   event processing, but the slower the overall processing.
 */
-QMovie::QMovie(QIODevice* src, int bufsize)
+QMovie::QMovie(QDataSource* src, int bufsize)
 {
-    d = new QMoviePrivate(new QIODeviceSource(src), this, bufsize);
+    d = new QMoviePrivate(src, this, bufsize);
 }
-#endif
 
 /*!
   \overload
@@ -827,7 +852,7 @@ void QMovie::disconnectStatus(QObject* receiver, const char* member)
 ** QMoviePrivate meta object code from reading C++ file 'qmovie.cpp'
 **
 ** Created: Thu Sep 4 15:31:20 1997
-**      by: The Qt Meta Object Compiler ($Revision: 1.26 $)
+**      by: The Qt Meta Object Compiler ($Revision: 1.39 $)
 **
 ** WARNING! All changes made in this file will be lost!
 *****************************************************************************/
@@ -838,7 +863,7 @@ void QMovie::disconnectStatus(QObject* receiver, const char* member)
 #error Moc format conflict - please regenerate all moc files
 #endif
 
-#include <qmetaobj.h>
+#include <qmetaobject.h>
 
 
 const char *QMoviePrivate::className() const
