@@ -1,35 +1,42 @@
 /****************************************************************************
-** $Id: qobject.cpp,v 2.20 1997/04/04 16:30:35 hanord Exp $
+** $Id: qobject.cpp,v 2.52 1998/07/03 00:09:35 hanord Exp $
 **
 ** Implementation of QObject class
 **
 ** Created : 930418
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qobject.h"
-#include "qobjcoll.h"
+#include "qobjectlist.h"
+#include "qobjectdict.h"
 #include "qregexp.h"
 #include <ctype.h>
-
-RCSTAG("$Id: qobject.cpp,v 2.20 1997/04/04 16:30:35 hanord Exp $");
-
+#if defined(DEBUG)
+#include "qapplication.h"
+#endif
 
 /*!
   \class QObject qobject.h
   \brief The QObject class is the base class of all Qt objects that can
   deal with signals, slots and events.
 
+  \ingroup kernel
+
   Qt provides a very powerful mechanism for seamless object
   communication; \link metaobjects.html signal/slot
   connections\endlink. The signal/slot mechanism is an advanced way
@@ -120,10 +127,9 @@ RCSTAG("$Id: qobject.cpp,v 2.20 1997/04/04 16:30:35 hanord Exp $");
   the object that emits a signal does not need to know what the
   signals are connected to.
 
-  All Qt widgets inherit QObject and they make good use of signals and
-  slots.  A QScrollBar, for example, emits \link
-  QScrollBar::valueChanged() valueChanged()\endlink whenever the
-  scroll bar value changes.
+  All Qt widgets inherit QObject and use signals and slots.  A
+  QScrollBar, for example, emits \link QScrollBar::valueChanged()
+  valueChanged()\endlink whenever the scroll bar value changes.
 
   Meta objects are useful for doing more than connecting signals to slots.
   They also allow the programmer to obtain information about the class to
@@ -137,13 +143,21 @@ RCSTAG("$Id: qobject.cpp,v 2.20 1997/04/04 16:30:35 hanord Exp $");
   \class QSenderObject qobject.h
   \brief Internal object used for sending signals.
 
+  \internal
+
   It is generally a very bad idea to use this class directly in
   application programs.
 
-  In particular, you cannot not use it to send signals from classes that do
-  not inherit QObject.	If you wish to do that, make an internal class
-  that inherits QObject and has the necessary signals and slots.
-  Or use the QSignal class.
+  In particular, you cannot not use it to send signals from classes
+  that do not inherit QObject. If you wish to do that, make an
+  internal class that inherits QObject and has the necessary signals
+  and slots.  Alternatively, you can use the QSignal class.
+*/
+
+/*!
+  \fn void QSenderObject::setSender (QObject* s)
+
+  Internal function, used in signal-slot connections.
 */
 
 
@@ -153,8 +167,9 @@ RCSTAG("$Id: qobject.cpp,v 2.20 1997/04/04 16:30:35 hanord Exp $");
 //
 
 inline bool isIdentChar( char x )
-{
-    return isalnum(x) || x == '_';
+{						// Avoid bug in isalnum
+    return x == '_' || (x >= '0' && x <= '9') ||
+	 (x >= 'a' && x <= 'z') || (x >= 'A' && x <= 'Z');
 }
 
 inline bool isSpace( char x )
@@ -188,6 +203,9 @@ static QString rmWS( const char *src )
 	    last = *d++ = ' ';
     }
     result.truncate( (int)(d - result.data()) );
+    int void_pos = result.find("(void)");
+    if ( void_pos >= 0 )
+	result.remove( void_pos+1, strlen("void") );
     return result;
 }
 
@@ -201,8 +219,11 @@ bool  qKillTimer( QObject *obj );
 void  qRemovePostedEvents( QObject * );
 
 
+#if !defined(Q_MOC_CONNECTIONLIST_DECLARED)
+#define Q_MOC_CONNECTIONLIST_DECLARED
 Q_DECLARE(QListM,QConnection);			// list of connections
 Q_DECLARE(QListIteratorM,QConnection);		// connection list iterator
+#endif
 Q_DECLARE(QDictM,QListM_QConnection);		// dict for multiple signals
 Q_DECLARE(QDictIteratorM,QListM_QConnection);	// dict iterator
 
@@ -225,13 +246,28 @@ static void removeObjFromList( QObjectList *objList, const QObject *obj,
 
 
 /*!
-  Finds a child of a QObject.
+  \relates QObject
 
-  Use the CHILD macro instead.
+  Returns a pointer to the child named \a name of QObject \a parent
+  which inherits type \a type.
 
-  Example:
+  Returns 0 if there is no such child.
+
+  The CHILD macro (defined in qwindowdefs.h) does all this, and also
+  casts the returt type to \a type *.
+
+  Examples:
   \code
-    CHILD(myWidget,QListBox,"listboxname")->insertItem( "another string" );
+    QListBox * c = CHILD(myWidget,QListBox,"listboxname");
+    if ( c )
+        c->insertItem( "another string" );
+  \endcode
+
+  \code
+    QListBox * c = (QListBox *)::qt_find_obj_child(myWidget,QListBox,
+                                                   "listboxname");
+    if ( c )
+        c->insertItem( "another string" );
   \endcode
 */
 
@@ -242,20 +278,12 @@ void *qt_find_obj_child( QObject *parent, const char *type, const char *name )
 	QObjectListIt it( *list );
 	QObject *obj;
 	while ( (obj=it.current()) ) {
-	    if ( strcmp(name,obj->name()) == 0 ) {
-#if defined(CHECK_RANGE)
-		if ( !obj->inherits(type) )
-		    warning( "CHILD: Object %s does not inherit %s",
-			     name, type );
-#endif
-		return obj;
-	    }
 	    ++it;
+	    if ( qstrcmp(name,obj->name()) == 0 &&
+		obj->inherits(type) )
+		return obj;
 	}
     }
-#if defined(CHECK_NULL)
-    warning( "CHILD: No such child object %s", name );
-#endif
     return 0;
 }
 
@@ -387,11 +415,9 @@ QObject::~QObject()
   mechanism.  The functions isA() and inherits() also make use of the
   meta object.
 
-  The meta object is created by the initMetaObject() function, which is
-  generated by the meta object compiler and called from the class constructor.
-
-  \warning If this function returns 0, the constructor probably forgot
-  to call initMetaObject().
+  The meta object is created by the initMetaObject() function, which
+  is generated by the meta object compiler and called from the QObject
+  constructor.
 */
 
 /*!
@@ -401,8 +427,7 @@ QObject::~QObject()
   Compiler. \endlink
 
   \warning This function will return an invalid name if the class
-  constructor did not call initMetaObject() or if the class definition
-  lacks the \c Q_OBJECT macro.
+  definition lacks the \c Q_OBJECT macro.
 
   \sa name(), inherits(), isA(), isWidgetType()
 */
@@ -435,7 +460,9 @@ bool QObject::isA( const char *clname ) const
 
 /*!
   Returns TRUE if this object is an instance of a class that inherits
-  \e clname.  (A class is considered to inherit itself.)
+  \e clname, and \a clname inherits QObject.
+
+  (A class is considered to inherit itself.)
 
   Example:
   \code
@@ -443,6 +470,10 @@ bool QObject::isA( const char *clname ) const
     t->inherits("QTimer");		// returns TRUE
     t->inherits("QObject");		// returns TRUE
     t->inherits("QButton");		// returns FALSE
+
+    QScrollBar * s = new QScrollBar;	// inherits QWidget and QRangeControl
+    s->inherits( "QWidget" );		// returns TRUE
+    s->inherits( "QRangeControl" ); 	// returns FALSE
   \endcode
 
   \sa isA(), metaObject()
@@ -476,6 +507,23 @@ bool QObject::inherits( const char *clname ) const
   \sa setName(), className(), queryList()
 */
 
+/*!
+  \overload const char *QObject::name( const char * defaultName ) const
+
+  Returns the name of this object, or \a defaultName if the object
+  does not have a name.
+
+  This version of name() is particularly useful for debug() calls such as:
+
+  \code
+    debug( "MyClass::setPrecision(): (%s) unable to set precision to %f",
+	    name( "unnamed" ), newPrecision );
+  \endcode
+
+  name( "unnamed" ) is used here so that printf() (used in debug())
+  will not be asked to output a null pointer.
+*/
+
 /*!
   Sets the name of this object to \e name.  The default name is the
   one assigned by the constructor.
@@ -548,6 +596,9 @@ bool QObject::event( QEvent *e )
 
   The default implementation does nothing.
 
+  QTimer provides a higher-level interface to the timer functionality,
+  and also more general information about timers.
+
   \sa startTimer(), killTimer(), killTimers(), event()
 */
 
@@ -586,8 +637,9 @@ bool QObject::activate_filters( QEvent *e )
     register QObject *obj = it.current();
     while ( obj ) {				// send to all filters
 	++it;					//   until one returns TRUE
-	if ( obj->eventFilter(this,e) )
+	if ( obj->eventFilter(this,e) ) {
 	    return TRUE;
+	}
 	obj = it.current();
     }
     return FALSE;				// don't do anything with it
@@ -723,7 +775,7 @@ static void objSearch( QObjectList *result,
 	    ok = FALSE;
 	if ( ok ) {
 	    if ( objName )
-		ok = strcmp(objName,obj->name()) == 0;
+		ok = qstrcmp(objName,obj->name()) == 0;
 	    else if ( rx )
 		ok = rx->match(obj->name()) >= 0;
 	}
@@ -747,6 +799,8 @@ static void objSearch( QObjectList *result,
   \fn const QObjectList *QObject::children() const
   Returns a list of child objects, or 0 if this object has no children.
 
+  The QObjectList class is defined in the qobjcoll.h header file.
+
   The latest child added is the \link QList::first() first\endlink object
   in the list and the first child is added is the \link QList::last()
   last\endlink object in the list.
@@ -754,7 +808,7 @@ static void objSearch( QObjectList *result,
   Note that the list order might change when \link QWidget widget\endlink
   children are \link QWidget::raise() raised\endlink or \link
   QWidget::lower() lowered\endlink. A widget that is raised becomes the
-  first object in the list.  A widget that is lowered becomes the last
+  last object in the list.  A widget that is lowered becomes the first
   object in the list.
 
   \sa queryList(), parent(), insertChild(), removeChild()
@@ -781,12 +835,13 @@ static void objSearch( QObjectList *result,
     // Sets a Courier 24 point fonts for all children in myWidget that
     // inherit QButton (i.e. QPushButton, QCheckBox, QRadioButton).
     //
-    QObjectList	 *list = myWidget->queryList( "QButton" );
+    QObjectList	*list = myWidget->queryList( "QButton" );
     QObjectListIt it( *list );		// iterate over the buttons
-    QFont	  newFont( "Courier", 24 );
-    while ( it.current() ) {		// for each found object...
-	it.current()->setFont( newFont );
+    QFont newFont( "Courier", 24 );
+    QObject * obj;
+    while ( (obj=it.current()) != 0 ) {	// for each found object...
 	++it;
+	((QButton*)obj)->setFont( newFont );
     }
     delete list;			// delete the list, not the objects
   \endcode
@@ -841,6 +896,10 @@ QConnectionList *QObject::receivers( const char *signal ) const
 }
 
 
+#if QT_VERSION == 200
+#error "insertChild and removeChild should be virtual."
+#endif
+
 /*!
   Inserts an object \e obj into the list of child objects.
 
@@ -869,7 +928,7 @@ void QObject::insertChild( QObject *obj )
 #if defined(CHECK_STATE)
     else if ( childObjects->findRef(obj) >= 0 ) {
 	warning( "QObject::insertChild: Object %s::%s already in list",
-		 obj->className(), obj->name() );
+		 obj->className(), obj->name( "unnamed" ) );
 	return;
     }
 #endif
@@ -1039,6 +1098,19 @@ static void err_member_notfound( int code, const QObject *object,
 		 func, type, object->className(), member );
 }
 
+
+static void err_info_about_objects( const char * func,
+				    const QObject * sender,
+				    const QObject * receiver )
+{
+    const char * a = sender->name(), * b = receiver->name();
+    if ( a )
+	warning( "QObject::%s:  (sender name:   '%s')", func, a );
+    if ( b )
+	warning( "QObject::%s:  (receiver name: '%s')", func, b );
+}
+
+
 #endif // CHECK_RANGE
 
 
@@ -1252,6 +1324,7 @@ bool QObject::connect( const QObject *sender,	const char *signal,
     if ( !(sm=smeta->signal(signal,TRUE)) ) {	// no such signal
 #if defined(CHECK_RANGE)
 	err_member_notfound( SIGNAL_CODE, sender, signal, "connect" );
+	err_info_about_objects( "connect", sender, receiver );
 #endif
 	return FALSE;
     }
@@ -1278,6 +1351,7 @@ bool QObject::connect( const QObject *sender,	const char *signal,
     if ( !rm ) {				// no such member
 #if defined(CHECK_RANGE)
 	err_member_notfound( membcode, r, member, "connect" );
+	err_info_about_objects( "connect", sender, receiver );
 #endif
 	return FALSE;
     }
@@ -1331,6 +1405,10 @@ bool QObject::connect( const QObject *sender,	const char *signal,
   involved are destroyed.
 */
 
+#if (QT_VERSION >= 200)
+#error "disconnect const foo"
+#endif
+
 /*!
   Disconnects \e signal in object \e sender from \e member in object \e
   receiver.
@@ -1413,6 +1491,7 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	if ( !rm ) {				// no such member
 #if defined(CHECK_RANGE)
 	    err_member_notfound( membcode, r, member, "disconnect" );
+	    err_info_about_objects( "disconnect", sender, receiver );
 #endif
 	    return FALSE;
 	}
@@ -1515,8 +1594,8 @@ void QObject::cleanupEventFilter()
 
 /*!
   \internal
-  Returns the meta object for this object. If there is no meta object,
-  it calls initMetaObject() and makes another try.
+  Returns the meta object for this object. If necessary, calls
+  initMetaObject().
   \sa metaObject()
 */
 
@@ -1531,14 +1610,15 @@ QMetaObject *QObject::queryMetaObject() const
 #if defined(CHECK_NULL)
     if ( !m )					// still no meta object: error
 	warning( "QObject: Object %s::%s has no meta object",
-		 x->className(), x->name() );
+		 x->className(), x->name( "unnamed" ) );
 #endif
     return m;
 }
 
 /*!
-  Initializes the \link metaObject() meta object\endlink of this object.
-
+  Initializes the \link metaObject() meta object\endlink of this
+  object. This method is automatically executed on demand from the
+  QObject constructor.
   \sa metaObject()
 */
 void QObject::initMetaObject()
@@ -1561,11 +1641,11 @@ void QObject::initMetaObject()
 }
 
 
-/*****************************************************************************
-  Signal activation with the most frequently used parameter/argument types.
-  All other combinations are generated by the meta object compiler.
- *****************************************************************************/
-
+/*!
+  \internal Signal activation with the most frequently used
+  parameter/argument types.  All other combinations are generated by the
+  meta object compiler.
+*/
 void QObject::activate_signal( const char *signal )
 {
     if ( !connections )
@@ -1588,6 +1668,23 @@ void QObject::activate_signal( const char *signal )
     }
 }
 
+/*!
+  \overload void QObject::activate_signal( const char *signal, short )
+*/
+
+/*!
+  \overload void QObject::activate_signal( const char *signal, int )
+*/
+
+/*!
+  \overload void QObject::activate_signal( const char *signal, long )
+*/
+
+/*!
+  \overload void QObject::activate_signal( const char *signal, const char * )
+*/
+
+
 #define ACTIVATE_SIGNAL_WITH_PARAM(TYPE)				      \
 void QObject::activate_signal( const char *signal, TYPE param )		      \
 {									      \
@@ -1637,8 +1734,23 @@ static void dumpRecursive( int level, QObject *object )
     if ( object ) {
 	QString buf;
 	buf.fill( '\t', level );
-	const char *name = object->name() ? object->name() : "????";
-	debug( "%s%s::%s", (const char*)buf, object->className(), name );
+	const char *name = object->name( "unnamed" );
+	QString flags="";
+	if ( qApp->focusWidget() == object )
+	    flags += 'F';
+	if ( object->isWidgetType() ) {
+	    QWidget * w = (QWidget *)object;
+	    if ( w->isVisible() ) {
+		QString t;
+		t.sprintf( "<%d,%d,%d,%d>",
+			   w->x(), w->y(), w->width(), w->height() );
+		flags += t;
+	    } else {
+		flags += 'I';
+	    }
+	}	
+	debug( "%s%s::%s %s", (const char*)buf, object->className(), name,
+	    (const char*)flags );
 	if ( object->children() ) {
 	    QObjectListIt it(*object->children());
 	    while ( it ) {
@@ -1671,7 +1783,7 @@ void QObject::dumpObjectTree()
 void QObject::dumpObjectInfo()
 {
 #if defined(DEBUG)
-    debug( "OBJECT %s::%s", className(), name() );
+    debug( "OBJECT %s::%s", className(), name( "unnamed" ) );
     debug( "  SIGNALS OUT" );
     int n = 0;
     if ( connections ) {
@@ -1686,7 +1798,7 @@ void QObject::dumpObjectInfo()
 	    while ( (c=cit.current()) ) {
 		++cit;
 		debug( "\t  --> %s::%s %s", c->object()->className(),
-		       c->object()->name(), c->memberName() );
+		       c->object()->name( "unnamed" ), c->memberName() );
 	    }
 	}
     }
@@ -1697,7 +1809,8 @@ void QObject::dumpObjectInfo()
     if ( senderObjects ) {
 	QObject *sender = senderObjects->first();
 	while ( sender ) {
-	    debug( "\t%s::%s", sender->className(), sender->name() );
+	    debug( "\t%s::%s",
+		   sender->className(), sender->name( "unnamed" ) );
 	    n++;
 	    sender = senderObjects->next();
 	}
@@ -1706,3 +1819,14 @@ void QObject::dumpObjectInfo()
 	debug( "\t<None>" );
 #endif
 }
+
+
+
+/*!  Returns a translated version of \a text, or \a text if there is
+  no appropriate translated version.
+*/
+
+const char * QObject::tr( const char * text ) const
+{
+    return text;
+}
