@@ -1,30 +1,28 @@
 /****************************************************************************
-** $Id: qpalette.cpp,v 2.13 1997/08/28 00:39:41 hanord Exp $
+** $Id: qpalette.cpp,v 2.19 1998/07/03 00:09:36 hanord Exp $
 **
 ** Implementation of QColorGroup and QPalette classes
 **
 ** Created : 950323
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qpalette.h"
-#include "qdstream.h"
-
-RCSTAG("$Id: qpalette.cpp,v 2.13 1997/08/28 00:39:41 hanord Exp $");
-
+#include "qdatastream.h"
 
 /*****************************************************************************
   QColorGroup member functions
@@ -35,6 +33,7 @@ RCSTAG("$Id: qpalette.cpp,v 2.13 1997/08/28 00:39:41 hanord Exp $");
   \brief The QColorGroup class contains a group of widget colors.
 
   \ingroup color
+  \ingroup drawing
 
   A color group contains a group of colors used by widgets for drawing
   themselves.  Widgets should not use colors like "red" and "turqoise"
@@ -173,10 +172,11 @@ bool QColorGroup::operator==( const QColorGroup &g ) const
 /*!
   \class QPalette qpalette.h
 
-  \brief The QPalette class contains color groups.
+  \brief The QPalette class contains color groups for each widget state.
 
   \ingroup color
   \ingroup shared
+  \ingroup drawing
 
   A palette consists of three color groups: a \e normal, a \e disabled
   and an \e active color group.	 All \link QWidget widgets\endlink
@@ -259,7 +259,7 @@ QPalette::QPalette( const QColorGroup &normal, const QColorGroup &disabled,
 }
 
 /*!
-  Constructs a palette that is a 
+  Constructs a palette that is a
   \link shclass.html shallow copy\endlink of \e p.
   \sa copy()
 */
@@ -297,7 +297,7 @@ QPalette &QPalette::operator=( const QPalette &p )
 
 
 /*!
-  Returns a 
+  Returns a
   \link shclass.html deep copy\endlink of the palette.
 */
 
@@ -493,3 +493,16 @@ QDataStream &operator>>( QDataStream &s, QPalette &p )
     p = newpal;
     return s;
 }
+
+
+/*!  Returns TRUE if this palette and \a p are copies of each other,
+  ie. one of them was created as a copy of the other and neither was
+  subsequently modified.  This is much stricter than equality.
+  
+  \sa operator=, operator==
+*/
+
+bool QPalette::isCopyOf( const QPalette & p )
+{
+    return data && data == p.data;
+}
