@@ -1,41 +1,40 @@
 /****************************************************************************
-** $Id: qpicture.cpp,v 2.2 1997/08/06 15:05:41 hanord Exp $
+** $Id: qpicture.cpp,v 2.7 1998/07/03 00:09:37 hanord Exp $
 **
 ** Implementation of QPicture class
 **
 ** Created : 940802
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qpicture.h"
-#include "qpaintdc.h"
+#include "qpaintdevicedefs.h"
 #include "qpainter.h"
 #include "qpixmap.h"
+#include "qimage.h"
 #include "qfile.h"
-#include "qdstream.h"
-
-RCSTAG("$Id: qpicture.cpp,v 2.2 1997/08/06 15:05:41 hanord Exp $");
-
+#include "qdatastream.h"
 
 /*!
   \class QPicture qpicture.h
   \brief The QPicture class is a paint device that records and replays QPainter
   commands.
 
-  \ingroup paintdevice
+  \ingroup drawing
 
   A picture serializes painter commands to an IO device in a
   platform-independent format.	A picture created under Windows
@@ -347,6 +346,12 @@ bool QPicture::exec( QPainter *painter, QDataStream &s, int nrecords )
 		painter->drawPixmap( p, pixmap );
 		}
 		break;
+	    case PDC_DRAWIMAGE: {
+		QImage image;
+		s >> p >> image;
+		painter->drawImage( p, image );
+		}
+		break;
 	    case PDC_BEGIN:
 		s >> ul;			// number of records
 		if ( !exec( painter, s, ul ) )
@@ -530,6 +535,10 @@ bool QPicture::cmd( int c, QPainter *, QPDevCmdParam *p )
 	    s << *p[0].point;
 	    s << *p[1].pixmap;
 	    break;
+	case PDC_DRAWIMAGE:
+	    s << *p[0].point;
+	    s << *p[1].image;
+	    break;
 	case PDC_SAVE:
 	case PDC_RESTORE:
 	    break;
