@@ -1,39 +1,39 @@
 /****************************************************************************
-** $Id: qpoint.cpp,v 2.1 1997/08/06 15:05:42 hanord Exp $
+** $Id: qpoint.cpp,v 2.5 1998/07/03 00:09:37 hanord Exp $
 **
 ** Implementation of QPoint class
 **
 ** Created : 931028
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #define QPOINT_C
 #include "qpoint.h"
-#include "qdstream.h"
-
-RCSTAG("$Id: qpoint.cpp,v 2.1 1997/08/06 15:05:42 hanord Exp $");
-
+#include "qdatastream.h"
 
 /*!
   \class QPoint qpoint.h
   \brief The QPoint class defines a point in the plane.
 
+  \ingroup drawing
+
   A point is specified by an x coordinate and a y coordinate.
 
-  The coordinate type is QCOORD (defined in qwindefs.h as \c short).
+  The coordinate type is QCOORD (defined in qwindowdefs.h as \c short).
   The minimum value of QCOORD is QCOORD_MIN (-32768) and the maximum
   value is  QCOORD_MAX (32767).
 
