@@ -1,46 +1,43 @@
 /**********************************************************************
-** $Id: qprinter.cpp,v 2.1 1997/02/24 19:19:51 agulbra Exp $
+** $Id: qprinter.cpp,v 2.10 1998/07/03 00:09:38 hanord Exp $
 **
 ** Implementation of QPrinter class
 **
 ** Created : 941003
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qprinter.h"
 
-RCSTAG("$Id: qprinter.cpp,v 2.1 1997/02/24 19:19:51 agulbra Exp $");
-
-
 /*!
   \class QPrinter qprinter.h
   \brief The QPrinter class is a paint device that prints graphics on a
   printer.
 
-  \ingroup paintdevice
+  \ingroup drawing
 
   All window systems that Qt supports, except X11, have built-in
   printer drivers.  For X11, Qt provides PostScript (tm)
   printing.
 
-  drawPixmap(), clipping, even-odd filling, landscape orientation/page
-  size support and pixmap brushes are all either unimplemented or
-  badly broken.
+  Clipping and even-odd filling are not supported.
 
   Font support is limited to Times, Helvetica, Courier and Symbol, and
-  will be for "a few weeks" (which is a six-month old quote from
-  Adobe's FTP server about AFM file availability) and font metrics
-  even for these four fonts can be inaccurate.
+  font metrics for these four fonts can be inaccurate.
 
   Drawing graphics on a printer is almost identical to drawing graphics
   in a widget or a pixmap.  The only difference is that the programmer
@@ -65,16 +62,17 @@ RCSTAG("$Id: qprinter.cpp,v 2.1 1997/02/24 19:19:51 agulbra Exp $");
   Example (a complete application):
 
   \code
-    #include <qapp.h>
+    #include <qapplication.h>
     #include <qpainter.h>
     #include <qprinter.h>
+    #include <qprintdialog.h>
 
     int main( int argc, char **argv )
     {
 	QApplication a( argc, argv );
 
 	QPrinter prt;
-	if ( prt.setup(0) ) {
+	if ( QPrintDialog::getPrinterSetup( &prt ) ) {
 	    QPainter p;
 	    p.begin( &prt );
 	    p.rotate( 55 );
@@ -274,7 +272,7 @@ void QPrinter::setCreator( const char *creator )
   Sets the print orientation.
 
   The orientation can be either \c QPrinter::Portrait or
-  \c QPainter::Landscape.
+  \c QPrinter::Landscape.
 
   The printer driver reads this setting and prints using the specified
   orientation.
@@ -294,8 +292,18 @@ void QPrinter::setOrientation( Orientation orientation )
   \sa setPageSize()
 */
 
+
+static QPrinter::PageSize makepagesize( QPrinter::PageSize ps,
+					QPrinter::PageOrder po,
+					QPrinter::ColorMode cm )
+{
+    return (QPrinter::PageSize)( ((int)ps & 255) + 
+				 ((po == QPrinter::LastPageFirst) ? 256 : 0) +
+				 ((cm == QPrinter::GrayScale) ? 512 : 0) );
+}
+
 /*!
-  Sets the printer page size.
+  Sets the printer page size to \a newPageSize.
 
   The page size can be one of:
   <ul>
@@ -309,9 +317,68 @@ void QPrinter::setOrientation( Orientation orientation )
   \sa pageSize()
 */
 
-void QPrinter::setPageSize( PageSize pageSize )
+void QPrinter::setPageSize( PageSize newPageSize )
 {
-    page_size = pageSize;
+    page_size = makepagesize( newPageSize, pageOrder(), colorMode() );
+}
+
+
+/*!  Sets the page order to \a newPageOrder.
+
+  The page order can be \c QPrinter::FirstPageFirst or \c
+  QPrinter::LastPageFirst.  The application programmer is responsible
+  for reading the page order and printing accordingly.
+*/
+
+void QPrinter::setPageOrder( PageOrder newPageOrder )
+{
+    page_size = makepagesize( pageSize(), newPageOrder, colorMode() );
+}
+
+
+/*!  Returns the current page order.
+
+  The default page order is \a FirstPageFirst.
+*/
+
+QPrinter::PageOrder QPrinter::pageOrder() const
+{
+    if ( ((int)page_size) & 256 )
+	return QPrinter::LastPageFirst;
+    else
+	return QPrinter::FirstPageFirst;
+}
+
+
+/*!  Sets the printer's color mode to \a newColorMode, which can be
+  one of \c Color (the default) and \c GrayScale.
+  
+  A future version of Qt will modify its printing accordingly.  At
+  present, QPrinter behaves as if \c Color is selected.
+  
+  \sa colorMode()  
+*/
+
+void QPrinter::setColorMode( ColorMode newColorMode )
+{
+    page_size = makepagesize( pageSize(), pageOrder(), newColorMode );
+}
+
+
+/*!  Returns the current color mode.  The default color more is \c
+  Color.
+  
+  \sa setColorMode()
+*/
+
+QPrinter::ColorMode QPrinter::colorMode() const
+{
+    if ( ((int)page_size) & 512 )
+	return QPrinter::GrayScale;
+    else
+	return QPrinter::Color;
+    
+    
 }
 
 
