@@ -1,29 +1,32 @@
 /****************************************************************************
-** $Id: qrect.h,v 2.5 1997/09/04 12:33:32 warwick Exp $
+** $Id: qrect.h,v 2.7 1998/07/03 00:09:40 hanord Exp $
 **
 ** Definition of QRect class
 **
 ** Created : 931028
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QRECT_H
 #define QRECT_H
 
+#ifndef QT_H
 #include "qsize.h"
+#endif // QT_H
 
 
 class QRect					// rectangle class
