@@ -1,28 +1,29 @@
 /****************************************************************************
-** $Id: qwindow.cpp,v 2.0 1996/09/24 10:16:05 agulbra Exp $
+** $Id: qwindow.cpp,v 2.4 1998/07/03 00:09:42 hanord Exp $
 **
 ** Implementation of QWindow class
 **
 ** Created : 931211
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qwindow.h"
 #include "qpixmap.h"
 
-RCSTAG("$Id: qwindow.cpp,v 2.0 1996/09/24 10:16:05 agulbra Exp $");
-
-
 /*!
   \class QWindow qwindow.h
   \brief The QWindow class is reserved for future extensions.
@@ -45,7 +46,7 @@ RCSTAG("$Id: qwindow.cpp,v 2.0 1996/09/24 10:16:05 agulbra Exp $");
 QWindow::QWindow( QWidget *parent, const char *name, WFlags f )
     : QWidget( parent, name, f )
 {
-    initMetaObject();
+    // nothing
 }
 
 /*!
