@@ -1,41 +1,41 @@
 /****************************************************************************
-** $Id: qwmatrix.cpp,v 2.1 1997/08/06 15:05:49 hanord Exp $
+** $Id: qwmatrix.cpp,v 2.6 1998/07/03 00:09:42 hanord Exp $
 **
 ** Implementation of QWMatrix class
 **
 ** Created : 941020
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qwmatrix.h"
-#include "qdstream.h"
+#include "qdatastream.h"
 #if defined(_WS_X11_)
-double qsincos( double, bool calcCos );		// defined in qptr_x11.cpp
+double qsincos( double, bool calcCos );		// defined in qpainter_x11.cpp
 #else
 #include <math.h>
 #endif
 
-RCSTAG("$Id: qwmatrix.cpp,v 2.1 1997/08/06 15:05:49 hanord Exp $");
-
-
 /*!
   \class QWMatrix qwmatrix.h
   \brief The QWMatrix class specifies 2D transformations of a
   coordinate system.
 
+  \ingroup drawing
+
   The standard coordinate system of a \link QPaintDevice paint
   device\endlink has the origin located at the top left position. X
   values increase to the right, and Y values increase downwards.
@@ -257,13 +257,15 @@ QPoint QWMatrix::map( const QPoint &p ) const
 
 QRect QWMatrix::map( const QRect &r ) const
 {
+    QRect result;
     if ( _m12 == 0.0F && _m21 == 0.0F ) {
-	return QRect( map(r.topLeft()), map(r.bottomRight()) );
+	result = QRect( map(r.topLeft()), map(r.bottomRight()) );
     } else {
 	QPointArray a( r );
 	a = map( a );
-	return a.boundingRect();
+	result = a.boundingRect();
     }
+    return result;
 }
 
 /*!
