@@ -13,12 +13,12 @@ void yyerror( char *msg );
 
 #include "qlist.h"
 #include "qstring.h"
-#include "qdatetm.h"
+#include "qdatetime.h"
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-RCSTAG("$Id: moc.y,v 2.14 1997/03/14 20:37:10 hanord Exp $");
+RCSTAG("$Id: moc.y,v 2.25 1998/07/06 19:22:17 hanord Exp $");
 
 static QString rmWS( const char * );
 
@@ -29,7 +29,10 @@ struct Argument					/* single arg meta data*/
 {
     Argument( char *left, char *right )
 	{ leftType  = rmWS( left );
-	  rightType = rmWS( right ); }
+	  rightType = rmWS( right );
+	  if ( leftType == "void" && rightType.isEmpty() )
+	      leftType = "";
+	}
     QString leftType;
     QString rightType;
 };
@@ -95,7 +98,7 @@ QString	   tmpExpression;
 
 const int  formatRevision = 2;			/* moc output format revision*/
 
-#line 120 "moc.y"
+#line 123 "moc.y"
 typedef union {
     char	char_val;
     int		int_val;
@@ -106,7 +109,7 @@ typedef union {
     ArgList    *arg_list;
     Argument   *arg;
 } YYSTYPE;
-#line 110 "y.tab.c"
+#line 113 "y.tab.c"
 #define CHAR_VAL 257
 #define INT_VAL 258
 #define DOUBLE_VAL 259
@@ -848,12 +851,14 @@ YYSTYPE yylval;
 short yyss[YYSTACKSIZE];
 YYSTYPE yyvs[YYSTACKSIZE];
 #define yystacksize YYSTACKSIZE
-#line 759 "moc.y"
+#line 760 "moc.y"
 
 #if defined(_OS_WIN32_)
 #if defined(_CC_MSVC_)
 extern "C" int _isatty( int );
 #define isatty _isatty
+#elif defined(_CC_SYM_)
+#include <io.h>
 #endif
 #endif
 
@@ -869,22 +874,27 @@ QString	  fileName;				// file name
 QString	  outputFile;				// output file name
 QString   includeFile;				// name of #include file
 QString	  includePath;				// #include file path
+QString	  qtPath;				// #include qt file path
 bool	  noInclude     = FALSE;		// no #include <filename>
 bool	  generatedCode = FALSE;		// no code generated
 QString	  className;				// name of parsed class
 QString	  superclassName;			// name of super class
 FuncList  signals;				// signal interface
 FuncList  slots;				// slots interface
+QDict<char> defines;				// simple #defines
 
 FILE  *out;					// output file
 
 int yyparse();
 
+void replace( char *s, char c1, char c2 );
 
 int main( int argc, char **argv )
 {
     bool autoInclude = TRUE;
     char *error	     = 0;
+    qtPath = "";
+
     for ( int n=1; n<argc && error==0; n++ ) {
 	QString arg = argv[n];
 	if ( arg[0] == '-' ) {			// option
@@ -901,6 +911,13 @@ int main( int argc, char **argv )
 	    } else if ( opt == "i" ) {		// no #include statement
 		noInclude   = TRUE;
 		autoInclude = FALSE;
+	    } else if ( opt[0] == 'D' ) {	// simple -DMACRO
+		int eq = opt.find('=');
+		if ( strlen(opt) > 1 && eq != 0 ) {
+		    defines.insert( opt.mid(1, eq < 0 ? 9999 : eq-1),
+				    qstrdup(eq < 0 ? "" :
+					    (const char*)opt.mid(eq,9999)) );
+		}
 	    } else if ( opt[0] == 'f' ) {	// produce #include statement
 		noInclude   = FALSE;
 		autoInclude = FALSE;
@@ -910,12 +927,26 @@ int main( int argc, char **argv )
 	    } else if ( opt[0] == 'p' ) {	// include file path
 		if ( opt[1] == '\0' ) {
 		    if ( !(n < argc-1) ) {
-			error = "Missing path name";
+			error = "Missing path name for the -p option.";
 			break;
 		    }
 		    includePath = argv[++n];
-		} else
+		} else {
 		    includePath = &opt[1];
+		}
+	    } else if ( opt[0] == 'q' ) {	// qt include file path
+		if ( opt[1] == '\0' ) {
+		    if ( !(n < argc-1) ) {
+			error = "Missing path name for the -q option.";
+			break;
+		    }
+		    qtPath = argv[++n];
+		} else {
+		    qtPath = &opt[1];
+		}
+		replace(qtPath.data(),'\\','/');
+		if ( qtPath.right(1) != "/" )
+		    qtPath += '/';
 	    } else if ( opt == "k" ) {		// don't stop on errors
 		errorControl = TRUE;
 	    } else if ( opt == "nw" ) {		// don't display warnings
@@ -963,16 +994,17 @@ int main( int argc, char **argv )
 	if ( error )
 	    fprintf( stderr, "moc: %s\n", error );
 	fprintf( stderr, "Usage:  moc [options] <header-file>\n"
-		 "\t-o file  Write output to file rather than stdout\n"
-		 "\t-i       Do not generate an #include statement\n"
-		 "\t-f[file] Force #include, optional file name\n"
-		 "\t-p path  Path prefix for included file\n"
-		 "\t-k       Do not stop on errors\n"
-		 "\t-nw      Do not display warnings\n"
+		 "\t-D[macro] Define a macro (for #ifdef processing)\n"
+		 "\t-o file   Write output to file rather than stdout\n"
+		 "\t-i        Do not generate an #include statement\n"
+		 "\t-f[file]  Force #include, optional file name\n"
+		 "\t-p path   Path prefix for included file\n"
+		 "\t-k        Do not stop on errors\n"
+		 "\t-nw       Do not display warnings\n"
 #if 0
-		 "\t-ldbg    lex debug output\n"
-		 "\t-ydbg    yacc debug output\n"
-		 "\t-dbg     test parser, all non-signal members are slots\n"
+		 "\t-ldbg     lex debug output\n"
+		 "\t-ydbg     yacc debug output\n"
+		 "\t-dbg      test parser, all non-signal members are slots\n"
 #endif
 		 );
 	return 1;
@@ -1009,7 +1041,6 @@ int main( int argc, char **argv )
     return 0;
 }
 
-
 void replace( char *s, char c1, char c2 )
 {
     if ( !s )
@@ -1110,8 +1141,9 @@ void initClass()				 // prepare for new class
 //
 
 inline bool isIdentChar( char x )
-{
-    return isalnum(x) || x == '_';
+{						// Avoid bug in isalnum
+    return x == '_' || (x >= '0' && x <= '9') ||
+	 (x >= 'a' && x <= 'z') || (x >= 'A' && x <= 'Z');
 }
 
 inline bool isSpace( char x )
@@ -1293,15 +1325,17 @@ void generateFuncs( FuncList *list, char *functype, int num )
 	int count = 0;
 	Argument *a = f->args->first();
 	while ( a ) {
-	    if ( count++ )
-		typstr += ",";
-	    typstr += a->leftType;
-	    typstr += a->rightType;
+	    if ( !a->leftType.isEmpty() || ! a->rightType.isEmpty() ) {
+		if ( count++ )
+		    typstr += ",";
+		typstr += a->leftType;
+		typstr += a->rightType;
+	    }
 	    a = f->args->next();
 	}
-	fprintf( out, "    typedef %s(%s::*m%d_t%d)(%s);\n",
-		 (const char*)f->type,
-		 (const char*)className, num, list->at(),(const char*)typstr );
+	fprintf( out, "    typedef %s(%s::*m%d_t%d)(%s)%s;\n",
+		 (const char*)f->type, (const char*)className, num, list->at(),
+		 (const char*)typstr,  (const char*)f->qualifier );
 	f->type = f->name.copy();
 	f->type += "(";
 	f->type += typstr;
@@ -1327,7 +1361,7 @@ void generateClass()		      // generate C++ source code for a class
     char *hdr1 = "/****************************************************************************\n"
 		 "** %s meta object code from reading C++ file '%s'\n**\n";
     char *hdr2 = "** Created: %s\n"
-		 "**      by: The Qt Meta Object Compiler ($Revision: 2.14 $)\n**\n";
+		 "**      by: The Qt Meta Object Compiler ($Revision: 2.25 $)\n**\n";
     char *hdr3 = "** WARNING! All changes made in this file will be lost!\n";
     char *hdr4 = "*****************************************************************************/\n\n";
     int   i;
@@ -1370,12 +1404,12 @@ void generateClass()		      // generate C++ source code for a class
 	fprintf( out, "#if !defined(Q_MOC_OUTPUT_REVISION)\n" );
 	fprintf( out, "#define Q_MOC_OUTPUT_REVISION %d\n", formatRevision );
 	fprintf( out, "#elif Q_MOC_OUTPUT_REVISION != %d\n", formatRevision );
-	fprintf( out, "#error Moc format conflict - "
-		 "please regenerate all moc files\n" );
+	fprintf( out, "#error \"Moc format conflict - "
+		 "please regenerate all moc files\"\n" );
 	fprintf( out, "#endif\n\n" );
-	fprintf( out, "#include <qmetaobj.h>\n" );
 	if ( !noInclude )
 	    fprintf( out, "#include \"%s\"\n", (const char*)includeFile );
+	fprintf( out, "#include <%sqmetaobject.h>\n", (const char*)qtPath );
 	fprintf( out, "\n\n" );
     } else {
 	fprintf( out, "\n\n" );
@@ -1452,23 +1486,25 @@ void generateClass()		      // generate C++ source code for a class
 
 	i = 0;
 	while ( a ) {				// argument list
-	    if ( i ) {
-		typstr += ",";
-		valstr += ", ";
-		argstr += ", ";
+	    if ( !a->leftType.isEmpty() || !a->rightType.isEmpty() ) {
+		if ( i ) {
+		    typstr += ",";
+		    valstr += ", ";
+		    argstr += ", ";
+		}
+		typstr += a->leftType;
+		typstr += a->rightType;
+		argstr += a->leftType;
+		argstr += " ";
+		sprintf( buf, "t%d", i );
+		valstr += buf;
+		argstr += buf;
+		argstr += a->rightType;
+		++i;
+		typvec[i] = typstr.copy();
+		valvec[i] = valstr.copy();
+		argvec[i] = argstr.copy();
 	    }
-	    typstr += a->leftType;
-	    typstr += a->rightType;
-	    argstr += a->leftType;
-	    argstr += " ";
-	    sprintf( buf, "t%d", i );
-	    valstr += buf;
-	    argstr += buf;
-	    argstr += a->rightType;
-	    ++i;
-	    typvec[i] = typstr.copy();
-	    valvec[i] = valstr.copy();
-	    argvec[i] = argstr.copy();
 	    a = f->args->next();
 	}
 
@@ -1480,7 +1516,7 @@ void generateClass()		      // generate C++ source code for a class
 	if ( !predef_call && !included_list_stuff ) {
 	    fprintf( out, "\n#if !defined(Q_MOC_CONNECTIONLIST_DECLARED)\n" );
 	    fprintf( out, "#define Q_MOC_CONNECTIONLIST_DECLARED\n" );
-	    fprintf( out, "#include <qlist.h>\n" );
+	    fprintf( out, "#include <%sqlist.h>\n", (const char*)qtPath );
 	    fprintf( out, "#if defined(Q_DECLARE)\n" );
 	    fprintf( out, "Q_DECLARE(QListM,QConnection);\n" );
 	    fprintf( out, "Q_DECLARE(QListIteratorM,QConnection);\n" );
@@ -1594,7 +1630,7 @@ void addMember( char m )
     tmpFunc  = new Function;
     CHECK_PTR( tmpFunc );
 }
-#line 1598 "y.tab.c"
+#line 1634 "y.tab.c"
 #define YYABORT goto yyabort
 #define YYREJECT goto yyabort
 #define YYACCEPT goto yyaccept
@@ -1736,463 +1772,463 @@ yyreduce:
     switch (yyn)
     {
 case 3:
-#line 218 "moc.y"
+#line 221 "moc.y"
 { initClass(); }
 break;
 case 4:
-#line 219 "moc.y"
+#line 222 "moc.y"
 { generateClass();
 						BEGIN OUTSIDE; }
 break;
 case 5:
-#line 226 "moc.y"
+#line 229 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 6:
-#line 227 "moc.y"
+#line 230 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 7:
-#line 231 "moc.y"
+#line 234 "moc.y"
 { yyval.string = stradd( yyvsp[-3].string, "<",
 				     tmpExpression =
 				     tmpExpression.stripWhiteSpace(), ">" ); }
 break;
 case 8:
-#line 242 "moc.y"
+#line 245 "moc.y"
 { initExpression();
 						    templLevel = 1;
 						    BEGIN IN_TEMPL_ARGS; }
 break;
 case 9:
-#line 255 "moc.y"
+#line 258 "moc.y"
 { initExpression();
 						    BEGIN IN_EXPR; }
 break;
 case 10:
-#line 261 "moc.y"
+#line 264 "moc.y"
 { yyval.string = ""; }
 break;
 case 11:
-#line 262 "moc.y"
+#line 265 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 12:
-#line 263 "moc.y"
+#line 266 "moc.y"
 { yyval.string = ""; }
 break;
 case 13:
-#line 264 "moc.y"
+#line 267 "moc.y"
 { skipFunc = TRUE; yyval.string = ""; }
 break;
 case 14:
-#line 265 "moc.y"
+#line 268 "moc.y"
 { skipFunc = TRUE; yyval.string = ""; }
 break;
 case 15:
-#line 269 "moc.y"
+#line 272 "moc.y"
 { yyval.string = straddSpc(yyvsp[-2].string,yyvsp[-1].string,yyvsp[0].string); }
 break;
 case 16:
-#line 272 "moc.y"
+#line 275 "moc.y"
 { yyval.string = ""; }
 break;
 case 17:
-#line 273 "moc.y"
-{ yyval.string = yyvsp[0].string; }
-break;
-case 18:
 #line 276 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
+case 18:
+#line 279 "moc.y"
+{ yyval.string = yyvsp[0].string; }
+break;
 case 19:
-#line 277 "moc.y"
+#line 280 "moc.y"
 { yyval.string = straddSpc(yyvsp[-1].string,yyvsp[0].string); }
 break;
 case 22:
-#line 282 "moc.y"
+#line 285 "moc.y"
 { skipFunc = TRUE; }
 break;
 case 26:
-#line 290 "moc.y"
+#line 293 "moc.y"
 { yyval.string = "const"; }
 break;
 case 27:
-#line 291 "moc.y"
+#line 294 "moc.y"
 { yyval.string = "volatile"; }
 break;
 case 28:
-#line 294 "moc.y"
+#line 297 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 29:
-#line 295 "moc.y"
+#line 298 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 30:
-#line 296 "moc.y"
+#line 299 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 31:
-#line 300 "moc.y"
+#line 303 "moc.y"
 { yyval.string = straddSpc(yyvsp[-1].string,yyvsp[0].string); }
 break;
 case 32:
-#line 301 "moc.y"
+#line 304 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 33:
-#line 303 "moc.y"
+#line 306 "moc.y"
 { yyval.string = "char"; }
 break;
 case 34:
-#line 304 "moc.y"
+#line 307 "moc.y"
 { yyval.string = "short"; }
 break;
 case 35:
-#line 305 "moc.y"
+#line 308 "moc.y"
 { yyval.string = "int"; }
 break;
 case 36:
-#line 306 "moc.y"
+#line 309 "moc.y"
 { yyval.string = "long"; }
 break;
 case 37:
-#line 307 "moc.y"
+#line 310 "moc.y"
 { yyval.string = "signed"; }
 break;
 case 38:
-#line 308 "moc.y"
+#line 311 "moc.y"
 { yyval.string = "unsigned"; }
 break;
 case 39:
-#line 309 "moc.y"
+#line 312 "moc.y"
 { yyval.string = "float"; }
 break;
 case 40:
-#line 310 "moc.y"
+#line 313 "moc.y"
 { yyval.string = "double"; }
 break;
 case 41:
-#line 311 "moc.y"
+#line 314 "moc.y"
 { yyval.string = "void"; }
 break;
 case 42:
-#line 315 "moc.y"
+#line 318 "moc.y"
 { yyval.string = stradd( "template<",
 				     tmpExpression =
 				     tmpExpression.stripWhiteSpace(), ">" ); }
 break;
 case 44:
-#line 321 "moc.y"
+#line 324 "moc.y"
 { templateClass = TRUE; }
 break;
 case 45:
-#line 325 "moc.y"
+#line 328 "moc.y"
 { yyval.string = "class"; }
 break;
 case 46:
-#line 326 "moc.y"
+#line 329 "moc.y"
 { yyval.string = "struct"; }
 break;
 case 47:
-#line 329 "moc.y"
+#line 332 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 48:
-#line 331 "moc.y"
+#line 334 "moc.y"
 { yyval.string = stradd( "::", yyvsp[0].string ); }
 break;
 case 49:
-#line 335 "moc.y"
+#line 338 "moc.y"
 { yyval.string = stradd( yyvsp[-2].string, "::", yyvsp[0].string );}
 break;
 case 50:
-#line 336 "moc.y"
+#line 339 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 51:
-#line 340 "moc.y"
+#line 343 "moc.y"
 { yyval.string = straddSpc(yyvsp[-1].string,yyvsp[0].string); }
 break;
 case 52:
-#line 341 "moc.y"
+#line 344 "moc.y"
 { yyval.string = stradd("enum ",yyvsp[0].string); }
 break;
 case 53:
-#line 342 "moc.y"
+#line 345 "moc.y"
 { yyval.string = stradd("union ",yyvsp[0].string); }
 break;
 case 54:
-#line 347 "moc.y"
+#line 350 "moc.y"
 { yyval.arg_list = yyvsp[-1].arg_list;}
 break;
 case 55:
-#line 348 "moc.y"
+#line 351 "moc.y"
 { yyval.arg_list = yyvsp[-2].arg_list;
 				       moc_warn("Ellipsis not supported"
 					       " in signals and slots.\n"
 					       "Ellipsis argument ignored."); }
 break;
 case 56:
-#line 354 "moc.y"
+#line 357 "moc.y"
 { yyval.arg_list = tmpArgList; }
 break;
 case 57:
-#line 355 "moc.y"
+#line 358 "moc.y"
 { yyval.arg_list = yyvsp[0].arg_list; }
 break;
 case 59:
-#line 359 "moc.y"
+#line 362 "moc.y"
 { moc_warn("Ellipsis not supported"
 					       " in signals and slots.\n"
 					       "Ellipsis argument ignored."); }
 break;
 case 60:
-#line 367 "moc.y"
+#line 370 "moc.y"
 { yyval.arg_list = addArg(yyvsp[0].arg); }
 break;
 case 61:
-#line 368 "moc.y"
+#line 371 "moc.y"
 { yyval.arg_list = addArg(yyvsp[0].arg); }
 break;
 case 62:
-#line 371 "moc.y"
+#line 374 "moc.y"
 { yyval.arg = new Argument(straddSpc(yyvsp[-1].string,yyvsp[0].string),"");
 				  CHECK_PTR(yyval.arg); }
 break;
 case 63:
-#line 374 "moc.y"
+#line 377 "moc.y"
 { expLevel = 1; }
 break;
 case 64:
-#line 376 "moc.y"
+#line 379 "moc.y"
 { yyval.arg = new Argument(straddSpc(yyvsp[-4].string,yyvsp[-3].string),"");
 				  CHECK_PTR(yyval.arg); }
 break;
 case 65:
-#line 380 "moc.y"
+#line 383 "moc.y"
 { yyval.arg = new Argument(straddSpc(yyvsp[-3].string,yyvsp[-2].string),yyvsp[0].string);
 				  CHECK_PTR(yyval.arg); }
 break;
 case 66:
-#line 384 "moc.y"
+#line 387 "moc.y"
 { expLevel = 1; }
 break;
 case 67:
-#line 386 "moc.y"
+#line 389 "moc.y"
 { yyval.arg = new Argument(straddSpc(yyvsp[-6].string,yyvsp[-5].string),yyvsp[-3].string);
 				  CHECK_PTR(yyval.arg); }
 break;
 case 68:
-#line 391 "moc.y"
+#line 394 "moc.y"
 { yyval.string = ""; }
 break;
 case 69:
-#line 392 "moc.y"
+#line 395 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 70:
-#line 396 "moc.y"
+#line 399 "moc.y"
 { yyval.string = straddSpc(yyvsp[-1].string,yyvsp[0].string); }
 break;
 case 71:
-#line 397 "moc.y"
+#line 400 "moc.y"
 { expLevel = 1; }
 break;
 case 72:
-#line 399 "moc.y"
+#line 402 "moc.y"
 { yyval.string = stradd( "[",
 				     tmpExpression =
 				     tmpExpression.stripWhiteSpace(), "]" ); }
 break;
 case 73:
-#line 402 "moc.y"
+#line 405 "moc.y"
 { expLevel = 1; }
 break;
 case 74:
-#line 404 "moc.y"
+#line 407 "moc.y"
 { yyval.string = stradd( yyvsp[-4].string,"[",
 				     tmpExpression =
 				     tmpExpression.stripWhiteSpace(),"]" ); }
 break;
 case 75:
-#line 407 "moc.y"
+#line 410 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 76:
-#line 408 "moc.y"
+#line 411 "moc.y"
 { yyval.string = yyvsp[-1].string; }
 break;
 case 77:
-#line 411 "moc.y"
+#line 414 "moc.y"
 { yyval.string = ""; }
 break;
 case 78:
-#line 413 "moc.y"
+#line 416 "moc.y"
 { yyval.string = straddSpc(yyvsp[-1].string,yyvsp[0].string);}
 break;
 case 79:
-#line 414 "moc.y"
+#line 417 "moc.y"
 { expLevel = 1; }
 break;
 case 80:
-#line 416 "moc.y"
+#line 419 "moc.y"
 { yyval.string = stradd( yyvsp[-4].string,"[",
 				     tmpExpression =
 				     tmpExpression.stripWhiteSpace(),"]" ); }
 break;
 case 81:
-#line 419 "moc.y"
+#line 422 "moc.y"
 { yyval.string = yyvsp[-1].string; }
 break;
 case 83:
-#line 430 "moc.y"
+#line 433 "moc.y"
 { tmpFunc->args	     = yyvsp[-3].arg_list;
 						  tmpFunc->qualifier = yyvsp[-1].string; }
 break;
 case 85:
-#line 436 "moc.y"
+#line 439 "moc.y"
 { func_warn("Variable as signal or slot."); }
 break;
 case 86:
-#line 437 "moc.y"
-{ expLevel=0; }
-break;
-case 87:
-#line 439 "moc.y"
-{ skipFunc = TRUE; }
-break;
-case 88:
 #line 440 "moc.y"
 { expLevel=0; }
 break;
-case 89:
+case 87:
 #line 442 "moc.y"
 { skipFunc = TRUE; }
 break;
+case 88:
+#line 443 "moc.y"
+{ expLevel=0; }
+break;
+case 89:
+#line 445 "moc.y"
+{ skipFunc = TRUE; }
+break;
 case 90:
-#line 446 "moc.y"
+#line 449 "moc.y"
 { expLevel = 1; }
 break;
 case 92:
-#line 448 "moc.y"
+#line 451 "moc.y"
 { expLevel = 1; }
 break;
 case 94:
-#line 453 "moc.y"
+#line 456 "moc.y"
 { yyval.string = ""; }
 break;
 case 95:
-#line 454 "moc.y"
-{ yyval.string = yyvsp[0].string; }
-break;
-case 96:
 #line 457 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
+case 96:
+#line 460 "moc.y"
+{ yyval.string = yyvsp[0].string; }
+break;
 case 97:
-#line 458 "moc.y"
+#line 461 "moc.y"
 { yyval.string = straddSpc(yyvsp[-1].string,yyvsp[0].string);}
 break;
 case 98:
-#line 461 "moc.y"
+#line 464 "moc.y"
 { yyval.string = straddSpc("*",yyvsp[0].string);}
 break;
 case 99:
-#line 462 "moc.y"
+#line 465 "moc.y"
 { yyval.string = stradd("&",yyvsp[0].string);}
 break;
 case 100:
-#line 469 "moc.y"
+#line 472 "moc.y"
 { yyval.string = ""; }
 break;
 case 101:
-#line 470 "moc.y"
-{ yyval.string = yyvsp[0].string; }
-break;
-case 102:
 #line 473 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
+case 102:
+#line 476 "moc.y"
+{ yyval.string = yyvsp[0].string; }
+break;
 case 103:
-#line 475 "moc.y"
+#line 478 "moc.y"
 { yyval.string = straddSpc(yyvsp[-1].string,yyvsp[0].string); }
 break;
 case 104:
-#line 478 "moc.y"
+#line 481 "moc.y"
 { yyval.string = "const"; }
 break;
 case 105:
-#line 479 "moc.y"
+#line 482 "moc.y"
 { yyval.string = "volatile"; }
 break;
 case 109:
-#line 487 "moc.y"
+#line 490 "moc.y"
 {BEGIN IN_FCT; fctLevel = 1;}
 break;
 case 110:
-#line 488 "moc.y"
+#line 491 "moc.y"
 {BEGIN QT_DEF; }
 break;
 case 111:
-#line 495 "moc.y"
+#line 498 "moc.y"
 { BEGIN IN_CLASS; level = 1; }
 break;
 case 112:
-#line 497 "moc.y"
+#line 500 "moc.y"
 { BEGIN QT_DEF; }
 break;
 case 113:
-#line 498 "moc.y"
+#line 501 "moc.y"
 { BEGIN QT_DEF;	  /* -- " -- */
 						  skipClass = TRUE; }
 break;
 case 114:
-#line 502 "moc.y"
+#line 505 "moc.y"
 { BEGIN QT_DEF; /* catch ';' */
 						  skipClass = TRUE; }
 break;
 case 115:
-#line 504 "moc.y"
+#line 507 "moc.y"
 { skipClass = TRUE;
 						  BEGIN GIMME_SEMICOLON; }
 break;
 case 121:
-#line 516 "moc.y"
+#line 519 "moc.y"
 { className = yyvsp[0].string; }
 break;
 case 122:
-#line 517 "moc.y"
+#line 520 "moc.y"
 { superclassName = yyvsp[0].string; }
 break;
 case 123:
-#line 520 "moc.y"
+#line 523 "moc.y"
 { yyval.string = 0; }
 break;
 case 124:
-#line 521 "moc.y"
+#line 524 "moc.y"
 { yyval.string = yyvsp[0].string; }
 break;
 case 129:
-#line 533 "moc.y"
+#line 536 "moc.y"
 { tmpAccessPerm = yyvsp[0].access; }
 break;
 case 130:
-#line 534 "moc.y"
+#line 537 "moc.y"
 { moc_err( "Missing access specifier"
 						   " before \"slots:\"." ); }
 break;
 case 131:
-#line 538 "moc.y"
+#line 541 "moc.y"
 { BEGIN QT_DEF; }
 break;
 case 133:
-#line 540 "moc.y"
+#line 543 "moc.y"
 { BEGIN QT_DEF; }
 break;
 case 135:
-#line 542 "moc.y"
+#line 545 "moc.y"
 { if ( tmpAccessPerm )
 				moc_warn("Q_OBJECT is not in the private"
 					" section of the class.\n"
@@ -2201,22 +2237,20 @@ case 135:
 						  Q_OBJECTdetected = TRUE; }
 break;
 case 136:
-#line 550 "moc.y"
+#line 553 "moc.y"
 { moc_err( "Signals cannot "
 						 "have access specifiers" ); }
 break;
 case 138:
-#line 553 "moc.y"
+#line 556 "moc.y"
 { if ( grammarDebug )
 						  BEGIN QT_DEF;
 					      else
 						  BEGIN IN_CLASS;
-					   if ( level != 1 )
-					       moc_warn( "unexpected ':'" );
 					 }
 break;
 case 140:
-#line 561 "moc.y"
+#line 562 "moc.y"
 { BEGIN IN_CLASS;
 					   if ( level != 1 )
 					       moc_warn( "unexpected access"
@@ -2224,84 +2258,84 @@ case 140:
 					 }
 break;
 case 145:
-#line 577 "moc.y"
+#line 578 "moc.y"
 { addMember('s'); }
 break;
 case 150:
-#line 588 "moc.y"
+#line 589 "moc.y"
 { addMember('t'); }
 break;
 case 153:
-#line 595 "moc.y"
+#line 596 "moc.y"
 { yyval.string=yyvsp[0].string; }
 break;
 case 156:
-#line 603 "moc.y"
+#line 604 "moc.y"
 { yyval.string = stradd( yyvsp[-3].string, "(", yyvsp[-1].string, ")" ); }
 break;
 case 157:
-#line 605 "moc.y"
+#line 606 "moc.y"
 { yyval.string = stradd( yyvsp[-3].string, "(", yyvsp[-1].string, ")" ); }
 break;
 case 158:
-#line 608 "moc.y"
-{yyval.string=yyvsp[0].string;}
-break;
-case 159:
 #line 609 "moc.y"
 {yyval.string=yyvsp[0].string;}
 break;
-case 160:
+case 159:
 #line 610 "moc.y"
 {yyval.string=yyvsp[0].string;}
 break;
-case 161:
+case 160:
 #line 611 "moc.y"
 {yyval.string=yyvsp[0].string;}
 break;
-case 162:
+case 161:
 #line 612 "moc.y"
 {yyval.string=yyvsp[0].string;}
 break;
-case 163:
+case 162:
 #line 613 "moc.y"
 {yyval.string=yyvsp[0].string;}
 break;
-case 164:
+case 163:
 #line 614 "moc.y"
 {yyval.string=yyvsp[0].string;}
 break;
-case 165:
+case 164:
 #line 615 "moc.y"
 {yyval.string=yyvsp[0].string;}
 break;
-case 166:
+case 165:
 #line 616 "moc.y"
 {yyval.string=yyvsp[0].string;}
 break;
-case 167:
+case 166:
 #line 617 "moc.y"
 {yyval.string=yyvsp[0].string;}
 break;
+case 167:
+#line 618 "moc.y"
+{yyval.string=yyvsp[0].string;}
+break;
 case 168:
-#line 620 "moc.y"
+#line 621 "moc.y"
 { yyval.access=_PRIVATE; }
 break;
 case 169:
-#line 621 "moc.y"
+#line 622 "moc.y"
 { yyval.access=_PROTECTED; }
 break;
 case 170:
-#line 622 "moc.y"
+#line 623 "moc.y"
 { yyval.access=_PUBLIC; }
 break;
 case 214:
-#line 674 "moc.y"
+#line 675 "moc.y"
 { tmpFunc->type = yyvsp[-1].string;
 						  tmpFunc->name = yyvsp[0].string; }
 break;
 case 215:
-#line 677 "moc.y"
+#line 678 "moc.y"
 { tmpFunc->type = "int";
 						  tmpFunc->name = yyvsp[0].string;
 				  if ( tmpFunc->name == className )
@@ -2310,14 +2344,14 @@ case 215:
 						}
 break;
 case 216:
-#line 684 "moc.y"
+#line 685 "moc.y"
 {
 				       func_warn( "Destructors cannot be"
 						  " signals or slots.");
 						}
 break;
 case 217:
-#line 690 "moc.y"
+#line 691 "moc.y"
 {   
 						    char *tmp = 
 							straddSpc(yyvsp[-4].string,yyvsp[-3].string,yyvsp[-2].string,yyvsp[-1].string);
@@ -2326,75 +2360,75 @@ case 217:
 						    tmpFunc->name = yyvsp[0].string; }
 break;
 case 218:
-#line 697 "moc.y"
+#line 698 "moc.y"
 { skipFunc = TRUE; }
 break;
 case 219:
-#line 699 "moc.y"
+#line 700 "moc.y"
 { tmpFunc->type =
 						      straddSpc(yyvsp[-2].string,yyvsp[-1].string);
 						  tmpFunc->name = yyvsp[0].string; }
 break;
 case 220:
-#line 704 "moc.y"
+#line 705 "moc.y"
 { tmpFunc->type =
 						      straddSpc(yyvsp[-3].string,yyvsp[-2].string,yyvsp[-1].string);
 						  tmpFunc->name = yyvsp[0].string; }
 break;
 case 221:
-#line 708 "moc.y"
+#line 709 "moc.y"
 { operatorError();    }
 break;
 case 222:
-#line 710 "moc.y"
+#line 711 "moc.y"
 { operatorError();    }
 break;
 case 223:
-#line 713 "moc.y"
+#line 714 "moc.y"
 { operatorError();    }
 break;
 case 224:
-#line 715 "moc.y"
+#line 716 "moc.y"
 { operatorError();    }
 break;
 case 225:
-#line 718 "moc.y"
+#line 719 "moc.y"
 { operatorError();    }
 break;
 case 227:
-#line 723 "moc.y"
+#line 724 "moc.y"
 { func_warn("Variable as signal or slot."); }
 break;
 case 228:
-#line 726 "moc.y"
+#line 727 "moc.y"
 { func_warn("Variable as signal or slot."); }
 break;
 case 229:
-#line 728 "moc.y"
+#line 729 "moc.y"
 { func_warn("Enum declaration as signal or"
 					  " slot."); }
 break;
 case 233:
-#line 738 "moc.y"
+#line 739 "moc.y"
 { expLevel = 0; }
 break;
 case 235:
-#line 740 "moc.y"
+#line 741 "moc.y"
 { expLevel = 0; }
 break;
 case 238:
-#line 745 "moc.y"
+#line 746 "moc.y"
 { expLevel = 0; }
 break;
 case 240:
-#line 750 "moc.y"
+#line 751 "moc.y"
 {BEGIN IN_FCT; fctLevel=1;}
 break;
 case 241:
-#line 751 "moc.y"
+#line 752 "moc.y"
 {BEGIN QT_DEF; }
 break;
-#line 2398 "y.tab.c"
+#line 2432 "y.tab.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
