@@ -1,27 +1,32 @@
 /****************************************************************************
-** $Id: qdir.cpp,v 2.3 1997/03/30 15:22:50 hanord Exp $
+** $Id: qdir.cpp,v 2.25 1998/07/03 00:09:43 hanord Exp $
 **
 ** Implementation of QDir class
 **
 ** Created : 950427
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qdir.h"
-#include "qfileinf.h"
-#include "qfiledef.h"
+#include "qfileinfo.h"
+#include "qfiledefs.h"
 #include "qregexp.h"
 #include <stdlib.h>
+#include <ctype.h>
 #if defined(_OS_WIN32_)
 #if defined(_CC_BOOL_DEF_)
 #undef	bool
@@ -31,17 +36,10 @@
 #include <windows.h>
 #endif
 #endif
-#if defined(_OS_OS2EMX_)
-#include <ctype.h>
-#define chdir _chdir2
-#endif
 
-RCSTAG("$Id: qdir.cpp,v 2.3 1997/03/30 15:22:50 hanord Exp $");
+#if defined(_OS_FATFS_) || defined(_OS_OS2EMX_)
 
-
-#if defined(_OS_FATFS_)
-
-static void convertSeparators( char *n )
+static void slashify( char *n )
 {
     if ( !n )
 	return;
@@ -54,7 +52,7 @@ static void convertSeparators( char *n )
 
 #elif defined(UNIX)
 
-static void convertSeparators( char * )
+static void slashify( char * )
 {
     return;
 }
@@ -64,10 +62,10 @@ static void convertSeparators( char * )
 
 /*!
   \class QDir qdir.h
-  \brief QDir can traverse the directory structure and read the contents of
-	 directories in a platform-independent way.
+  \brief Traverses directory structures and contents in a
+	    platform-independent way.
 
-  \ingroup tools
+  \ingroup io
 
   A QDir can point to a file using either a relative or an absolute file
   path. Absolute file paths begin with the directory separator ('/') or a
@@ -273,7 +271,8 @@ void QDir::setPath( const char *path )
 
   The returned path can be either absolute or relative (see setPath()).
 
-  \sa setPath(), absPath(), exists(), cleanDirPath(), dirName(), absFilePath()
+  \sa setPath(), absPath(), exists(), cleanDirPath(), dirName(),
+  absFilePath(), convertSeparators()
 */
 
 /*!
@@ -292,9 +291,9 @@ QString QDir::absPath() const
 	if ( tmp.right(1) != "/" )
 	    tmp += '/';
 	tmp += dPath;
-	return cleanDirPath( tmp.data() );
+	return cleanDirPath( tmp );
     } else {
-	return dPath.copy();
+	return cleanDirPath( dPath );
     }
 }
 
@@ -315,9 +314,9 @@ QString QDir::canonicalPath() const
     QString tmp( PATH_MAX );
 
     GETCWD( cur.data(), PATH_MAX );
-    if ( chdir( dPath ) >= 0 )
+    if ( CHDIR(dPath) >= 0 )
 	GETCWD( tmp.data(), PATH_MAX );
-    chdir( cur );
+    CHDIR( cur );
 
     return tmp;
 }
@@ -398,6 +397,32 @@ QString QDir::absFilePath( const char *fileName,
     return tmp;
 }
 
+
+/*!
+  Converts the '/' separators in \a pathName to system native
+  separators.  Returns the translated string.
+
+  On Windows, convertSeparators("c:/winnt/system32") returns
+  "c:\winnt\system32".
+
+  No conversion is done on UNIX.
+*/
+
+QString QDir::convertSeparators( const char *pathName )
+{
+    QString n( pathName );
+#if defined(_OS_FATFS_) || defined(_OS_OS2EMX_)
+    char *p = n.data();
+    while ( p && *p ) {
+	if ( *p == '/' )
+	    *p = '\\';
+	p++;
+    }
+#endif
+    return n;
+}
+
+
 /*!
   Changes directory by descending into the given directory. Returns
   TRUE if the new directory exists and is readable. Note that the logical
@@ -750,7 +775,7 @@ const QFileInfoList *QDir::entryInfoList( const char *nameFilter,
 
 bool QDir::mkdir( const char *dirName, bool acceptAbsPath ) const
 {
-#if defined (UNIX)
+#if defined (UNIX) || defined(__CYGWIN32__)
     return MKDIR( filePath(dirName,acceptAbsPath), 0777 ) == 0;
 #else
     return MKDIR( filePath(dirName,acceptAbsPath) ) == 0;
@@ -796,9 +821,9 @@ bool QDir::isReadable() const
 
 /*!
   Returns TRUE if the directory exists. (If a file with the same
-   name is found this function will of course return FALSE).
+  name is found this function will of course return FALSE).
 
-   \sa QFileInfo::exists(), QFile::exists()
+  \sa QFileInfo::exists(), QFile::exists()
 */
 
 bool QDir::exists() const
@@ -816,7 +841,7 @@ bool QDir::exists() const
 
   Example:
   \code
-    QDir d( "/yo/root_link" );
+    QDir d( "/tmp/root_link" );
     d = d.canonicalPath();
     if ( d.isRoot() )
 	warning( "It IS a root link!" );
@@ -827,7 +852,12 @@ bool QDir::exists() const
 
 bool QDir::isRoot() const
 {
+#if defined(_OS_FATFS_) || defined(_OS_OS2EMX_)
+    return dPath == "/" || dPath == "//" ||
+	(isalpha(dPath[0]) && dPath.mid(1,dPath.length()) == ":/");
+#else
     return dPath == "/";
+#endif
 }
 
 /*!
@@ -925,12 +955,8 @@ bool QDir::remove( const char *fileName, bool acceptAbsPath )
 #endif
 	return FALSE;
     }
-    QString tmp = filePath( fileName, acceptAbsPath );
-#if defined(UNIX)
-    return unlink( tmp ) == 0;			// unlink more common in UNIX
-#else
-    return ::remove( tmp ) == 0;		// use standard ANSI remove
-#endif
+    QString p = filePath( fileName, acceptAbsPath );
+    return QFile::remove( p );
 }
 
 /*!
@@ -941,6 +967,12 @@ bool QDir::remove( const char *fileName, bool acceptAbsPath )
   any number of separators at the beginning of \e name will be removed.
 
   Returns TRUE if successful, otherwise FALSE.
+
+  On most file systems, rename() fails only if oldName does not exist
+  or if \a newName and \a oldName are not on the same partition, but
+  there are also other reasons why rename() can fail.  For example, on
+  at least one file system rename() fails if newName points to an open
+  file
 */
 
 bool QDir::rename( const char *name, const char *newName,
@@ -1010,7 +1042,7 @@ char QDir::separator()
 
 bool QDir::setCurrent( const char *path )
 {
-    if ( chdir( path ) >= 0 )
+    if ( CHDIR(path) >= 0 )
 	return TRUE;
     else
 	return FALSE;
@@ -1038,7 +1070,7 @@ QDir QDir::home()
 
 /*!
   Returns the root directory.
-  \sa rootDirPath()
+  \sa rootDirPath() drives()
 */
 
 QDir QDir::root()
@@ -1057,7 +1089,7 @@ QString QDir::currentDirPath()
     static bool forcecwd = TRUE;
     static ino_t cINode;
     static dev_t cDevice;
-    static QString currentName( PATH_MAX );
+    QString currentName( PATH_MAX );
 
     STATBUF st;
 
@@ -1066,7 +1098,7 @@ QString QDir::currentDirPath()
 	    if ( GETCWD( currentName.data(), PATH_MAX ) != 0 ) {
 		cINode	 = st.st_ino;
 		cDevice	 = st.st_dev;
-		convertSeparators( currentName.data() );
+		slashify( currentName.data() );
 		// forcecwd = FALSE;   ### caching removed, not safe
 	    } else {
 		warning( "QDir::currentDirPath: getcwd() failed" );
@@ -1093,6 +1125,7 @@ QString QDir::homeDirPath()
 {
     QString d( PATH_MAX );
     d = getenv("HOME");
+    slashify( d.data() );
     if ( d.isNull() )
 	d = rootDirPath();
     return d;
@@ -1100,7 +1133,8 @@ QString QDir::homeDirPath()
 
 /*!
   Returns the absolute path for the root directory ("/" under UNIX).
-  \sa root()
+
+  \sa root() drives()
 */
 
 QString QDir::rootDirPath()
@@ -1147,10 +1181,10 @@ QString QDir::cleanDirPath( const char *filePath )
     if ( name.isEmpty() )
 	return name;
 
-    convertSeparators( name.data() );
+    slashify( name.data() );
 
     bool addedSeparator;
-    if ( isRelativePath(filePath) ) {
+    if ( isRelativePath(name) ) {
 	addedSeparator = TRUE;
 	name.insert( 0, '/' );
     } else {
@@ -1163,15 +1197,15 @@ QString QDir::cleanDirPath( const char *filePath )
     upLevel = 0;
     int len;
 
-    while( pos && (pos = name.findRev( '/', --pos)) != -1 ) {
+    while ( pos && (pos = name.findRev('/',--pos)) != -1 ) {
 	len = ePos - pos - 1;
-	if ( len == 2 && name.at( pos + 1 ) == '.'
-		      && name.at( pos + 2 ) == '.' ) {
+	if ( len == 2 && name.at(pos + 1) == '.'
+		      && name.at(pos + 2) == '.' ) {
 	    upLevel++;
 	} else {
-	    if ( len != 0 && ( len != 1 || name.at( pos + 1 ) != '.' ) ) {
+	    if ( len != 0 && (len != 1 || name.at(pos + 1) != '.') ) {
 		if ( !upLevel )
-		    newPath = "/" + name.mid( pos + 1, len) + newPath;
+		    newPath = "/" + name.mid(pos + 1, len) + newPath;
 		else
 		    upLevel--;
 	    }
@@ -1179,7 +1213,7 @@ QString QDir::cleanDirPath( const char *filePath )
 	ePos = pos;
     }
     if ( addedSeparator ) {
-	while( upLevel-- )
+	while ( upLevel-- )
 	    newPath.insert( 0, "/.." );
 	if ( !newPath.isEmpty() )
 	    newPath.remove( 0, 1 );
@@ -1188,6 +1222,14 @@ QString QDir::cleanDirPath( const char *filePath )
     } else {
 	if ( newPath.isEmpty() )
 	    newPath = "/";
+#if defined(_OS_FATFS_) || defined(_OS_OS2EMX_)
+	if ( name[0] == '/' ) {
+	    if ( name[1] == '/' )		// "\\machine\x\ ..."
+		newPath.insert( 0, '/' );
+	} else {
+	    newPath = name.left(2) + newPath;
+	}
+#endif
     }
     return newPath;
 }
@@ -1301,6 +1343,9 @@ bool QDir::readDirEntries( const QString &nameFilter,
     bool doExecable = (filterSpec & Executable) != 0;
     bool doHidden   = (filterSpec & Hidden)	!= 0;
 #if !defined(UNIX)
+    // show hidden files if the user asks explicitly for e.g. .*
+    if ( !doHidden && !nameFilter.isEmpty() && nameFilter[0] == '.' )
+	doHidden = TRUE;
     bool doModified = (filterSpec & Modified)	!= 0;
     bool doSystem   = (filterSpec & System)	!= 0;
 #endif
@@ -1479,9 +1524,6 @@ bool QDir::readDirEntries( const QString &nameFilter,
 	return FALSE;
     }
 
-    if ( !nameFilter.isEmpty() && nameFilter[0] == '.' )
-	filterSpec |= Hidden;
-
     while ( (file = readdir(dir)) ) {
 	fi.setFile( *this, file->d_name );
 	if ( wc.match(file->d_name) == -1 && !(allDirs && fi.isDir()) )
@@ -1531,3 +1573,42 @@ bool QDir::readDirEntries( const QString &nameFilter,
 	dirty = TRUE;
     return TRUE;
 }
+
+
+/*!  Returns a list if the root directories on this system.  On
+  win32, this returns a number of QFileInfo objects containing "C:/",
+  "D:/" etc.  On other operating systems, it returns a list containing
+  just one root directory (e.g. "/").
+*/
+
+const QFileInfoList * QDir::drives()
+{
+    // at most one instance of QFileInfoList is leaked, and this variable
+    // points to that list
+    static QFileInfoList * knownMemoryLeak = 0;
+
+    if ( !knownMemoryLeak ) {
+	knownMemoryLeak = new QFileInfoList;
+
+#if defined(_OS_WIN32_)
+
+	Q_UINT32 driveBits = (Q_UINT32) GetLogicalDrives() & 0x3ffffff;
+	char driveName[4];
+	qstrcpy( driveName, "a:/" );
+	while( driveBits ) {
+	    if ( driveBits & 1 )
+		knownMemoryLeak->append( new QFileInfo( driveName ) );
+	    driveName[0]++;
+	    driveBits = driveBits >> 1;
+	}
+
+#else
+
+	// non-win32 versions both use just one root directory
+	knownMemoryLeak->append( new QFileInfo( rootDirPath() ) );
+
+#endif
+    }
+
+    return knownMemoryLeak;
+}
