@@ -1,19 +1,23 @@
 /****************************************************************************
-** $Id: qgarray.cpp,v 2.2 1997/01/26 19:38:46 agulbra Exp $
+** $Id: qgarray.cpp,v 2.7 1998/07/03 00:09:44 hanord Exp $
 **
 ** Implementation of QGArray class
 **
 ** Created : 930906
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
@@ -22,9 +26,6 @@
 #include "qstring.h"
 #include <stdlib.h>
 
-RCSTAG("$Id: qgarray.cpp,v 2.2 1997/01/26 19:38:46 agulbra Exp $");
-
-
 #define USE_MALLOC				// comment to use new/delete
 
 #undef NEW
@@ -425,7 +426,7 @@ void QGArray::store( const char *d, uint len )
   Setting raw data is useful because it set QArray data without allocating
   memory or copying data.
 
-  Example I (intended use):
+  Example of intended use:
   \code
     static uchar bindata[] = { 231, 1, 44, ... };
     QByteArray	a;
@@ -436,7 +437,7 @@ void QGArray::store( const char *d, uint len )
     a.resetRawData( bindata, sizeof(bindata) ); // finished
   \endcode
 
-  Example II (you don't want to do this):
+  Example of misuse (do not do this):
   \code
     static uchar bindata[] = { 231, 1, 44, ... };
     QByteArray	a, b;
@@ -454,8 +455,7 @@ void QGArray::store( const char *d, uint len )
 
 QGArray &QGArray::setRawData( const char *d, uint len )
 {
-    if ( shd->data )
-	duplicate( 0, 0 );			// set null data
+    duplicate( 0, 0 );				// set null data
     shd->data = (char *)d;
     shd->len  = len;
     return *this;
