@@ -1,31 +1,32 @@
 /****************************************************************************
-** $Id: qgdict.cpp,v 2.5 1997/01/30 11:38:05 hanord Exp $
+** $Id: qgdict.cpp,v 2.18 1998/07/07 18:02:31 hanord Exp $
 **
 ** Implementation of QGDict and QGDictIterator classes
 **
 ** Created : 920529
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qgdict.h"
 #include "qlist.h"
 #include "qstring.h"
-#include "qdstream.h"
+#include "qdatastream.h"
 #include <ctype.h>
 
-RCSTAG("$Id: qgdict.cpp,v 2.5 1997/01/30 11:38:05 hanord Exp $");
-
-
 /*!
   \class QGDict qgdict.h
   \brief The QGDict class is an internal class for implementing QDict and QIntDict.
@@ -43,6 +44,9 @@ RCSTAG("$Id: qgdict.cpp,v 2.5 1997/01/30 11:38:05 hanord Exp $");
   Normally, you do not have to reimplement any of these functions.
 */
 
+static const int op_find = 0;
+static const int op_insert = 1;
+static const int op_replace = 2;
 
 Q_DECLARE(QListM,QGDictIterator);		// list of iterators: QGDItList
 
@@ -147,16 +151,23 @@ private:
 
 QGDict::QGDict( uint len, bool cs, bool ck, bool th )
 {
-    vec = new QBucket *[vlen = len];		// allocate hash table
-    CHECK_PTR( vec );
-    memset( (char*)vec, 0, vlen*sizeof(QBucket*) );
-    numItems = 0;
+    init( len );
     cases = cs;
     copyk = ck;
     triv = th;
     if ( triv )					// copyk must be FALSE for
 	copyk = FALSE;				//   int-hashed dicts
+}
+
+void QGDict::init( uint len )
+{
+    vec = new QBucket *[vlen = len];		// allocate hash table
+    CHECK_PTR( vec );
+    memset( (char*)vec, 0, vlen*sizeof(QBucket*) );
+    numItems = 0;
     iterators = 0;
+
+    // Be careful not to break resize() if you add something.
 }
 
 /*!
@@ -165,18 +176,15 @@ QGDict::QGDict( uint len, bool cs, bool ck, bool th )
 */
 
 QGDict::QGDict( const QGDict & dict )
+    : QCollection( dict )
 {
-    vec = new QBucket *[vlen = dict.vlen];	// allocate hash table
-    CHECK_PTR( vec );
-    memset( (char*)vec, 0, vlen*sizeof(QBucket*) );
-    numItems = 0;
+    init( dict.vlen );
     cases = dict.cases;
     copyk = dict.copyk;
     triv  = dict.triv;
-    iterators = 0;
     QGDictIterator it( dict );
     while ( it.get() ) {			// copy from other dict
-	look( it.getKey(), it.get(), TRUE );
+	look( it.getKey(), it.get(), op_insert );
 	++it;
     }
 }
@@ -211,7 +219,7 @@ QGDict &QGDict::operator=( const QGDict &dict )
     clear();
     QGDictIterator it( dict );
     while ( it.get() ) {			// copy from other dict
-	look( it.getKey(), it.get(), TRUE );
+	look( it.getKey(), it.get(), op_insert );
 	++it;
     }
     return *this;
@@ -233,7 +241,7 @@ QGDict &QGDict::operator=( const QGDict &dict )
 
 /*!
   \internal
-  The do-it-all function; find (op==0), insert (op==1), replace (op==2)
+  The do-it-all function; op is one of op_find, op_insert, op_replace
 */
 
 GCI QGDict::look( const char *key, GCI d, int op )
@@ -242,7 +250,7 @@ GCI QGDict::look( const char *key, GCI d, int op )
     int	 index;
     if ( triv ) {				// key is a long/ptr
 	index = (int)(long(key) % vlen);	// simple hash
-	if ( op == 0 ) {			// find
+	if ( op == op_find ) {			// find
 	    for ( n=vec[index]; n; n=n->getNext() ) {
 		if ( n->getKey() == key )
 		    return n->getData();	// item found
@@ -251,7 +259,7 @@ GCI QGDict::look( const char *key, GCI d, int op )
 	}
     } else {					// key is a string
 	index = hashKey( key ) % vlen;
-	if ( op == 0 ) {			// find
+	if ( op == op_find ) {			// find
 	    for ( n=vec[index]; n; n=n->getNext() ) {
 		if ( (cases ? strcmp(n->getKey(),key)
 			 : stricmp(n->getKey(),key)) == 0 )
@@ -260,7 +268,7 @@ GCI QGDict::look( const char *key, GCI d, int op )
 	    return 0;				// did not find the item
 	}
     }
-    if ( op == 2 ) {				// replace
+    if ( op == op_replace ) {			// replace
 	if ( vec[index] != 0 )			// maybe something there
 	    remove( key );
     }
@@ -280,6 +288,49 @@ GCI QGDict::look( const char *key, GCI d, int op )
     return node->getData();
 }
 
+/*!
+  \internal
+  Changes the size of the hashtable.
+  The contents of the dictionary are preserved,
+  but all iterators on the dictionary become invalid.
+*/
+void QGDict::resize( uint newsize )
+{
+    // Save old information
+    QBucket   **old_vec = vec;
+    uint	old_vlen = vlen;
+    QGDItList  *old_iterators = iterators;
+    bool	old_copyk = copyk;
+
+    init( newsize );
+    copyk = FALSE;
+
+    // Reinsert every item from vec, deleting vec as we go
+    for ( uint index = 0; index < old_vlen; index++ ) {
+	QBucket *n=old_vec[index];
+	while ( n ) {
+	    look( n->getKey(), n->getData(), op_insert );
+	    QBucket *t=n->getNext();
+	    delete n;
+	    n = t;
+	}
+    }
+    delete [] old_vec;
+
+    // Restore state
+    iterators = old_iterators;
+    copyk = old_copyk;
+
+    // `Invalidate' all iterators, since order is lost
+    if ( iterators ) {			// update iterators
+	register QGDictIterator *i = iterators->first();
+	while ( i ) {			// fix all iterators
+	    i->toFirst();
+	    i = iterators->next();
+	}
+    }
+}
+
 /*!
   \internal
   Unlinks the bucket with the specified key.
@@ -334,7 +385,7 @@ bool QGDict::remove( const char *key )
     register QBucket *n = unlink( key );
     if ( n ) {
 	if ( copyk )
-	    delete n->getKey();
+	    delete [] n->getKey();
 	deleteItem( n->getData() );
 	delete n;				// delete bucket
     }
@@ -349,12 +400,14 @@ bool QGDict::remove( const char *key )
 GCI QGDict::take( const char *key )
 {
     register QBucket *n = unlink( key );
-    GCI tmp = 0;
+    GCI tmp;
     if ( n ) {
 	tmp = n->getData();
 	if ( copyk )
-	    delete n->getKey();
+	    delete [] n->getKey();
 	delete n;
+    } else {
+	tmp = 0;
     }
     return tmp;
 }
@@ -372,16 +425,18 @@ void QGDict::clear()
     register QBucket *n;
     numItems = 0;				// disable remove() function
     for ( uint j=0; j<vlen; j++ ) {		// destroy hash table
-	n = vec[j];
-	while ( n ) {
-	    if ( copyk )
-		delete n->getKey();
-	    deleteItem( n->getData() );
-	    QBucket *next = n->getNext();
-	    delete n;
-	    n = next;
+	if ( vec[j] ) {
+	    n = vec[j];
+	    vec[j] = 0;				// detach list of buckets
+	    while ( n ) {
+		if ( copyk )
+		    delete [] n->getKey();
+		QBucket *next = n->getNext();
+		deleteItem( n->getData() );
+		delete n;
+		n = next;
+	    }
 	}
-	vec[j] = 0;
     }
     if ( !iterators )				// no iterators for this dict
 	return;
@@ -481,7 +536,7 @@ QDataStream &QGDict::read( QDataStream &s )
 	    s >> k;				// key is string
 	}
 	read( s, d );				// read data
-	look( k, d, TRUE );
+	look( k, d, op_insert );
     }
     return s;
 }
@@ -499,7 +554,7 @@ QDataStream& QGDict::write( QDataStream &s ) const
 	QBucket *n = vec[i];
 	while ( n ) {				// write all buckets
 	    if ( triv )
-		s << (Q_UINT32)n->getKey();	// write key as 32-bit int
+		s << (Q_UINT32)(long)n->getKey(); // write key as 32-bit int
 	    else
 		s << n->getKey();		// write key as string
 	    write( s, n->getData() );		// write data
@@ -517,7 +572,7 @@ QDataStream& QGDict::write( QDataStream &s ) const
 
 /*!
   \class QGDictIterator qgdict.h
-  \brief The QGDictIterator is an internal class for implementing QDictIterator and QIntDictIterator.
+  \brief An internal class for implementing QDictIterator and QIntDictIterator.
 
   QGDictIterator is a strictly internal class that does the heavy work for
   QDictIterator and QIntDictIterator.
