@@ -1,29 +1,32 @@
 /****************************************************************************
-** $Id: qgeneric.h,v 2.10 1997/08/06 15:05:52 hanord Exp $
+** $Id: qgeneric.h,v 2.12 1998/07/03 00:09:45 hanord Exp $
 **
 ** Macros for pasting tokens; utilized by our generic classes
 **
 ** Created : 920529
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QGENERIC_H
 #define QGENERIC_H
 
+#ifndef QT_H
 #include "qglobal.h"
+#endif // QT_H
 
 
 // Try to include the system defines where it is sure to exist
