@@ -1,22 +1,23 @@
 /****************************************************************************
-** $Id: qgvector.cpp,v 2.4 1997/09/12 15:32:04 hanord Exp $
+** $Id: qgvector.cpp,v 2.8 1998/07/03 00:09:46 hanord Exp $
 **
 ** Implementation of QGVector class
 **
 ** Created : 930907
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
@@ -24,12 +25,9 @@
 #include "qgvector.h"
 #include "qglist.h"
 #include "qstring.h"
-#include "qdstream.h"
+#include "qdatastream.h"
 #include <stdlib.h>
 
-RCSTAG("$Id: qgvector.cpp,v 2.4 1997/09/12 15:32:04 hanord Exp $");
-
-
 #define USE_MALLOC				// comment to use new/delete
 
 #undef NEW
@@ -90,6 +88,7 @@ QGVector::QGVector( uint size )			// create vectors with nullptrs
 }
 
 QGVector::QGVector( const QGVector &a )		// make copy of other vector
+    : QCollection( a )
 {
     len = a.len;
     numItems = a.numItems;
