@@ -1,22 +1,23 @@
 /****************************************************************************
-** $Id: qregexp.cpp,v 2.4 1997/08/06 15:05:54 hanord Exp $
+** $Id: qregexp.cpp,v 2.12 1998/07/03 00:09:46 hanord Exp $
 **
 ** Implementation of QRegExp class
 **
 ** Created : 950126
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
@@ -24,12 +25,10 @@
 #include <ctype.h>
 #include <stdlib.h>
 
-RCSTAG("$Id: qregexp.cpp,v 2.4 1997/08/06 15:05:54 hanord Exp $");
-
-
 /*!
   \class QRegExp qregexp.h
   \ingroup tools
+  \ingroup misc
   \brief The QRegExp class provides pattern matching using regular expressions and wildcards.
 
   QRegExp knows these regexp primitives:
@@ -48,17 +47,17 @@ RCSTAG("$Id: qregexp.cpp,v 2.4 1997/08/06 15:05:54 hanord Exp $");
   <li><dfn>\n</dfn> matches newline (10)
   <li><dfn>\r</dfn> matches return (13)
   <li><dfn>\s</dfn> matches white space (9,10,11,12,13,32)
-  <li><dfn>\x12</dfn> matches the character hex 12.
-  <li><dfn>\022</dfn> matches the character octal 22.
+  <li><dfn>\x12</dfn> matches the character 0x12 (18 decimal, 12 hexadecimal).
+  <li><dfn>\022</dfn> matches the character 022 (18 decimal, 22 octal).
   </ul>
 
   In wildcard mode, it only knows three primitives:
   <ul plain>
   <li><dfn>?</dfn> matches any character
   <li><dfn>*</dfn> matches any sequence of characters
-  <li><dfn>[]</dfn> matches a defines set of characters,
+  <li><dfn>[]</dfn> matches a defined set of characters,
     e.g. [a-zA-Z0-9\.] matches upper and lower case ASCII letters, digits,
-    and dot.
+    and dot, and [^z] matches everything except lower-case z.
   </ul>
 */
 
@@ -148,7 +147,8 @@ QRegExp::QRegExp( const QRegExp &r )
 
 QRegExp::~QRegExp()
 {
-    delete [] rxdata;
+    if ( rxdata )                      // Avoid purify complaints
+	delete [] rxdata;
 }
 
 /*!
@@ -340,8 +340,9 @@ char *QRegExp::matchstr( ushort *rxd, char *str, char *bol ) const
     while ( *d ) {
 	if ( *d & CHR ) {			// match char
 	    if ( cs ) {				// case sensitive
-		if ( *p++ != (char)*d )
+		if ( *p != (char)*d )
 		    return 0;
+		p++;
 	    } else {				// case insensitive
 		if ( tolower(*p) != (char)*d )
 		    return 0;
@@ -354,7 +355,7 @@ char *QRegExp::matchstr( ushort *rxd, char *str, char *bol ) const
 		    return 0;
 		break;
 	    case CCL:				// match char class
-		if ( (d[*p >> 4] & (1 << (*p & 0xf))) == 0 )
+		if ( (d[(uchar)*p >> 4] & (1 << ((uchar)*p & 0xf))) == 0 )
 		    return 0;
 		p++;
 		d += 16;
@@ -394,7 +395,8 @@ char *QRegExp::matchstr( ushort *rxd, char *str, char *bol ) const
 			break;
 		    case CCL:
 			d++;
-			while ( *p && d[*p >> 4] & (1 << (*p & 0xf)) )
+			while ( *p && d[(uchar)*p >> 4] &
+				(1 << ((uchar)*p & 0xf)) )
 			    p++;
 			d += 15;
 			break;
@@ -430,7 +432,8 @@ char *QRegExp::matchstr( ushort *rxd, char *str, char *bol ) const
 			break;
 		    case CCL:
 			d++;
-			if ( *p && d[*p >> 4] & (1 << (*p & 0xf)) )
+			if ( *p &&
+			     d[(uchar)*p >> 4] & (1 << ((uchar)*p & 0xf)) )
 			    p++;
 			d += 15;
 			break;
@@ -467,20 +470,25 @@ static QString wc2rx( const char *pattern )
     char c;
     while ( (c=*p++) ) {
 	switch ( c ) {
-	    case '*':				// '*' ==> '.*'
-		wcpattern += '.';
-		break;
-	    case '?':				// '?' ==> '.'
-		c = '.';
-		break;
-	    case '.':				// quote special regexp chars
-	    case '+':
-	    case '\\':
-	    case '^':
-	    case '$':
-	    case '[':
-		wcpattern += '\\';
-		break;
+	case '*':				// '*' ==> '.*'
+	    wcpattern += '.';
+	    break;
+	case '?':				// '?' ==> '.'
+	    c = '.';
+	    break;
+	case '.':				// quote special regexp chars
+	case '+':
+	case '\\':
+	case '$':
+	case '^':
+	    wcpattern += '\\';
+	    break;
+	case '[':
+	    if ( *p == '^' ) {
+		wcpattern += '[';
+		c = *p++;
+	    }
+	    break;
 	}
 	wcpattern += c;
     }
@@ -545,9 +553,9 @@ static int char_val( char **str )		// get char value
 		}
 	    }
 	}
+    } else {
+	v = (uchar)*p;
     }
-    else
-	v = *p;
     *str += len;
     return v;
 }
@@ -675,18 +683,18 @@ void QRegExp::compile()
 		if ( *p == '^' ) {		// negate!
 		    neg = 1;
 		    p++;
-		}
-		else
+		} else {
 		    neg = 0;
+		}
 		if ( *p == ']' )		// bracket, not end
-		    cc[(unsigned char)(*p++)] = 1;
+		    cc[(uchar)(*p++)] = 1;
 		int prev_c = -1;
 		while ( *p && *p != ']' ) {	// scan the char set
 		    if ( *p == '-' && *(p+1) && *(p+1) != ']' ) {
 			p++;			// range!
-			if ( prev_c == -1 )	// no previous char
-			    cc[(unsigned char)'-'] = 1;
-			else {
+			if ( prev_c == -1 ) {	// no previous char
+			    cc[(uchar)'-'] = 1;
+			} else {
 			    int start = prev_c;
 			    int stop = char_val( &p );
 			    if ( start > stop ) { // swap start and stop
@@ -697,9 +705,9 @@ void QRegExp::compile()
 			    while ( start++ < stop )
 				cc[start] = 1;
 			}
-		    }
-		    else			// normal char
+		    } else {			// normal char
 			cc[(prev_c=char_val(&p))] = 1;
+		    }
 		}
 		if ( *p != ']' ) {		// missing close bracket
 		    error = PatSyntax;
@@ -776,8 +784,7 @@ void QRegExp::compile()
 			p++;
 			p++;
 			break;
-		    }
-		    else if ( *(p+1) == '<' || *(p+1) == '>' ) {
+		    } else if ( *(p+1) == '<' || *(p+1) == '>' ) {
 			GEN( *++p == '<' ? BOW : EOW );
 			p++;
 			break;
