@@ -1,34 +1,35 @@
 /****************************************************************************
-** $Id: qstring.cpp,v 2.5 1997/03/11 12:00:48 paul Exp $
+** $Id: qstring.cpp,v 2.22 1998/07/03 00:09:47 hanord Exp $
 **
 ** Implementation of extended char array operations, and QByteArray and
 ** QString classes
 **
 ** Created : 920722
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #define	 QSTRING_C
 #include "qstring.h"
-#include "qdstream.h"
+#include "qdatastream.h"
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <ctype.h>
 
-RCSTAG("$Id: qstring.cpp,v 2.5 1997/03/11 12:00:48 paul Exp $");
-
-
 /*****************************************************************************
   Safe and portable C string functions; extensions to standard string.h
  *****************************************************************************/
@@ -323,6 +324,32 @@ UINT16 qchecksum( const char *data, uint len )
     return (~crc & 0xffff);
 }
 
+//#define QSTRING_COMPATIBILITY 200
+
+#ifndef QSTRING_COMPATIBILITY
+#define QSTRING_COMPATIBILITY QT_VERSION
+#endif
+
+#if QSTRING_COMPATIBILITY < 200
+//
+// Explicitly-shared QString (ie. QString before Qt 2.0):
+//
+# define QSTRING_DETACH(s)
+#else
+# if QSTRING_COMPATIBILITY == 200
+//
+// Implicitly-shared QString, with warnings on potentially buggy old calls.
+//
+#  define QSTRING_DETACH(s) { if ((s)->nrefs()>1) warning("Changed behavior of QString"); (s)->detach(); }
+# else
+//
+// Implicitly-shared QString, with warnings turned off, once it is safe
+// to rely on the new QString behavior.
+//
+#  define QSTRING_DETACH(s) (s)->detach();
+# endif
+#endif
+
 
 /*****************************************************************************
   QByteArray member functions
@@ -366,6 +393,10 @@ QDataStream &operator>>( QDataStream &s, QByteArray &a )
 {
     Q_UINT32 len;
     s >> len;					// read size of array
+    if ( len == 0 || s.eof() ) {		// end of file reached
+	a.resize( 0 );
+	return s;
+    }
     if ( !a.resize( (uint)len ) ) {		// resize array
 #if defined(CHECK_NULL)
 	warning( "QDataStream: Not enough memory to read QByteArray" );
@@ -433,16 +464,21 @@ QDataStream &operator>>( QDataStream &s, QByteArray &a )
   \sa resize(), isNull()
 */
 
-QString::QString( int size ) : QByteArray( size )
+QString::QString( int size )
+    : QByteArray( size )
 {
     if ( size > 0 ) {
 	*data() = '\0';				// set terminator
-	*(data()+size-1) = '\0';
+	*(data()+(size-1)) = '\0';
     }
 }
 
 /*!
   Constructs a string that is a deep copy of \e str.
+
+  If \a str is 0 a null string is created.
+
+  \sa isNull()
 */
 
 QString::QString( const char *str )
@@ -451,6 +487,33 @@ QString::QString( const char *str )
 }
 
 
+/*!
+  Constructs a string that is a deep copy of \e str, that is no more
+  than \a maxlen bytes long including the '\0'-terminator.
+
+  Example:
+  \code
+    QString str("helloworld",6); // Assigns "hello" to str.
+  \endcode
+
+  If \a str contains a 0 byte within the first \a maxlen bytes, the
+  resulting QString will be terminated by the 0.  If \a str is 0 a
+  null string is created.
+
+  \sa isNull()
+*/
+
+QString::QString( const char *str, uint maxlen )
+{
+    uint len; // index of last nul character
+    // cannot use memchr - we need the *index*, not void*
+    for (len=0; len<maxlen-1; len++) {
+	if (!str[len]) break;
+    }
+    duplicate( str, len+1 );
+    data()[len] = 0;
+}
+
 /*!
   \fn QString &QString::operator=( const QString &s )
   Assigns a shallow copy of \e s to this string and returns a reference to
@@ -459,8 +522,12 @@ QString::QString( const char *str )
 
 /*!
   \fn QString &QString::operator=( const char *str )
-  Assigns a deep copy of \e str to this string and returns a reference to
+  Assigns a deep copy of \a str to this string and returns a reference to
   this string.
+
+  If \a str is 0 a null string is created.
+
+  \sa isNull()
 */
 
 /*!
@@ -535,6 +602,7 @@ QString::QString( const char *str )
 
 bool QString::resize( uint len )
 {
+    QSTRING_DETACH(this)
     if ( !QByteArray::resize(len) )
 	return FALSE;
     if ( len )
@@ -573,6 +641,7 @@ bool QString::resize( uint len )
 
 QString &QString::sprintf( const char *format, ... )
 {
+    QSTRING_DETACH(this)
     va_list ap;
     va_start( ap, format );
     if ( size() < 256 )
@@ -596,6 +665,7 @@ QString &QString::sprintf( const char *format, ... )
 
 bool QString::fill( char c, int len )
 {
+    QSTRING_DETACH(this)
     if ( len < 0 )
 	len = length();
     if ( !QByteArray::fill(c,len+1) )
@@ -1048,12 +1118,13 @@ QString QString::stripWhiteSpace() const
 
     register char *s = data();
     QString result = s;
-    if ( !isspace(s[0]) && !isspace(s[length()-1]) )
+    int reslen = result.length();
+    if ( !isspace(s[0]) && !isspace(s[reslen-1]) )
 	return result;				// returns a copy
 
     s = result.data();
     int start = 0;
-    int end = result.length() - 1;
+    int end = reslen - 1;
     while ( isspace(s[start]) )			// skip white space from start
 	start++;
     if ( s[start] == '\0' ) {			// only white space
@@ -1132,11 +1203,13 @@ QString &QString::insert( uint index, const char *s )
     uint olen = length();
     int nlen = olen + len;
     if ( index >= olen ) {			// insert after end of string
+	QSTRING_DETACH(this)
 	if ( QByteArray::resize(nlen+index-olen+1) ) {
 	    memset( data()+olen, ' ', index-olen );
 	    memcpy( data()+index, s, len+1 );
 	}
     } else if ( QByteArray::resize(nlen+1) ) {	// normal insert
+	QSTRING_DETACH(this)
 	memmove( data()+index+len, data()+index, olen-index+1 );
 	memcpy( data()+index, s, len );
     }
@@ -1183,8 +1256,9 @@ QString &QString::insert( uint index, char c )	// insert char
   \e len is too large, the rest of the string is removed.
 
   \code
-    QString s = "neutrino";
-    s.remove( 1, 6 );				// s == "no"
+    QString s = "Montreal";
+    s.remove( 1, 4 );
+    // s == "Meal"
   \endcode
 
   \sa insert(), replace()
@@ -1194,9 +1268,12 @@ QString &QString::remove( uint index, uint len )
 {
     uint olen = length();
     if ( index + len >= olen ) {		// range problems
-	if ( index < olen )			// index ok
+	if ( index < olen ) {			// index ok
+	    QSTRING_DETACH(this)
 	    resize( index+1 );
+	}
     } else if ( len != 0 ) {
+	QSTRING_DETACH(this)
 	memmove( data()+index, data()+index+len, olen-index-len+1 );
 	QByteArray::resize(olen-len+1);
     }
@@ -1379,13 +1456,9 @@ uint QString::toUInt( bool *ok ) const
 double QString::toDouble( bool *ok ) const
 {
     char *end;
-    double val = strtod( data(), &end );
-    if ( ok ) {
-	if ( end == 0 || *end == '\0' )
-	    *ok = TRUE;
-	else
-	    *ok = FALSE;
-    }
+    double val = strtod( data() ? data() : "", &end );
+    if ( ok )
+	*ok = ( data() && *data() && ( end == 0 || *end == '\0' ) );
     return val;
 }
 
@@ -1411,6 +1484,7 @@ float QString::toFloat( bool *ok ) const
 
 QString &QString::setStr( const char *str )
 {
+    QSTRING_DETACH(this)
     if ( str )					// valid string
 	store( str, strlen(str)+1 );
     else					// empty
@@ -1425,6 +1499,7 @@ QString &QString::setStr( const char *str )
 
 QString &QString::setNum( long n )
 {
+    QSTRING_DETACH(this)
     char buf[20];
     register char *p = &buf[19];
     bool neg;
@@ -1452,6 +1527,7 @@ QString &QString::setNum( long n )
 
 QString &QString::setNum( ulong n )
 {
+    QSTRING_DETACH(this)
     char buf[20];
     register char *p = &buf[19];
     *p = '\0';
@@ -1535,7 +1611,7 @@ QString &QString::setNum( double n, char f, int prec )
 
 /*!
   Sets the character at position \e index to \e c and expands the
-  string if necessary.
+  string if necessary, filling with spaces.
 
   Returns FALSE if this \e index was out of range and the string could
   not be expanded, otherwise TRUE.
@@ -1543,12 +1619,14 @@ QString &QString::setNum( double n, char f, int prec )
 
 bool QString::setExpand( uint index, char c )
 {
-    if ( index >= length() ) {
-	uint oldlen = length();
+    QSTRING_DETACH(this)
+    uint oldlen = length();
+    if ( index >= oldlen ) {
 	if ( !QByteArray::resize( index+2 ) )	// no memory
 	    return FALSE;
-	memset( data() + oldlen, ' ', length() - oldlen );
-	*(data() + length()) = '\0';		// terminate padded string
+	if ( index > oldlen )
+	    memset( data() + oldlen, ' ', index - oldlen );
+	*(data() + index+1) = '\0';		// terminate padded string
     }
     *(data() + index) = c;
     return TRUE;
@@ -1580,6 +1658,7 @@ QString& QString::operator+=( const char *str )
 {
     if ( !str )
 	return *this;				// nothing to append
+    QSTRING_DETACH(this)
     uint len1 = length();
     uint len2 = strlen(str);
     if ( !QByteArray::resize( len1 + len2 + 1 ) )
@@ -1594,6 +1673,7 @@ QString& QString::operator+=( const char *str )
 
 QString &QString::operator+=( char c )
 {
+    QSTRING_DETACH(this)
     uint len = length();
     if ( !QByteArray::resize( len + 2 ) )
 	return *this;				// no memory
@@ -1626,8 +1706,13 @@ QDataStream &operator<<( QDataStream &s, const QString &str )
 
 QDataStream &operator>>( QDataStream &s, QString &str )
 {
+    QSTRING_DETACH(&str)
     Q_UINT32 len;
     s >> len;					// read size of string
+    if ( len == 0 || s.eof() ) {		// end of file reached
+	str.resize( 0 );
+	return s;
+    }
     if ( !str.QByteArray::resize( (uint)len )) {// resize string
 #if defined(CHECK_NULL)
 	warning( "QDataStream: Not enough memory to read QString" );
