@@ -1,29 +1,32 @@
 /****************************************************************************
-** $Id: qvector.h,v 2.1 1997/08/06 15:05:56 hanord Exp $
+** $Id: qvector.h,v 2.3 1998/07/03 00:09:47 hanord Exp $
 **
 ** Definition of QVector template/macro class
 **
 ** Created : 930907
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QVECTOR_H
 #define QVECTOR_H
 
+#ifndef QT_H
 #include "qgvector.h"
+#endif // QT_H
 
 
 #if defined(USE_MACROCLASS)
