@@ -1,42 +1,46 @@
 /**********************************************************************
-** $Id: qlabel.cpp,v 2.8 1997/03/22 16:26:12 hanord Exp $
+** $Id: qlabel.cpp,v 2.34 1998/07/04 13:48:11 agulbra Exp $
 **
 ** Implementation of QLabel widget class
 **
 ** Created : 941215
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qlabel.h"
 #include "qpixmap.h"
 #include "qpainter.h"
-#include "qdrawutl.h"
+#include "qdrawutil.h"
 #include "qaccel.h"
 #include "qkeycode.h"
+#include "qmovie.h"
 #include <ctype.h>
 
-RCSTAG("$Id: qlabel.cpp,v 2.8 1997/03/22 16:26:12 hanord Exp $");
-
 #if QT_VERSION == 200
 #error "Remove QLabel dict!"
 #endif
 
-#include <qintdict.h>
+#include "qintdict.h"
 
 struct QLabel_Private
 {
     QWidget * buddy;
     QAccel * accel;
+    QMovie * movie;
 };
 
 static QIntDict<QLabel_Private> *qlabel_extraStuff = 0;
@@ -66,7 +70,7 @@ static void cleanupLabel()
 
   \code
      QLineEdit * phone = new QLineEdit( this, "phone number" );
-     QLabel * phoneLabel = new QLineEdit( phone, "&Phone", this );
+     QLabel * phoneLabel = new QLabel( phone, "&Phone", this );
   \endcode
 
   In this example, keyboard focus is transferred to the label's buddy
@@ -93,8 +97,14 @@ static void cleanupLabel()
   \endcode
 
   Both lines are flush with the right side of the label.
+
+  <img src=qlabel-m.gif> <img src=qlabel-w.gif>
+
+  \sa QLineEdit QMovie
+  <a href="guibooks.html#fowler">GUI Design Handbook: Label</a>
 */
 
+
 /*!
   Constructs an empty label which is left-aligned, vertically centered,
   has an automatic margin and with manual resizing.
@@ -108,13 +118,13 @@ static void cleanupLabel()
 QLabel::QLabel( QWidget *parent, const char *name, WFlags f )
     : QFrame( parent, name, f )
 {
-    initMetaObject();
     lpixmap    = 0;
     align      = AlignLeft | AlignVCenter | ExpandTabs;
     extraMargin= -1;
     autoresize = FALSE;
 }
 
+
 /*!
   Constructs a label with a text. The label is left-aligned, vertically
   centered, has an automatic margin and with manual resizing.
@@ -128,7 +138,6 @@ QLabel::QLabel( QWidget *parent, const char *name, WFlags f )
 QLabel::QLabel( const char *text, QWidget *parent, const char *name, WFlags f )
 	: QFrame( parent, name, f ), ltext(text)
 {
-    initMetaObject();
     lpixmap    = 0;
     align      = AlignLeft | AlignVCenter | ExpandTabs;
     extraMargin= -1;
@@ -153,7 +162,7 @@ QLabel::QLabel( const char *text, QWidget *parent, const char *name, WFlags f )
     QLineEdit *phone   = new QLineEdit( this );
     QLabel    *phone_l = new QLabel( phone, "&Phone:", this );
     // geometry management setup not shown
-  \\endcode
+  \endcode
 
   With the code above, the focus jumps to the Name field when the user
   presses Alt-N, and to the Phone field when the user presses Alt-P.
@@ -165,7 +174,6 @@ QLabel::QLabel( QWidget *buddy,  const char *text,
 		QWidget *parent, const char *name, WFlags f )
     : QFrame( parent, name, f ), ltext("")
 {
-    initMetaObject();
     lpixmap    = 0;
     align      = ShowPrefix | AlignLeft | AlignVCenter | ExpandTabs;
     extraMargin= -1;
@@ -181,11 +189,14 @@ QLabel::QLabel( QWidget *buddy,  const char *text,
 
 QLabel::~QLabel()
 {
+    unsetMovie();
     delete lpixmap;
-    QLabel_Private * d;
-    if ( qlabel_extraStuff && (d=qlabel_extraStuff->find( (long)this )) ) {
-	qlabel_extraStuff->take( (long)this );
-	delete d;
+    if ( qlabel_extraStuff ) {
+	QLabel_Private *d = qlabel_extraStuff->find( (long)this );
+        if ( d ) {
+	    qlabel_extraStuff->take( (long)this );
+	    delete d;
+	}
     }
 }
 
@@ -208,6 +219,7 @@ QLabel::~QLabel()
 
 void QLabel::setText( const char *text )
 {
+    unsetMovie();
     if ( ltext == text )
 	return;
     ltext = text;
@@ -234,9 +246,18 @@ void QLabel::setText( const char *text )
 	    repaint();
     } else {
 	updateLabel();
-    } 
+    }
 }
 
+
+/*!  Clears the label.  Equivalent with setText( "" ). */
+
+void QLabel::clear()
+{
+    setText( "" );
+}
+
+
 /*!
   \fn QPixmap *QLabel::pixmap() const
   Returns the label pixmap.
@@ -257,6 +278,7 @@ void QLabel::setText( const char *text )
 
 void QLabel::setPixmap( const QPixmap &pixmap )
 {
+    unsetMovie();
     int w, h;
     if ( lpixmap ) {
 	w = lpixmap->width();
@@ -450,14 +472,21 @@ void QLabel::setAutoResize( bool enable )
 
 QSize QLabel::sizeHint() const
 {
-    QPainter p;
-    p.begin( this );
+    QPainter p( this );
     QRect br;
     QPixmap *pix = pixmap();
-    if ( pix )
+    QMovie *mov = movie();
+    if ( pix ) {
 	br = QRect( 0, 0, pix->width(), pix->height() );
-    else
+    } else if ( mov ) {
+	br = QRect( 0, 0, mov->framePixmap().width(),
+		mov->framePixmap().height() );
+    } else {
 	br = p.boundingRect( 0,0, 1000,1000, alignment(), text() );
+	// adjust so "Yes" and "yes" will have the same height
+	int h = fontMetrics().lineSpacing();
+	br.setHeight( ((br.height() + h-1) / h)*h - fontMetrics().leading() );
+    }
     int m  = 2*margin();
     int fw = frameWidth();
     if ( m < 0 ) {
@@ -468,7 +497,6 @@ QSize QLabel::sizeHint() const
     }
     int w = br.width()	+ m + 2*fw;
     int h = br.height() + m + 2*fw;
-    p.end();
 
     return QSize( w, h );
 }
@@ -498,6 +526,19 @@ void QLabel::drawContents( QPainter *p )
 	if ( align & AlignBottom )
 	    cr.setBottom( cr.bottom() - m );
     }
+
+    QMovie *mov = movie();
+    if ( mov ) {
+	// ### should add movie to qDrawItem when this Dict workaround is gone
+	QRect r = qItemRect( p, style(),
+			cr.x(), cr.y(), cr.width(), cr.height(),
+			align, isEnabled(), &(mov->framePixmap()), ltext );
+	// ### could resize movie frame at this point
+	p->drawPixmap(r.x(), r.y(), mov->framePixmap());
+	return;
+    }
+
+    // Not a movie
     qDrawItem( p, style(), cr.x(), cr.y(), cr.width(), cr.height(),
 	       align, colorGroup(), isEnabled(),
 	       lpixmap, ltext );
@@ -510,11 +551,12 @@ void QLabel::drawContents( QPainter *p )
 
 void QLabel::updateLabel()
 {
-    QPainter paint;
-    paint.begin( this );
-    paint.eraseRect( contentsRect() );
+    // ##### perhaps we should just use repaint(contentsRect())
+
+    QPainter paint( this );
+    if ( backgroundMode() != NoBackground )
+	paint.eraseRect( contentsRect() );
     drawContents( &paint );
-    paint.end();
 }
 
 
@@ -528,15 +570,12 @@ void QLabel::acceleratorSlot()
 	return;
 
     QLabel_Private * that = qlabel_extraStuff->find( (long)this );
-    if ( that && that->buddy && !that->buddy->hasFocus() ) {
-	QWidget * w = that->buddy;
-	while ( w ) {
-	    if ( !w->isVisible() )
-		return;
-	    w = w->parentWidget();
-	}
+    if ( that && that->buddy && 
+	 !that->buddy->hasFocus() &&
+	 that->buddy->isEnabledToTLW() &&
+	 that->buddy->isVisibleToTLW() &&
+	 that->buddy->focusPolicy() != NoFocus )
 	that->buddy->setFocus();
-    }
 }
 
 
@@ -544,7 +583,7 @@ void QLabel::acceleratorSlot()
   Internal slot, used to clean up if the buddy widget dies.
 */
 
-void QLabel::buddyDied()
+void QLabel::buddyDied() // I can't remember if I cried.
 {
     if ( !qlabel_extraStuff )
 	return;
@@ -582,10 +621,11 @@ void QLabel::setBuddy( QWidget *buddy )
 	that->buddy = buddy;
 	that->accel = new QAccel( this, "accel label accel" );
     }
-    const char * p = strchr( ltext, '&' );
+		
+    const char * p = ltext.isEmpty() ? 0 : strchr( ltext, '&' );
     while( p && *p && p[1] == '&' )
 	p = strchr( p+2, '&' );
-    if ( p && *p && isalpha(p[1]) ) {
+    if ( p && *p && isalnum(p[1]) ) {
 	that->accel->connectItem( that->accel->insertItem(ALT+
 							  toupper(p[1])),
 				  this, SLOT(acceleratorSlot()) );
@@ -609,3 +649,107 @@ QWidget * QLabel::buddy() const
     QLabel_Private * that = qlabel_extraStuff->find( (long)this );
     return that && that->buddy ? that->buddy : 0;
 }
+
+
+void QLabel::movieUpdated(const QRect& rect)
+{
+    QMovie *mov = movie();
+    if ( mov && !mov->isNull() ) {
+	QRect r = contentsRect();
+	r = qItemRect( 0, style(), r.x(), r.y(), r.width(), r.height(),
+		   align, isEnabled(), &(mov->framePixmap()), ltext );
+	r.moveBy(rect.x(), rect.y());
+	r.setWidth(QMIN(r.width(), rect.width()));
+	r.setHeight(QMIN(r.height(), rect.height()));
+	repaint( r );
+    }
+}
+
+void QLabel::movieResized(const QSize& size)
+{
+    if (autoresize) adjustSize();
+    movieUpdated(QRect(QPoint(0,0),size));
+}
+
+/*!
+  Sets a QMovie to display in the label, or removes any existing movie
+  if the given movie QMovie::isNull().
+
+  Any current pixmap or text label is cleared.
+
+  If the label has a buddy, the accelerator is disabled since the
+  movie doesn't contain any suitable character.
+*/
+void QLabel::setMovie( const QMovie& movie )
+{
+    //## ugle private data.
+    if ( !qlabel_extraStuff ) {
+	qlabel_extraStuff = new QIntDict<QLabel_Private>;
+	CHECK_PTR( qlabel_extraStuff );
+	qAddPostRoutine( cleanupLabel );
+    }
+    QLabel_Private * d = qlabel_extraStuff->find( (long)this );
+    if ( !d ) {
+	d = new QLabel_Private;
+	d->buddy = 0;
+	d->movie = 0;
+	d->accel = new QAccel( this, "accel label accel" );
+	qlabel_extraStuff->insert( (long)this, d );
+    }
+
+    if ( d->movie ) {
+	d->movie->disconnectResize(this, SLOT(movieResized(const QSize&)));
+	d->movie->disconnectUpdate(this, SLOT(movieUpdated(const QRect&)));
+    }
+
+    if ( movie.isNull() ) {
+	delete d->movie;
+	d->movie = 0;
+    } else {
+	if ( !d->movie ) d->movie = new QMovie;
+	*d->movie = movie;
+	ltext = "MOVIE";
+    }
+    d->accel->clear();
+
+    if ( lpixmap ) {
+	delete lpixmap;
+	lpixmap = 0;
+    }
+
+    if ( d->movie ) {
+	d->movie->connectResize(this, SLOT(movieResized(const QSize&)));
+	d->movie->connectUpdate(this, SLOT(movieUpdated(const QRect&)));
+    }
+}
+
+/*
+  Efficiently unset the movie.
+*/
+void QLabel::unsetMovie()
+{
+    if (!lpixmap && ltext=="MOVIE" && qlabel_extraStuff) {
+	// You think you are a movie...
+	QLabel_Private * d = qlabel_extraStuff->find( (long)this );
+	if (d && d->movie) {
+	    d->movie->disconnectResize(this, SLOT(movieResized(const QSize&)));
+	    d->movie->disconnectUpdate(this, SLOT(movieUpdated(const QRect&)));
+	    delete d->movie;
+	    d->movie = 0;
+	}
+    }
+}
+
+/*!
+  Returns the QMovie currently displaying in the label, or 0
+  if none has been set.
+*/
+QMovie* QLabel::movie() const
+{
+    if (!lpixmap && ltext=="MOVIE" && qlabel_extraStuff) {
+	// You think you are a movie...
+	QLabel_Private * d = qlabel_extraStuff->find( (long)this );
+	return d ? d->movie : 0;
+    }
+    return 0;
+}
