@@ -1,29 +1,32 @@
 /**********************************************************************
-** $Id: qlabel.h,v 2.4 1997/09/15 13:02:38 hanord Exp $
+** $Id: qlabel.h,v 2.8 1998/07/03 00:09:48 hanord Exp $
 **
 ** Definition of QLabel widget class
 **
 ** Created : 941215
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #ifndef QLABEL_H
 #define QLABEL_H
 
+#ifndef QT_H
 #include "qframe.h"
+#endif // QT_H
 
 
 class QLabel : public QFrame
@@ -59,6 +62,7 @@ public slots:
     void	setMovie( const QMovie & );
     void	setNum( int );
     void	setNum( double );
+    void	clear();
 
 protected:
     void	drawContents( QPainter * );
@@ -79,8 +83,8 @@ private:
     void	unsetMovie();
 
 private:	// Disabled copy constructor and operator=
-    QLabel( const QLabel & ) {}
-    QLabel &operator=( const QLabel & ) { return *this; }
+    QLabel( const QLabel & );
+    QLabel &operator=( const QLabel & );
 };
 
 
