@@ -1,38 +1,42 @@
 /**********************************************************************
-** $Id: qlistbox.cpp,v 2.22 1997/03/16 13:45:27 paul Exp $
+** $Id: qlistbox.cpp,v 2.71 1998/07/03 00:09:49 hanord Exp $
 **
 ** Implementation of QListBox widget class
 **
 ** Created : 941121
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qlistbox.h"
-#include "qfontmet.h"
+#include "qfontmetrics.h"
 #include "qpainter.h"
 #include "qstrlist.h"
 #include "qkeycode.h"
+#include "qscrollbar.h"
 #include "qpixmap.h"
-#include "qapp.h"
-
-RCSTAG("$Id: qlistbox.cpp,v 2.22 1997/03/16 13:45:27 paul Exp $");
-
+#include "qapplication.h"
 
 Q_DECLARE(QListM, QListBoxItem);
 
 class QLBItemList : public QListM(QListBoxItem) // internal class
 {
     int compareItems( GCI i1, GCI i2);
+public:
+    int timerId;				//### bincomp
 };
 
 int QLBItemList::compareItems( GCI i1, GCI i2)
@@ -43,24 +47,28 @@ int QLBItemList::compareItems( GCI i1, GCI i2)
 }
 
 
-static inline bool checkInsertIndex( const char *method, int count, int *index)
+static inline bool checkInsertIndex( const char *method, const char * name,
+				     int count, int *index)
 {
     bool range_err = (*index > count);
 #if defined(CHECK_RANGE)
     if ( *index > count )
-	warning( "QListBox::%s: Index %i out of range", method, *index );
+	warning( "QListBox::%s: (%s) Index %i out of range",
+		 method, name ? name : "<no name>", *index );
 #endif
     if ( *index < 0 )				// append
 	*index = count;
     return !range_err;
 }
 
-static inline bool checkIndex( const char *method, int count, int index )
+static inline bool checkIndex( const char *method, const char * name,
+			       int count, int index )
 {
     bool range_err = ((uint)index >= (uint)count);
 #if defined(CHECK_RANGE)
     if ( range_err )
-	warning( "QListBox::%s: Index %d out of range", method, index );
+	warning( "QListBox::%s: (%s) Index %d out of range",
+		 method, name ? name : "<no name>", index );
 #endif
     return !range_err;
 }
@@ -70,8 +78,6 @@ static inline bool checkIndex( const char *method, int count, int index )
   \class QListBoxItem qlistbox.h
   \brief This is the base class of all list box items.
 
-  \ingroup abstract
-
   This class is the abstract base class of all list box items. If you
   need to insert customized items into a QListBox, you must inherit
   this class and reimplement paint(), height() and width().
@@ -194,7 +200,7 @@ QListBoxItem::~QListBoxItem()
 
 /*!
   \class QListBoxText qlistbox.h
-  \brief QListBoxText provides list box items with text.
+  \brief The QListBoxText class provides list box items with text.
 
   The text is drawn in the widget's current font. If you need several
   different fonts, you have to make your own subclass of QListBoxItem.
@@ -260,7 +266,7 @@ int QListBoxText::width( const QListBox *lb ) const
 
 /*!
   \class QListBoxPixmap qlistbox.h
-  \brief QListBoxPixmap provides list box items with a pixmap.
+  \brief The QListBoxPixmap class provides list box items with a pixmap.
 
   \sa QListBox, QListBoxItem
 */
@@ -338,7 +344,7 @@ int QListBoxPixmap::width( const QListBox * ) const
   If the user does not select anything, no signals are emitted and
   currentItem() returns -1.
 
-  A list box has \c StrongFocus as a default focusPolicy(), i.e. it can 
+  A list box has \c StrongFocus as a default focusPolicy(), i.e. it can
   get keyboard focus both by tabbing and clicking.
 
   New items may be inserted using either insertItem(), insertStrList()
@@ -368,6 +374,12 @@ int QListBoxPixmap::width( const QListBox * ) const
 
   \warning The list box assumes ownership of all list box items
   and will delete them when they are not needed.
+
+  <img src=qlistbox-m.gif> <img src=qlistbox-w.gif>
+
+  \sa QListView QComboBox QButtonGroup
+  <a href="guibooks.html#fowler">GUI Design Handbook: List Box (two
+  sections)</a>
 */
 
 
@@ -378,7 +390,7 @@ int QListBoxPixmap::width( const QListBox * ) const
 #error "Remove QListBox dict."
 #endif
 
-#include <qintdict.h>
+#include "qintdict.h"
 
 static QIntDict<int> *qlb_maxLenDict = 0;
 
@@ -402,26 +414,27 @@ static void cleanupListbox()
 QListBox::QListBox( QWidget *parent, const char *name, WFlags f )
     : QTableView( parent, name, f )
 {
-    initMetaObject();
     doDrag	  = TRUE;
     doAutoScroll  = TRUE;
     current	  = -1;
     isTiming	  = FALSE;
     stringsOnly	  = TRUE;
+    multiSelect   = FALSE;
     goingDown	  = FALSE;
     itemList	  = new QLBItemList;
     CHECK_PTR( itemList );
+    itemList->timerId = 0;
     setCellWidth( 0 );
     QFontMetrics fm = fontMetrics();
     setCellHeight( fm.lineSpacing() + 1 );
     setNumCols( 1 );
-    setTableFlags( Tbl_autoVScrollBar|Tbl_autoHScrollBar|
-		   Tbl_smoothVScrolling | Tbl_clipCellPainting );
+    setTableFlags( Tbl_autoVScrollBar|Tbl_autoHScrollBar | //Tbl_snapToVGrid |
+		   Tbl_smoothVScrolling | Tbl_clipCellPainting  );
     switch ( style() ) {
 	case WindowsStyle:
 	case MotifStyle:
 	    setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
-	    setBackgroundColor( colorGroup().base() );
+	    setBackgroundMode( PaletteBase );
 	    break;
 	default:
 	    setFrameStyle( QFrame::Panel | QFrame::Plain );
@@ -435,12 +448,20 @@ QListBox::QListBox( QWidget *parent, const char *name, WFlags f )
     }
 }
 
+#if QT_VERSION == 200
+#error "Remove QListBox pointer."
+#endif
+
+static QListBox * changedListBox = 0;
+
 /*!
   Destroys the list box.  Deletes all list box items.
 */
 
 QListBox::~QListBox()
 {
+    if ( changedListBox == this )
+	changedListBox = 0;
     goingDown = TRUE;
     clearList();
     if ( qlb_maxLenDict )
@@ -448,23 +469,55 @@ QListBox::~QListBox()
     delete itemList;
 }
 
-/*!
-  \fn void QListBox::highlighted( int index )
+/*! \fn void QListBox::selectionChanged()
+
+  This signal is emitted when the selection set of a multiple-choice
+  listbox changes. If the user selects five items by drag-selecting,
+  QListBox tries to emit just one selectionChanged() signal, so the
+  signal can be connected to computationally expensive slots.
+
+  \sa selected() currentItem()
+*/
+
+/*! \fn void QListBox::highlighted( int index )
+
   This signal is emitted when the user highlights a new current item.
+  The argument is the index of the new item, which is already current.
 
-  \sa selected()
+  \sa selected() currentItem() selectionChanged()
+*/
+
+/*! \fn void QListBox::highlighted( const char * )
+
+  This signal is emitted when the user highlights a new current item
+  and the new item is a string.  The argument is the text of the
+  new current item.
+
+  \sa selected() currentItem() selectionChanged()
+*/
+
+/*! \fn void QListBox::selected( int index )
+
+  This signal is emitted when the user double-clicks on an item or
+  presses return when an item is highlighted.  The argument is the
+  index of the selected item.
+
+  \sa highlighted() selectionChanged()
+*/
+
+/*! \fn void QListBox::selected( const char * )
+
+  This signal is emitted when the user double-clicks on an item or
+  presses return while an item is highlighted, and the selected item
+  is (or has) a string.  The argument is the text of the selected
+  item.
+
+  \sa highlighted() selectionChanged()
 */
 
 /*!
-  \fn void QListBox::selected( int index )
-  This signal is emitted when the user double-clicks on an item
-  or presses return when an item is highlighted.
-
-  \sa highlighted()
-*/
-
-/*!
-  Reimplements QWidget::setFont() to update the list box line height.
+  Reimplements QWidget::setFont() to update the list box line height
+  and maxItemWidth().
 */
 
 void QListBox::setFont( const QFont &font )
@@ -472,6 +525,9 @@ void QListBox::setFont( const QFont &font )
     QWidget::setFont( font );
     if ( stringsOnly )
 	setCellHeight( fontMetrics().lineSpacing() + 1 );
+    // else ...?
+
+    updateCellWidth();
 }
 
 
@@ -496,7 +552,7 @@ uint QListBox::count() const
 
 void QListBox::insertStrList( const QStrList *list, int index )
 {
-    if ( !checkInsertIndex( "insertStrList", count(), &index ) )
+    if ( !checkInsertIndex( "insertStrList", name(), count(), &index ) )
 	return;
     if ( !list ) {
 #if defined(CHECK_NULL)
@@ -512,6 +568,8 @@ void QListBox::insertStrList( const QStrList *list, int index )
 	++it;
 	insert( new QListBoxText(txt), index++, FALSE );
     }
+    if ( currentItem() < 0 && numRows() > 0 && hasFocus() )
+	setCurrentItem( 0 );
     updateNumRows( TRUE );
     if ( autoUpdate() && isVisible() )
 	repaint();
@@ -529,7 +587,7 @@ void QListBox::insertStrList( const QStrList *list, int index )
 
 void QListBox::insertStrList( const char **strings, int numStrings, int index )
 {
-    if ( !checkInsertIndex( "insertStrList", count(), &index ) )
+    if ( !checkInsertIndex( "insertStrList", name(), count(), &index ) )
 	return;
     if ( !strings ) {
 #if defined(CHECK_NULL)
@@ -545,6 +603,8 @@ void QListBox::insertStrList( const char **strings, int numStrings, int index )
 	i++;
     }
     updateNumRows( TRUE );
+    if ( currentItem() < 0 && numRows() > 0 && hasFocus() )
+	setCurrentItem( 0 );
     if ( autoUpdate() && isVisible() )
 	repaint();
 }
@@ -561,7 +621,7 @@ void QListBox::insertStrList( const char **strings, int numStrings, int index )
 
 void QListBox::insertItem( const QListBoxItem *lbi, int index )
 {
-    if ( !checkInsertIndex( "insertItem", count(), &index ) )
+    if ( !checkInsertIndex( "insertItem", name(), count(), &index ) )
 	return;
     if ( !lbi ) {
 #if defined ( CHECK_NULL )
@@ -575,6 +635,8 @@ void QListBox::insertItem( const QListBoxItem *lbi, int index )
     }
     insert( lbi, index, TRUE );
     updateNumRows( FALSE );
+    if ( currentItem() < 0 && numRows() > 0 && hasFocus() )
+	setCurrentItem( 0 );
     if ( autoUpdate() )
 	repaint();
 }
@@ -589,7 +651,7 @@ void QListBox::insertItem( const QListBoxItem *lbi, int index )
 
 void QListBox::insertItem( const char *text, int index )
 {
-    if ( !checkInsertIndex( "insertItem", count(), &index ) )
+    if ( !checkInsertIndex( "insertItem", name(), count(), &index ) )
 	return;
     if ( !text ) {
 #if defined ( CHECK_NULL )
@@ -599,6 +661,8 @@ void QListBox::insertItem( const char *text, int index )
     }
     insert( new QListBoxText(text), index, TRUE );
     updateNumRows( FALSE );
+    if ( currentItem() < 0 && numRows() > 0 && hasFocus() )
+	setCurrentItem( 0 );
     if ( autoUpdate() && itemVisible(index) ) {
 	int x, y;
 	colXPos( 0, &x );
@@ -617,7 +681,7 @@ void QListBox::insertItem( const char *text, int index )
 
 void QListBox::insertItem( const QPixmap &pixmap, int index )
 {
-    if ( !checkInsertIndex( "insertItem", count(), &index ) )
+    if ( !checkInsertIndex( "insertItem", name(), count(), &index ) )
 	return;
     if ( stringsOnly ) {
 	stringsOnly = FALSE;
@@ -625,6 +689,8 @@ void QListBox::insertItem( const QPixmap &pixmap, int index )
     }
     insert( new QListBoxPixmap(pixmap), index, TRUE );
     updateNumRows( FALSE );
+    if ( currentItem() < 0 && numRows() > 0 && hasFocus() )
+	setCurrentItem( 0 );
     if ( autoUpdate() && itemVisible(index) ) {
 	int x, y;
 	colXPos( index, &x );
@@ -681,24 +747,35 @@ void QListBox::inSort( const char *text )
 
 
 /*!
-  Removes the item at position \e index.
+  Removes the item at position \e index. If \a index is equal to
+  currentItem(), a new item gets selected and the highlighted()
+  signal is emitted.
   \sa insertItem(), clear()
 */
 
 void QListBox::removeItem( int index )
 {
-    if ( !checkIndex( "removeItem", count(), index ) )
+    if ( !checkIndex( "removeItem", name(), count(), index ) )
 	return;
+    bool currentChanged = ( current == index );
+
     if ( current >= index && current > 0 )
 	current--;
     bool    updt = autoUpdate() && itemVisible( index );
     QListBoxItem *lbi = itemList->take( index );
-    QFontMetrics fm   = fontMetrics();
     int w             = lbi->width( this );
     updateNumRows( w == cellWidth() );
     delete lbi;
-    if ( count() == 0 )
+    if ( count() == 0 ) {
 	current = -1;
+    } else if ( currentChanged ) {
+	QString tmp = 0;
+	if ( item( currentItem() ) )
+	    tmp = item( currentItem() )->text();
+	emit highlighted( current );
+	if ( !tmp.isNull() )
+	    emit highlighted( tmp );
+    }
     if ( updt )
 	repaint();
 }
@@ -753,7 +830,7 @@ const QPixmap *QListBox::pixmap( int index ) const
 
 void QListBox::changeItem( const char *text, int index )
 {
-    if ( !checkIndex( "changeItem", count(), index ) )
+    if ( !checkIndex( "changeItem", name(), count(), index ) )
 	return;
     change( new QListBoxText(text), index );
 }
@@ -768,7 +845,7 @@ void QListBox::changeItem( const char *text, int index )
 
 void QListBox::changeItem( const QPixmap &pixmap, int index )
 {
-    if ( !checkIndex( "changeItem", count(), index ) )
+    if ( !checkIndex( "changeItem", name(), count(), index ) )
 	return;
     change( new QListBoxPixmap(pixmap), index );
 }
@@ -783,7 +860,7 @@ void QListBox::changeItem( const QPixmap &pixmap, int index )
 
 void QListBox::changeItem( const QListBoxItem *lbi, int index )
 {
-    if ( !checkIndex( "changeItem", count(), index ) )
+    if ( !checkIndex( "changeItem", name(), count(), index ) )
 	return;
     change( lbi, index );
 }
@@ -827,6 +904,8 @@ void QListBox::setAutoUpdate( bool enable )
 /*!
   Returns the number of visible items.	This may change at any time
   since the user may resize the widget.
+
+  \sa setFixedVisibleLines()
 */
 
 int QListBox::numItemsVisible() const
@@ -856,13 +935,18 @@ void QListBox::setCurrentItem( int index )
 {
     if ( index == current )
 	return;
-    if ( !checkIndex( "setCurrentItem", count(), index ) )
+    if ( !checkIndex( "setCurrentItem", name(), count(), index ) )
 	return;
     int oldCurrent = current;
     current	   = index;
     updateItem( oldCurrent );
     updateItem( current, FALSE ); // Do not clear, current marker covers item
+    QString tmp = 0;
+    if ( item( currentItem() ) )
+	tmp = item( currentItem() )->text();
     emit highlighted( current );
+    if ( !tmp.isNull() )
+	emit highlighted( tmp );
 }
 
 /*!
@@ -1112,7 +1196,7 @@ void QListBox::setSmoothScrolling( bool enable )
 
 QListBoxItem *QListBox::item( int index ) const
 {
-    if (!checkIndex( "item", count(), index ) )
+    if (!checkIndex( "item", name(), count(), index ) )
 	return 0;
     return itemList->at( index );
 }
@@ -1176,18 +1260,15 @@ void QListBox::paintCell( QPainter *p, int row, int col )
 	return;
 
     QColorGroup g = colorGroup();
-    if ( current == row ) {
+    if ( isSelected( row ) ) {
 	QColor	 fc;				// fill color
 	if ( style() == WindowsStyle )
-	    fc = darkBlue;			// !!!hardcoded
+	    fc = QApplication::winStyleHighlightColor();
 	else
 	    fc = g.text();
-	if ( hasFocus() )
-	    p->fillRect( 0, 0, cellWidth(col), cellHeight(row), fc );
-	else
-	    p->fillRect( 1, 1, cellWidth(col) - 2, cellHeight(row) - 2, fc );
-	p->setPen( g.base() );
-	p->setBackgroundColor( g.text() );
+	p->fillRect( 0, 0, cellWidth(col), cellHeight(row), fc );
+	p->setPen( style() == WindowsStyle ? white : g.base() );
+	p->setBackgroundColor( fc );
     } else {
 	p->setBackgroundColor( g.base() );
 	p->setPen( g.text() );
@@ -1195,11 +1276,15 @@ void QListBox::paintCell( QPainter *p, int row, int col )
     lbi->paint( p );
     if ( current == row && hasFocus() ) {
 	if ( style() == WindowsStyle ) {
-	    p->drawWinFocusRect( 1, 1, viewWidth()-2 , cellHeight(row)-2 );
+	    p->drawWinFocusRect( 1, 1, cellWidth(col)-2 , cellHeight(row)-2,
+				 QApplication::winStyleHighlightColor() );
 	} else {
-	    p->setPen( g.base() );
+	    if ( isSelected( row ) )
+		p->setPen( g.base() );
+	    else
+		p->setPen( g.text() );
 	    p->setBrush( NoBrush );
-	    p->drawRect( 1, 1, viewWidth()-2 , cellHeight(row)-2 );
+	    p->drawRect( 1, 1, cellWidth(col)-2 , cellHeight(row)-2 );
 	}
     }
     p->setBackgroundColor( g.base() );
@@ -1217,9 +1302,11 @@ void QListBox::mousePressEvent( QMouseEvent *e )
     int itemClicked = findItem( e->pos().y() );
     if ( itemClicked != -1 ) {
 	setCurrentItem( itemClicked );
+	toggleCurrentItem();
     } else if ( contentsRect().contains( e->pos() ) &&
 		lastRowVisible() >= (int) count() ) {
 	setCurrentItem( count()-1 );
+	toggleCurrentItem();
     }
 }
 
@@ -1232,9 +1319,10 @@ void QListBox::mouseReleaseEvent( QMouseEvent *e )
     if ( doDrag )
 	mouseMoveEvent( e );
     if ( isTiming ) {
-	killTimers();
+	killTimer( itemList->timerId );
 	isTiming = FALSE;
     }
+    emitChangedSignal( FALSE );
 }
 
 /*!
@@ -1245,8 +1333,13 @@ void QListBox::mouseReleaseEvent( QMouseEvent *e )
 void QListBox::mouseDoubleClickEvent( QMouseEvent *e )
 {
     mouseReleaseEvent( e );
-    if ( currentItem() >= 0 )
+    if ( currentItem() >= 0 ) {
+	ASSERT(item( currentItem() ));
+	QString tmp = item( currentItem() )->text();
 	emit selected( currentItem());
+	if ( !tmp.isNull() )
+	    emit selected( tmp );
+    }
 }
 
 /*!
@@ -1259,11 +1352,22 @@ void QListBox::mouseMoveEvent( QMouseEvent *e )
 {
     if ( doDrag && (e->state() & (RightButton|LeftButton|MidButton)) != 0 ) {
 	int itemClicked = findItem( e->pos().y() );
-	if ( itemClicked != -1 ) {
+	if ( itemClicked >= 0 ) {
 	    if ( isTiming ) {
-		killTimers();
+		killTimer( itemList->timerId );
 		isTiming = FALSE;
 	    }
+	    if ( multiSelect ) {
+		bool s = currentItem() >= 0
+			 ? isSelected( currentItem() ) : TRUE;
+		int i = QMIN( itemClicked, currentItem() );
+		if ( i < 0 )
+		    i = 0;
+		while( i <= itemClicked || i <= currentItem() ) {
+		    setSelected( i, s );
+		    i++;
+		}
+	    }
 	    setCurrentItem( itemClicked );	// already current -> return
 	    return;
 	} else {
@@ -1275,7 +1379,7 @@ void QListBox::mouseMoveEvent( QMouseEvent *e )
 		scrollDown = TRUE;
 	    if ( !isTiming ) {
 		isTiming = TRUE;
-		startTimer( 100 );
+		itemList->timerId = startTimer( 100 );
 	    }
 	}
     }
@@ -1303,58 +1407,99 @@ void QListBox::keyPressEvent( QKeyEvent *e )
     int pageSize, delta;
 
     switch ( e->key() ) {
-	case Key_Up:
-	    if ( currentItem() > 0 ) {
-		setCurrentItem( currentItem() - 1 );
-		if ( currentItem() < topItem()	)
-		    setTopItem( currentItem() );
-	    }
-	    break;
-	case Key_Down:
-	    if ( currentItem() < (int)count() - 1 ) {
-		setCurrentItem( currentItem() + 1 );
-		if ( currentItem() > lastRowVisible() )
-		    setTopItem( topItem() + currentItem() - lastRowVisible() );
-	    }
-	    break;
-	case Key_Next:
+    case Key_Up:
+	if ( currentItem() > 0 ) {
+	    ensureCurrentVisible( currentItem() - 1 );
+	    if ( e->state() & ShiftButton )
+		toggleCurrentItem();
+	}
+	e->accept();
+	break;
+    case Key_Down:
+	if ( currentItem() < (int)count() - 1 ) {
+	    ensureCurrentVisible( currentItem()+1 );
+	    if ( e->state() & ShiftButton )
+		toggleCurrentItem();
+	}
+	e->accept();
+	break;
+    case Key_Next:
+	if ( style() == MotifStyle ) {
 	    delta = currentItem() - topItem();
-	    pageSize = lastRowVisible() - topItem();
+	    pageSize = QMAX( 1, lastRowVisible() - 1 - topItem() );
 	    setTopItem( QMIN( topItem() + pageSize, (int)count() - 1 ) );
 	    setCurrentItem( QMIN( topItem() + delta, (int)count() - 1 ) );
-	    break;
-	case Key_Prior:
+	} else {
+	    pageSize = QMAX( 1, lastRowVisible() - 1 - topItem() );
+	    setTopItem( QMIN(currentItem(),(int)count()-pageSize+1) );
+	    setCurrentItem( QMIN(lastRowVisible(), (int)count()-1) );
+	}
+	e->accept();
+	break;
+    case Key_Prior:
+	if ( style() == MotifStyle ) {
 	    delta = currentItem() - topItem();
-	    pageSize = lastRowVisible() - topItem();
+	    pageSize = QMAX( 1, lastRowVisible() - 1 - topItem() );
 	    setTopItem( QMAX( topItem() - pageSize, 0 ) );
 	    setCurrentItem( QMAX( topItem() + delta, 0 ) );
-	    break;
+	} else {
+	    pageSize = QMAX( 1, lastRowVisible() - topItem() );
+	    setTopItem( QMAX(0,currentItem()-pageSize+1) );
+	    setCurrentItem( topItem() );
+	}
+	e->accept();
+	break;
 
-	case Key_Return:
-	case Key_Enter:
-	    if ( currentItem() >= 0 )
-		emit selected( currentItem());
-	    break;
-	default:
-	    break;
+    case Key_Space:
+	toggleCurrentItem();
+	e->accept();
+	break;
+
+    case Key_Return:
+    case Key_Enter:
+	if ( currentItem() >= 0 ) {
+	    QString tmp = item( currentItem() )->text();
+	    emit selected( currentItem());
+	    if ( !tmp.isEmpty() )
+		emit selected( tmp );
+	}
+	// do NOT accept here.  qdialog.
+	break;
+    default:
+	break;
     }
+    emitChangedSignal( FALSE );
 }
 
 
 /*!
-  Handles focus events.	 Repaints, and sets the current item to first one
-  if there is no current item.
+  Handles focus events.  Repaints the current item (if not set,
+  topItem() is made current).
   \sa keyPressEvent(), focusOutEvent()
 */
 
 void QListBox::focusInEvent( QFocusEvent * )
 {
+    emitChangedSignal( FALSE );
     if ( currentItem() < 0 && numRows() > 0 )
 	setCurrentItem( topItem() );
     updateCell( currentItem(), 0); //show focus
 }
 
 
+/*!
+  Handles focus out events. Repaints the current item, if set.
+  \sa keyPressEvent(), focusOutEvent()
+*/
+
+void QListBox::focusOutEvent( QFocusEvent * )
+{
+    emitChangedSignal( FALSE );
+    if ( currentItem() >= 0 )
+	updateCell( currentItem(), 0); //show lack of focus
+}
+
+
 /*!
   Handles resize events.  Updates internal parameters for the new list box
   size.
@@ -1363,7 +1508,7 @@ void QListBox::focusInEvent( QFocusEvent * )
 void QListBox::resizeEvent( QResizeEvent *e )
 {
     QTableView::resizeEvent( e );
-    setCellWidth( QMAX(maxItemWidth(), viewWidth()) );
+    setCellWidth( QMAX((int)maxItemWidth(), viewWidth()) );
 }
 
 
@@ -1374,13 +1519,31 @@ void QListBox::resizeEvent( QResizeEvent *e )
 void QListBox::timerEvent( QTimerEvent * )
 {
     if ( scrollDown ) {
-	if ( topItem() != maxRowOffset() ) {
-	    setTopItem( topItem() + 1 );
-	    setCurrentItem( lastRowVisible() );
+	if ( currentItem() + 1 < (int)count() ) {
+	    int y = QMAX(currentItem(),lastRowVisible())+1;
+	    if ( y >= (int)count() )
+		y = count() - 1;
+	    if ( currentItem() >= 0 && multiSelect ) {
+		bool s = isSelected( currentItem() );
+		int i = currentItem();
+		while( i <= y ) {
+		    setSelected( i, s );
+		    i++;
+		}
+	    }
+	    ensureCurrentVisible( y );
 	}
     } else {
-	if ( topItem() != 0 ) {
+	if ( topItem() > 0 ) {
 	    setTopItem( topItem() - 1 );
+	    if ( currentItem() > 0 && multiSelect ) {
+		bool s = isSelected( currentItem() );
+		int i = currentItem();
+		while( i >= topItem() ) {
+		    setSelected( i, s );
+		    i--;
+		}
+	    }
 	    setCurrentItem( topItem() );
 	}
     }
@@ -1457,7 +1620,6 @@ void QListBox::clearList()
 void QListBox::updateCellWidth()
 {
     QListBoxItem *lbi = itemList->first();
-    QFontMetrics fm = fontMetrics();
     int maxW = 0;
     int w;
     while ( lbi ) {
@@ -1508,15 +1670,22 @@ void QListBox::change( const QListBoxItem *lbi, int index )
     ASSERT( lbi );
     ASSERT( (uint)index < itemList->count() );
 #endif
+
     QListBoxItem *old = itemList->take( index );
     int w = old->width( this );
     int h = old->height( this );
-    if ( w == cellWidth() )
-	updateCellWidth();
     delete old;
-    int oldpos = current;
-    insert( lbi, index, TRUE );
-    current = oldpos;
+    itemList->insert( index, lbi );
+    if ( w == cellWidth() ) {		     // I.e. index was the widest item
+	updateCellWidth();
+    }
+    else {
+	int ww = lbi->width( this );
+	if ( ww > maxItemWidth() )
+	    setMaxItemWidth( ww );
+	if ( ww > cellWidth() )
+	    setCellWidth( ww );
+    }
     int nh = cellHeight( index );
     int y;
     if ( autoUpdate() && rowYPos( index, &y ) ) {
@@ -1548,16 +1717,28 @@ void QListBox::updateNumRows( bool updateWidth )
 
 
 /*!
-  Returns the width in pixels of the longest item. 
+  Returns the width in pixels of the widest item.
 */
 
-long QListBox::maxItemWidth()
+long QListBox::maxItemWidth() const
 {
     if ( !qlb_maxLenDict )
 	return 0;
     return (long) qlb_maxLenDict->find( (long)this );
 }
 
+/*!
+  For binary compatibility.
+*/
+long QListBox::maxItemWidth()
+{
+    // This is only here for binary compatibility
+    if ( !qlb_maxLenDict )
+	return 0;
+    return (long) qlb_maxLenDict->find( (long)this );
+    // This is only here for binary compatibility
+}
+
 
 /*!
   Updates the cached value used by maxItemWidth().
@@ -1570,3 +1751,212 @@ void QListBox::setMaxItemWidth( int len )
     if ( len )
 	qlb_maxLenDict->insert( (long)this, (int*)len );
 }
+
+
+/*!
+  \fn bool QListBox::isMultiSelection() const
+
+  Returns TRUE if the listbox is in multi-selection mode, and FALSE if
+  it is in single-selection mode.
+
+  \sa setMultiSelection()
+*/
+
+/*!
+
+  Sets the list box to multi-selection mode if \a enable is TRUE, and to
+  single-selection mode if \a enable is FALSE.
+
+  Single- and multi-selections modes work the same, except that the
+  highlighted() and selected() signals are emitted at different times.
+
+  \sa isMultiSelection()
+*/
+
+void QListBox::setMultiSelection( bool enable )
+{
+    if ( enable != (bool)multiSelect ) {
+	multiSelect = enable;
+	update();
+    }
+}
+
+
+/*!
+  Toggles the selection status of currentItem() and repaints, if
+  the listbox is a multi-selection listbox.
+
+  Does nothing if the listbox is a single-selection listbox.
+
+  \sa setMultiSelection()
+*/
+
+void QListBox::toggleCurrentItem()
+{
+    if ( !multiSelect || currentItem() < 0 )
+	return;
+
+    QListBoxItem * i = item( currentItem() );
+    if ( !i )
+	return;
+
+    i->selected = !i->selected;
+    updateItem( currentItem() );
+    emitChangedSignal( TRUE );
+}
+
+
+/*!
+  Selects the item at position \a index if \a select is TRUE, or
+  unselects it if \a select is FALSE.  May also repaint the item.
+
+  If the listbox is a single-selection listbox and and \a select is TRUE,
+  setCurrentItem will be called.
+
+  If the listbox is a single-selection listbox and and \a select is FALSE,
+  clearSelection() will be called if \a index is the currently selected
+  item.
+
+  \sa setMultiSelection(), setCurrentItem(), clearSelection(), currentItem()
+*/
+
+void QListBox::setSelected( int index, bool select )
+{
+    if ( !multiSelect ) {
+	if ( select ) {
+	    setCurrentItem( index );
+	} else {
+	    if ( index == current )
+		clearSelection();
+	}
+	return;
+    }
+
+    if ( currentItem() < 0 )
+	return;
+
+    QListBoxItem *lbi = item( index );
+    if ( !lbi || lbi->selected == select )
+	return;
+
+    lbi->selected = select;
+    updateItem( index );
+    emitChangedSignal( TRUE );
+}
+
+
+/*!
+  Returns TRUE if item \a i is selected. Returns FALSE if it is not
+  selected or if there is an error.
+*/
+
+bool QListBox::isSelected( int i ) const
+{
+    if ( !multiSelect )
+	return i == current;
+
+    QListBoxItem * lbi = item( i );
+    return lbi ? lbi->selected : FALSE;
+}
+
+
+
+/*!
+  Deselects all items. Note that a single-selection listbox
+  will automatically select its first item if it has keyboard focus.
+*/
+
+void QListBox::clearSelection()
+{
+    if ( multiSelect ) {
+	for ( int i = 0; i < (int)count(); i++ )
+	    setSelected( i, FALSE );
+    } else {
+	int i = current;
+	if ( hasFocus() ) {
+	    current = 0;
+	    updateItem( current );
+	} else {
+	    current = -1;
+	}
+	updateItem( i );
+    }
+}
+
+
+
+
+
+/*!  If \a lazy is FALSE, maybe emit the changed() signal.  If \a lazy
+  is TRUE, note that it's to be sent out at some later time.
+*/
+
+void QListBox::emitChangedSignal( bool lazy ) {
+    if ( !multiSelect )
+	return;
+
+    if ( changedListBox && (!lazy || changedListBox != this) )
+	emit changedListBox->selectionChanged();
+
+    changedListBox = lazy ? this : 0;
+}
+
+
+/*!  Make sure that all of currentItem() is on-screen */
+
+void QListBox::ensureCurrentVisible( int newCurrent )
+{
+    if ( newCurrent < 0 )
+	newCurrent = currentItem();
+    if ( newCurrent < topItem() ) {
+	setTopItem( newCurrent );
+    } else if ( newCurrent >= lastRowVisible() ) {
+	int h = viewHeight();
+	int i = newCurrent;
+	while( i > 0 && h > 0 ) {
+	    h -= cellHeight( i );
+	    if ( h >= 0 )
+		i--;
+	}
+	if ( h <= 0 && i < newCurrent ) // make sure ALL of y is visible
+	    i++;
+	setTopItem( i );
+    }
+    if ( newCurrent != currentItem() )
+	setCurrentItem( newCurrent );
+}
+
+/*!
+  Sets a \link QWidget::setFixedHeight() fixed height\endlink for the
+  widget, so that it shows the given number of lines of text for the
+  current font size.
+*/
+void QListBox::setFixedVisibleLines( int lines )
+{
+    int ls = fontMetrics().lineSpacing() + 1; // #### explain +1
+    // #### What about auto-scrollbars?
+    int sb = testTableFlags(Tbl_hScrollBar)
+		? horizontalScrollBar()->height() : 0;
+    setFixedHeight( frameWidth()*2 + ls*lines + sb );
+    return;
+}
+
+/*!
+  Returns a size mased on maxItemWidth() and any value set by
+  setFixedVisibleLines().
+*/
+QSize QListBox::sizeHint() const
+{
+    QSize sz = QTableView::sizeHint();
+
+    int w = (int)maxItemWidth() + 2*frameWidth();
+    if ( testTableFlags(Tbl_vScrollBar) )
+	w += verticalScrollBar()->width();
+    sz.setWidth(w);
+
+    // For when setFixedVisibleLines is used
+    int h = maximumSize().height();
+    if ( h<1000 ) sz.setHeight(h);
+
+    return sz;
+}
