@@ -1,34 +1,32 @@
 /****************************************************************************
-** $Id: qslider.cpp,v 2.44 1997/09/26 10:57:41 eiriken Exp $
+** $Id: qslider.cpp,v 2.53 1998/07/03 00:09:53 hanord Exp $
 **
 ** Implementation of QSlider class
 **
 ** Created : 961019
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qslider.h"
 #include "qpainter.h"
-#include "qdrawutl.h"
+#include "qdrawutil.h"
 #include "qtimer.h"
 #include "qkeycode.h"
 
-RCSTAG("$Id: qslider.cpp,v 2.44 1997/09/26 10:57:41 eiriken Exp $");
-
-
 static const int motifBorder = 2;
 static const int motifLength = 30;
 static const int winLength = 9; // Must be odd
@@ -57,7 +55,11 @@ static int sliderStartVal = 0; //##### class member?
 
   A slider has a default focusPolicy() of \a TabFocus.
 
-  <img src=qslider-m.gif> <img src=qslider-w.gif> */
+  <img src=qslider-m.gif> <img src=qslider-w.gif>
+  
+  \sa QScrollBar QSpinBox
+  <a href="guibooks.html#fowler">GUI Design Handbook: Slider</a>
+*/
 
 
 /*!
@@ -108,6 +110,7 @@ QSlider::QSlider( int minValue, int maxValue, int step,
 {
     orient = orientation;
     init();
+    sliderVal = value;
 }
 
 
@@ -225,7 +228,7 @@ int QSlider::available() const
 	break;
     default:
     case MotifStyle:
-	a = (orient == Horizontal) ? width() -motifLength - 2*motifBorder 
+	a = (orient == Horizontal) ? width() -motifLength - 2*motifBorder
 	    : height() - motifLength - 2*motifBorder;
 	break;
     }
@@ -251,7 +254,7 @@ void QSlider::rangeChange()
 {
     int newPos = positionFromValue( value() );
     if ( newPos != sliderPos ) {
-	reallyMoveSlider( newPos ); 
+	reallyMoveSlider( newPos );
     }
 }
 
@@ -330,7 +333,7 @@ QRect QSlider::sliderRect() const
     switch ( style() ) {
     case WindowsStyle:
 	if (orient == Horizontal )
-	    r.setRect( sliderPos, tickOffset, 
+	    r.setRect( sliderPos, tickOffset,
 		       winLength, thickness()  );
 	else
 	    r.setRect ( tickOffset, sliderPos,
@@ -339,10 +342,10 @@ QRect QSlider::sliderRect() const
     default:
     case MotifStyle:
 	if (orient == Horizontal )
-	    r.setRect ( sliderPos + motifBorder, tickOffset + motifBorder, 
+	    r.setRect ( sliderPos + motifBorder, tickOffset + motifBorder,
 			motifLength, thickness() - 2 * motifBorder );
 	else
-	    r.setRect ( tickOffset + motifBorder, sliderPos + motifBorder, 
+	    r.setRect ( tickOffset + motifBorder, sliderPos + motifBorder,
 			thickness() - 2 * motifBorder, motifLength );
 	break;
     }
@@ -478,7 +481,7 @@ static void drawWinPointedSlider( QPainter *p,
 /*!
   Paints the slider button using painter \a p with size and
   position given by \a r. Reimplement this function to change the
-  look of the slider button.  
+  look of the slider button.
 */
 
 void QSlider::paintSlider( QPainter *p, const QRect &r )
@@ -527,12 +530,12 @@ void QSlider::reallyMoveSlider( int newPos )
     if ( orient == Horizontal ) {
 	if ( oldR.left() < newR.left() )
 	    oldR.setRight( QMIN ( oldR.right(), newR.left()));
-	else           //oldR.right() >= newR.right() 
+	else           //oldR.right() >= newR.right()
 	    oldR.setLeft( QMAX ( oldR.left(), newR.right()));
     } else {
 	if ( oldR.top() < newR.top() )
 	    oldR.setBottom( QMIN ( oldR.bottom(), newR.top()));
-	else           //oldR.bottom() >= newR.bottom() 
+	else           //oldR.bottom() >= newR.bottom()
 	    oldR.setTop( QMAX ( oldR.top(), newR.bottom()));
     }
     repaint( oldR );
@@ -566,9 +569,8 @@ void QSlider::drawWinGroove( QPainter *p, QCOORD c )
 
 void QSlider::paintEvent( QPaintEvent *e )
 {
-    
-    QPainter p;
-    p.begin( this );
+
+    QPainter p( this );
     QRect paintRect = e->rect();
     p.setClipRect( paintRect );
     QRect sliderR = sliderRect();
@@ -602,13 +604,13 @@ void QSlider::paintEvent( QPaintEvent *e )
 			     g, TRUE );
 	    p.fillRect( 0, 0, width(), tickOffset, g.background() );
 	    p.fillRect( 0, tickOffset + thickness(),
-			width(), height()/*###*/, g.background() ); 
+			width(), height()/*###*/, g.background() );
 	} else {
 	    qDrawShadePanel( &p, tickOffset, 0, thickness(), height(),
 			     g, TRUE );
 	    p.fillRect( 0, 0,  tickOffset, height(), g.background() );
 	    p.fillRect( tickOffset + thickness(), 0,
-			width()/*###*/, height(), g.background() ); 
+			width()/*###*/, height(), g.background() );
 	}
 
 	if ( hasFocus() ) {
@@ -622,7 +624,7 @@ void QSlider::paintEvent( QPaintEvent *e )
 	break;
     }
 
-    
+
     int interval = tickInt;
     if ( interval <= 0 ) {
 	interval = lineStep();
@@ -637,7 +639,6 @@ void QSlider::paintEvent( QPaintEvent *e )
 	avail -= tickOffset + thickness();
 	drawTicks( &p, tickOffset + thickness() + 1, avail - 2, interval );
     }
-    p.end();
 }
 
 
@@ -649,24 +650,20 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 {
     resetState();
     sliderStartVal = sliderVal;
-    if ( e->button() == MidButton ) {
-	int pos = goodPart( e->pos() );
-	moveSlider( pos - slideLength() / 2 );
-	return;
-    }
-    if ( e->button() != LeftButton )
-	return;
     QRect r = sliderRect();
 
-    if ( r.contains( e->pos() ) ) {
+    if ( e->button() == RightButton ) {
+	return;
+    } else if ( r.contains( e->pos() ) ) {
 	state = Dragging;
 	clickOffset = (QCOORD)( goodPart( e->pos() ) - sliderPos );
 	emit sliderPressed();
-    } else if ( funnyWindowsStyle && style() == WindowsStyle) {
+    } else if ( e->button() == MidButton ||
+		(funnyWindowsStyle && style() == WindowsStyle) ) {
 	int pos = goodPart( e->pos() );
 	moveSlider( pos - slideLength() / 2 );
 	state = Dragging;
-	clickOffset = slideLength() / 2;     
+	clickOffset = slideLength() / 2;
     } else if ( orient == Horizontal && e->pos().x() < r.left() //### goodPart
 		|| orient == Vertical && e->pos().y() < r.top() ) {
 	state = TimingDown;
@@ -674,7 +671,7 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 	if ( !timer )
 	    timer = new QTimer( this );
 	connect( timer, SIGNAL(timeout()), SLOT(repeatTimeout()) );
-	timer->start( thresholdTime, TRUE ); 
+	timer->start( thresholdTime, TRUE );
     } else if ( orient == Horizontal && e->pos().x() > r.right() //### goodPart
 		|| orient == Vertical && e->pos().y() > r.bottom() ) {
 	state = TimingUp;
@@ -682,7 +679,7 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 	if ( !timer )
 	    timer = new QTimer( this );
 	connect( timer, SIGNAL(timeout()), SLOT(repeatTimeout()) );
-	timer->start( thresholdTime, TRUE ); 
+	timer->start( thresholdTime, TRUE );
     }
 }
 
@@ -692,31 +689,24 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 
 void QSlider::mouseMoveEvent( QMouseEvent *e )
 {
-    
+    if ( state != Dragging )
+	return;
+
     if ( style() == WindowsStyle ) {
 	QRect r = rect();
 	if ( orientation() == Horizontal )
-	    r.setRect( r.x() - 20, r.y() - 30, r.width() + 40, r.height() + 60 );
+	    r.setRect( r.x() - 20, r.y() - 30,
+		       r.width() + 40, r.height() + 60 );
 	else
-	    r.setRect( r.x() - 30, r.y() - 20, r.width() + 60, r.height() + 40 );
-	if ( !r.contains( e->pos() ) ) { 
+	    r.setRect( r.x() - 30, r.y() - 20,
+		       r.width() + 60, r.height() + 40 );
+	if ( !r.contains( e->pos() ) ) {
 	    moveSlider( positionFromValue( sliderStartVal) );
 	    return;
 	}
     }
 
-    if ( (e->state() & MidButton) ) { 		// middle button wins
-	int pos = goodPart( e->pos() );
-	moveSlider( pos - slideLength() / 2 );
-	return;	
-    }
-    if ( !(e->state() & LeftButton) )
-	return;					// left mouse button is up
-    if ( state != Dragging )
-	return;
-
     int pos = goodPart( e->pos() );
-    
     moveSlider( pos - clickOffset );
 }
 
@@ -725,10 +715,8 @@ void QSlider::mouseMoveEvent( QMouseEvent *e )
   Handles mouse release events for the slider.
 */
 
-void QSlider::mouseReleaseEvent( QMouseEvent *e )
+void QSlider::mouseReleaseEvent( QMouseEvent * )
 {
-    if ( !(e->button() & LeftButton) )
-	return;
     resetState();
 }
 
@@ -742,7 +730,7 @@ void QSlider::focusInEvent( QFocusEvent * )
 }
 
 /*!
-  Moves the left (or top) edge of the slider to position 
+  Moves the left (or top) edge of the slider to position
   \a pos. Performs snapping.
 */
 
@@ -767,7 +755,7 @@ void QSlider::moveSlider( int pos )
     default:
     case MotifStyle:
 	break;
-    }	    
+    }	
 
     if ( sliderPos != newPos )
 	reallyMoveSlider( newPos );
@@ -796,7 +784,7 @@ void QSlider::resetState()
     case Idle:
 	break;
     default:
-	warning("QSlider: in wrong state");
+	warning("QSlider: (%s) in wrong state", name( "unnamed" ) );
     }
     state = Idle;
 }
@@ -850,7 +838,7 @@ void QSlider::keyPressEvent( QKeyEvent *e )
   Returns the length of the slider.
 */
 
-int QSlider::slideLength() const 
+int QSlider::slideLength() const
 {
     switch ( style() ) {
     case WindowsStyle:
@@ -904,7 +892,7 @@ void QSlider::repeatTimeout()
 	connect( timer, SIGNAL(timeout()), SLOT(subtractStep()) );
     else if ( state == TimingUp )
 	connect( timer, SIGNAL(timeout()), SLOT(addStep()) );
-    timer->start( repeatTime, FALSE );   
+    timer->start( repeatTime, FALSE );
 }
 
 
@@ -917,7 +905,7 @@ int QSlider::goodPart( const QPoint &p ) const
     return (orient == Horizontal) ?  p.x() : p.y();
 }
 
-/*!  
+/*!
   Returns the recommended size of the slider. Only the thickness is
   relevant.
 */
@@ -942,8 +930,8 @@ QSize QSlider::sizeHint() const
 
 
 /*!
-  Returns the number of pixels to use for the business part of the 
-  slider (i.e. the non-tickmark portion). The remaining space is shared 
+  Returns the number of pixels to use for the business part of the
+  slider (i.e. the non-tickmark portion). The remaining space is shared
   equally between the tickmark regions. This function and  sizeHint()
   are closely related; if you change one, you almost certainly
   have to change the other.
@@ -972,9 +960,9 @@ int QSlider::thickness() const
 }
 
 
-/*!  
+/*!
   Using \a p, draws tickmarks at a distance of \a d from the edge
-  of the widget, using \a w pixels and with an interval of \a i.  
+  of the widget, using \a w pixels and with an interval of \a i.
 */
 
 void QSlider::drawTicks( QPainter *p, int d, int w, int i ) const
@@ -994,7 +982,7 @@ void QSlider::drawTicks( QPainter *p, int d, int w, int i ) const
 
 
 /*!
-  Sets the way tickmarks are displayed by the slider. \a s can take 
+  Sets the way tickmarks are displayed by the slider. \a s can take
   the following values:
   <ul>
   <li> \c NoMarks
@@ -1031,9 +1019,9 @@ void QSlider::setTickmarks( TickSetting s )
 */
 
 /*!
-  Sets the interval between tickmarks to \a i. This is a value interval, 
+  Sets the interval between tickmarks to \a i. This is a value interval,
   not a pixel interval. If \a i is 0, the slider
-  will choose between lineStep() and pageStep(). The initial value of 
+  will choose between lineStep() and pageStep(). The initial value of
   tickInterval() is 0.
   \sa tickInterval(), QRangeControl::lineStep(), QRangeControl::pageStep()
 */
