@@ -1,20 +1,21 @@
 /****************************************************************************
-** $Id: qtabbar.cpp,v 2.29 1997/09/26 10:57:41 eiriken Exp $
+** $Id: qtabbar.cpp,v 2.36 1998/07/03 00:09:53 hanord Exp $
 **
 ** Implementation of QTabBar class
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
 ** IMPORTANT NOTE: You may NOT copy this file or any part of it into
 ** your own programs or libraries.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
@@ -24,9 +25,6 @@
 
 #include <ctype.h>
 
-RCSTAG("$Id: qtabbar.cpp,v 2.29 1997/09/26 10:57:41 eiriken Exp $");
-
-
 QTab::~QTab()
 {
     // gcc says: just give me a vtbl and I'll be satisfied
@@ -48,6 +46,8 @@ struct QTabPrivate {
   \brief The QTabBar class provides a tab bar, for use in e.g. tabbed
   dialogs.
 
+  \ingroup realwidgets
+
   The class is quite simple; it draws the tabs in one of four shapes
   and emits a signal when one is selected.  It can be subclassed to
   tailor the look and feel.
@@ -272,17 +272,13 @@ void QTabBar::paint( QPainter * p, QTab * t, bool selected ) const
 	    p->drawLine( r.left()+1, r.bottom(), r.right()-2, r.bottom() );
 	    p->drawLine( r.left()+1, r.bottom(), r.left()+1, r.top()+2 );
 	    p->setPen( colorGroup().light() );
-	    QFont bold( font() );
-	    bold.setWeight( QFont::Bold );
-	    p->setFont( bold );
 	} else {
 	    p->setPen( colorGroup().light() );
 	    p->drawLine( r.left(), r.bottom(), r.right(), r.bottom() );
 	    r.setRect( r.left() + 2, r.top() + 2,
 		       r.width() - 4, r.height() - 2 );
-	    p->setFont( font() );
 	}
-    
+
 	p->drawLine( r.left(), r.bottom(), r.left(), r.top() + 2 );
 	p->drawPoint( r.left()+1, r.top() + 1 );
 	p->drawLine( r.left()+2, r.top(),
@@ -290,25 +286,21 @@ void QTabBar::paint( QPainter * p, QTab * t, bool selected ) const
 
 	p->setPen( colorGroup().dark() );
 	p->drawLine( r.right() - 1, r.top() + 2,
-		     r.right() - 1, r.bottom() - 1 ); 
+		     r.right() - 1, r.bottom() - 1 );
 	p->setPen( colorGroup().foreground() );
 	p->drawPoint( r.right() - 1, r.top() + 1 );
-	p->drawLine( r.right(), r.top() + 2, r.right(), r.bottom() - 1 ); 
+	p->drawLine( r.right(), r.top() + 2, r.right(), r.bottom() - 1 );
     } else if ( d->s == RoundedBelow ) {
 	if ( selected ) {
 	    p->setPen( colorGroup().background() );
 	    p->drawLine( r.left()+1, r.top(), r.right()-2, r.top() );
 	    p->drawLine( r.left()+1, r.top(), r.left()+1, r.bottom()-2 );
 	    p->setPen( colorGroup().dark() );
-	    QFont bold( font() );
-	    bold.setWeight( QFont::Bold );
-	    p->setFont( bold );
 	} else {
 	    p->setPen( colorGroup().dark() );
 	    p->drawLine( r.left(), r.top(), r.right(), r.top() );
 	    r.setRect( r.left() + 2, r.top(),
 		       r.width() - 4, r.height() - 2 );
-	    p->setFont( font() );
 	}
 
 	p->drawLine( r.right() - 1, r.top(),
@@ -317,7 +309,7 @@ void QTabBar::paint( QPainter * p, QTab * t, bool selected ) const
 	p->drawLine( r.right() - 2, r.bottom() - 1,
 		     r.left() + 1, r.bottom() - 1 );
 	p->drawPoint( r.left() + 1, r.bottom() - 2 );
-    
+
 	p->setPen( colorGroup().foreground() );
 	p->drawLine( r.right(), r.top(),
 		     r.right(), r.bottom() - 1 );
@@ -341,7 +333,7 @@ void QTabBar::paint( QPainter * p, QTab * t, bool selected ) const
 	a.setPoint( 3, x++, y );
 	a.setPoint( 3, x++, y++ );
 	a.setPoint( 4, x, y );
-
+	
 	int i;
 	int right = t->r.width() - 1;
 	for ( i = 0; i < 5; i++ )
@@ -360,6 +352,8 @@ void QTabBar::paint( QPainter * p, QTab * t, bool selected ) const
 	p->setBrush( NoBrush );
     }
 
+    p->setFont( font() );
+
     int w = p->fontMetrics().width( t->label ) + 4;
     int h = p->fontMetrics().height() + 4;
     paintLabel( p, QRect( r.left() + (r.width()-w)/2 - 3,
@@ -412,9 +406,7 @@ void QTabBar::paintLabel( QPainter* p, const QRect& br,
 
 void QTabBar::paintEvent( QPaintEvent * e )
 {
-    QPainter p;
-
-    p.begin( this );
+    QPainter p( this );
     p.setClipping( TRUE );
     p.setClipRect( e->rect() );
 
@@ -427,7 +419,6 @@ void QTabBar::paintEvent( QPaintEvent * e )
 	t = n;
     } while ( t != 0 );
 
-    p.end();
 }
 
 
@@ -521,7 +512,7 @@ void QTabBar::show()
   the user can see the relevant page, or that the tab \link
   isTabEnabled() is enabled.\endlink
 
-  When when you need to display something, the return value from this
+  When you need to display something, the return value from this
   function represents the best page to display.  That's all.
 
   \sa selected()
