@@ -1,34 +1,32 @@
 /****************************************************************************
-** $Id: qtooltip.cpp,v 2.23 1997/03/26 11:15:43 aavit Exp $
+** $Id: qtooltip.cpp,v 2.51 1998/07/03 00:09:54 hanord Exp $
 **
 ** Tool Tips (or Balloon Help) for any widget or rectangle
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of Qt Free Edition, version 1.40.
 **
 ** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** and distribution terms, or http://www.troll.no/free-license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** Please see http://www.troll.no/pricing.html for information about 
+** Qt Professional Edition, which is this same library but with a
+** license which allows creation of commercial/proprietary software.
 **
 *****************************************************************************/
 
 #include "qtooltip.h"
-#include "qstring.h"
-#include "qwidget.h"
-#include "qcolor.h"
 #include "qlabel.h"
-#include "qpoint.h"
-#include "qapp.h"
-
-RCSTAG("$Id: qtooltip.cpp,v 2.23 1997/03/26 11:15:43 aavit Exp $");
-
+#include "qptrdict.h"
+#include "qapplication.h"
 
 // Magic value meaning an entire widget - if someone tries to insert a
 // tool tip on this part of a widget it will be interpreted as the
-// entire widget
+// entire widget.
 
 static inline QRect entireWidget()
 {
@@ -36,7 +34,8 @@ static inline QRect entireWidget()
 		  QCOORD_MAX-QCOORD_MIN, QCOORD_MAX-QCOORD_MIN );
 }
 
-// internal class - don't touch
+
+// Internal class - don't touch
 
 class QTipManager : public QObject
 {
@@ -47,17 +46,17 @@ public:
 
     struct Tip
     {
-	QRect rect;
-	QString text;
-	QString groupText;
-	QToolTipGroup * group;
-	QToolTip * tip;
-	bool autoDelete;
-	Tip * next;
+	QRect		rect;
+	QString		text;
+	QString	        groupText;
+	QToolTipGroup  *group;
+	QToolTip       *tip;
+	bool	        autoDelete;
+	Tip	       *next;
     };
 
     bool    eventFilter( QObject * o, QEvent * e );
-    void    add( QWidget *, const QRect &, const char *, 
+    void    add( QWidget *, const QRect &, const char *,
 		 QToolTipGroup *, const char *, QToolTip *, bool );
     void    remove( QWidget *, const QRect & );
     void    remove( QWidget * );
@@ -76,12 +75,15 @@ protected:
 private:
     QTimer  wakeUp;
     QTimer  fallAsleep;
+    QTimer  leaveWindow;
 
-    QIntDict<Tip> *tips;
-    QLabel *label;
-    QPoint  pos;
-    QWidget *widget;
-    QTipManager::Tip *currentTip;
+    QPtrDict<Tip>    *tips;
+    QLabel	     *label;
+    QPoint	      pos;
+    QWidget	     *widget;
+    Tip *currentTip;
+    Tip *previousTip;
+    bool isApplicationFilter;
 };
 
 
@@ -91,7 +93,7 @@ private:
 ** QTipManager meta object code from reading C++ file 'qtooltip.cpp'
 **
 ** Created: Mon Mar 17 12:39:34 1997
-**      by: The Qt Meta Object Compiler ($Revision: 2.23 $)
+**      by: The Qt Meta Object Compiler ($Revision: 2.51 $)
 **
 ** WARNING! All changes made in this file will be lost!
 *****************************************************************************/
@@ -102,7 +104,7 @@ private:
 #error Moc format conflict - please regenerate all moc files
 #endif
 
-#include <qmetaobj.h>
+#include <qmetaobject.h>
 
 
 const char *QTipManager::className() const
@@ -145,37 +147,56 @@ void QTipManager::initMetaObject()
 // moc code ends here
 
 
-// here is the only QTipManager object:
-static QTipManager * tipManager;
+// We have a global, internal QTipManager object
 
+static QTipManager *tipManager	  = 0;
+static bool	    initializedTM = FALSE;
 
 static void cleanupTipManager()
 {
     delete tipManager;
     tipManager = 0;
+    initializedTM = FALSE;
+}
+
+static void initTipManager()
+{
+    if ( !tipManager ) {
+	tipManager = new QTipManager;
+	CHECK_PTR( tipManager );
+    }
+    if ( !initializedTM ) {
+	initializedTM = TRUE;
+	qAddPostRoutine( cleanupTipManager );
+    }
 }
 
 
 QTipManager::QTipManager()
-    : QObject( 0, "tool tip workhorse object" )
+    : QObject( 0, "toolTipManager" )
 {
-    initMetaObject();
-    tips = new QIntDict<QTipManager::Tip>( 313 );
+    tips = new QPtrDict<QTipManager::Tip>( 313 );
     currentTip = 0;
+    previousTip = 0;
     label = 0;
-
+    isApplicationFilter = FALSE;
     connect( &wakeUp, SIGNAL(timeout()), SLOT(showTip()) );
     connect( &fallAsleep, SIGNAL(timeout()), SLOT(hideTip()) );
+    connect( &leaveWindow, SIGNAL(timeout()), SLOT(hideTip()) );
 }
 
 
 QTipManager::~QTipManager()
 {
-    if ( tips ) {
-	QIntDictIterator<QTipManager::Tip> i( *tips );
-	QTipManager::Tip * t, * n;
-	long k;
+    if ( isApplicationFilter && !qApp->closingDown() ) {
+	qApp->setGlobalMouseTracking( FALSE );
+	qApp->removeEventFilter( tipManager );
+    }
 
+    if ( tips ) {
+	QPtrDictIterator<QTipManager::Tip> i( *tips );
+	QTipManager::Tip *t, *n;
+	void *k;
 	while( (t = i.current()) != 0 ) {
 	    k = i.currentKey();
 	    ++i;
@@ -192,42 +213,44 @@ QTipManager::~QTipManager()
     delete label;
 }
 
-void QTipManager::add( QWidget * w, const QRect & r, const char * s,
-		       QToolTipGroup * g, const char * gs,
-		       QToolTip * tt, bool a )
+void QTipManager::add( QWidget *w, const QRect &r, const char *s,
+		       QToolTipGroup *g, const char *gs,
+		       QToolTip *tt, bool a )
 {
-    QTipManager::Tip * h = (*tips)[ (long)w ];
-    QTipManager::Tip * t = new QTipManager::Tip;
-    if ( h ) {
-	tips->take( (long)w );
-    } else {
-	w->setMouseTracking( TRUE );
-	w->installEventFilter( tipManager );
-    }
+    QTipManager::Tip *h = (*tips)[ w ];
+    QTipManager::Tip *t = new QTipManager::Tip;
+    if ( h )
+	tips->take( w );
     t->next = h;
 
     t->tip = tt;
     t->autoDelete = a;
-
     t->text = s;
     t->rect = r;
     t->groupText = gs;
     t->group = g;
 
-    tips->insert( (long)w, t );
+    tips->insert( w, t );
+    connect( w, SIGNAL(destroyed()), this, SLOT(clientWidgetDestroyed()) );
     if ( a ) {
-	showTip();
-	tips->take( (long)w );
+	if ( t->rect.contains( pos ) )
+	    showTip();
+	tips->take( w );
 	if( t->next )
-	    tips->insert( (long)w, t->next );
+	    tips->insert( w, t->next );
 	t->next = 0;
     }
+    if ( !isApplicationFilter && qApp ) {
+	isApplicationFilter = TRUE;
+	qApp->installEventFilter( tipManager );
+	qApp->setGlobalMouseTracking( TRUE );
+    }
 }
 
 
 void QTipManager::remove( QWidget *w, const QRect & r )
 {
-    QTipManager::Tip * t = (*tips)[ (long)w ];
+    QTipManager::Tip *t = (*tips)[ w ];
     if ( t == 0 )
 	return;
 
@@ -235,20 +258,20 @@ void QTipManager::remove( QWidget *w, const QRect & r )
 	hideTip();
 
     if ( t->rect == r ) {
-	(void) tips->take( (long)w );
+	tips->take( w );
 	if ( t->next ) {
-	    tips->insert( (long)w, t->next );
+	    tips->insert( w, t->next );
 	} else {
 	    // ### w->setMouseTracking( FALSE );
 	    // ### need to disable sometimes
-	    w->removeEventFilter( tipManager );
+	    // w->removeEventFilter( tipManager );
 	}
 	delete t;
     } else {
 	while( t->next && t->next->rect != r )
 	    t = t->next;
 	if ( t->next ) {
-	    QTipManager::Tip * d = t->next;
+	    QTipManager::Tip *d = t->next;
 	    t->next = t->next->next;
 	    delete d;
 	}
@@ -278,7 +301,7 @@ void QTipManager::labelDestroyed()
 
 void QTipManager::clientWidgetDestroyed()
 {
-    const QObject * s = sender();
+    const QObject *s = sender();
     if ( s )
 	remove( (QWidget*) s );
 }
@@ -286,11 +309,11 @@ void QTipManager::clientWidgetDestroyed()
 
 void QTipManager::remove( QWidget *w )
 {
-    QTipManager::Tip * t = (*tips)[ (long)w ];
+    QTipManager::Tip *t = (*tips)[ w ];
     if ( t == 0 )
 	return;
 
-    (void) tips->take( (long)w );
+    tips->take( w );
     QTipManager::Tip * d;
     while ( t ) {
 	if ( t == currentTip )
@@ -307,11 +330,10 @@ void QTipManager::remove( QWidget *w )
 }
 
 
-void QTipManager::removeFromGroup( QToolTipGroup * g )
+void QTipManager::removeFromGroup( QToolTipGroup *g )
 {
-    QIntDictIterator<QTipManager::Tip> i( *tips );
-    QTipManager::Tip * t;
-
+    QPtrDictIterator<QTipManager::Tip> i( *tips );
+    QTipManager::Tip *t;
     while( (t = i.current()) != 0 ) {
 	++i;
 	while ( t ) {
@@ -324,24 +346,33 @@ void QTipManager::removeFromGroup( QToolTipGroup * g )
 
 
 
-bool QTipManager::eventFilter( QObject * o, QEvent * e )
+bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 {
-    // avoid dumping core in case of application madness
-    if ( !tips || !e || !o || !o->isWidgetType() )
+    // avoid dumping core in case of application madness, and return
+    // quickly for some common but irrelevant events
+    if ( !tips || !e || !obj || !obj->isWidgetType() ||
+	 e->type() == Event_Timer || e->type() == Event_Paint ||
+	 e->type() == Event_SockAct )
 	return FALSE;
-    QWidget * w = (QWidget *)o;
+    QWidget *w = (QWidget *)obj;
 
-    QTipManager::Tip * t = (*tips)[ (long int)w ];
-    if ( !t )
+    QTipManager::Tip *t = 0;
+    while( w && !t ) {
+	t = (*tips)[ w ];
+	if ( !t )
+	    w = w->testWFlags(WType_TopLevel) ? 0 : w->parentWidget();
+    }
+
+    if ( !t ) {
+	if ( e->type() >= Event_MouseButtonPress &&
+	     e->type() <= Event_Leave )
+	    wakeUp.stop();
 	return FALSE;
+    }
 
     // with that out of the way, let's get down to action
 
     switch( e->type() ) {
-    case Event_Timer: // fall through
-    case Event_Paint:
-	// no processing at all
-	break;
     case Event_MouseButtonPress:
     case Event_MouseButtonRelease:
     case Event_MouseButtonDblClick:
@@ -350,15 +381,20 @@ bool QTipManager::eventFilter( QObject * o, QEvent * e )
 	// input - turn off tool tip mode
 	hideTip();
 	fallAsleep.stop();
+	leaveWindow.stop();
 	break;
     case Event_MouseMove:
 	{ // a whole scope just for one variable
 	    QMouseEvent * m = (QMouseEvent *)e;
 
-	    if ( currentTip && !currentTip->rect.contains( m->pos() ) ) {
+	    QPoint mousePos( m->pos() );
+	    mousePos = ((QWidget*)obj)->mapToGlobal( mousePos );
+	    mousePos = w->mapFromGlobal( mousePos );
+
+	    if ( currentTip && !currentTip->rect.contains( mousePos ) ) {
 		hideTip();
 		if ( m->state() == 0 )
-		    return TRUE;
+		    return FALSE;
 	    }
 
 	    wakeUp.stop();
@@ -367,19 +403,29 @@ bool QTipManager::eventFilter( QObject * o, QEvent * e )
 		    return TRUE;
 		else if ( fallAsleep.isActive() )
 		    wakeUp.start( 100, TRUE );
-		else
-		    wakeUp.start( 1000, TRUE );
+		else {
+		    previousTip = 0;
+		    wakeUp.start( 700, TRUE );
+		}
 		widget = w;
-		pos = m->pos();
-		return TRUE;
+		pos = mousePos;
+		return FALSE;
 	    } else {
 		hideTip();
 	    }
 	}
 	break;
-    case Event_Enter: // fall through
+    case Event_Enter:
+	if ( label && label->isVisible() && w == widget )
+	    leaveWindow.stop();
+	else if ( w ) // test in event_leave below should always hit first
+	    wakeUp.stop();
+	break;
     case Event_Leave:
-	hideTip();
+	if ( label && label->isVisible() )
+	    leaveWindow.start( 50, TRUE );
+	else if ( widget != w )
+	    wakeUp.stop();
 	break;
     default:
 	hideTip();
@@ -392,18 +438,15 @@ bool QTipManager::eventFilter( QObject * o, QEvent * e )
 
 void QTipManager::showTip()
 {
-    if ( widget == 0 ||
-	 QApplication::widgetAt(widget->mapToGlobal(pos),TRUE) != widget ) {
+    if ( widget == 0 ) {
 	widget = 0;
 	return;
     }
 
-    QTipManager::Tip * t = (*tips)[ (long)widget ];
-
+    QTipManager::Tip *t = (*tips)[ widget ];
     while ( t && !t->rect.contains( pos ) )
 	t = t->next;
-
-    if ( t == 0 )
+    if ( t == 0 || t == previousTip )
 	return;
 
     if ( t->tip ) {
@@ -414,15 +457,13 @@ void QTipManager::showTip()
     if ( label ) {
 	label->setText( t->text );
     } else {
-	label = new QLabel( 0, "tool tip tip",
+	label = new QLabel( 0, "toolTipTip",
 			    WStyle_Customize | WStyle_NoBorder | WStyle_Tool );
 	CHECK_PTR( label );
 	connect( label, SIGNAL(destroyed()), SLOT(labelDestroyed()) );
+	label->setFont( QToolTip::font() );
+	label->setPalette( QToolTip::palette() );
 	label->setText( t->text );
-	QColorGroup g( black, QColor(255,255,220),
-		       QColor(96,96,96), black, black,
-		       black, QColor(255,255,220) );
-	label->setPalette( QPalette( g, g, g ) );
 	if ( QApplication::style() == MotifStyle )
 	    label->setFrameStyle( QFrame::Plain | QFrame::Box );
 	else
@@ -436,16 +477,18 @@ void QTipManager::showTip()
     if ( p.x() + label->width() > QApplication::desktop()->width() )
 	p.setX( QApplication::desktop()->width() - label->width() );
     if ( p.y() + label->height() > QApplication::desktop()->height() )
-	p.setY( pos.y() - 20 - label->height() );
+	p.setY( p.y() - 20 - label->height() );
     label->move( p );
     label->show();
     label->raise();
 
-    fallAsleep.start( 4000, TRUE );
+    fallAsleep.start( 5000, TRUE );
+    leaveWindow.stop();
 
     if ( t->group && !t->groupText.isEmpty() )
 	emit t->group->showTip( t->groupText );
     currentTip = t;
+    previousTip = 0;
 }
 
 
@@ -453,7 +496,7 @@ void QTipManager::hideTip()
 {
     if ( label && label->isVisible() ) {
 	label->hide();
-	fallAsleep.start( 5000, TRUE );
+	fallAsleep.start( 10000, TRUE );
 	if ( currentTip && currentTip->group )
 	    emit currentTip->group->removeTip();
     } else if ( wakeUp.isActive() ) {
@@ -462,17 +505,22 @@ void QTipManager::hideTip()
 
     if ( currentTip && currentTip->autoDelete )
 	delete currentTip;
+    else
+	previousTip = currentTip;
 
     currentTip = 0;
     widget = 0;
 }
 
 
-/*! \class QToolTip qtooltip.h
+/*!
+  \class QToolTip qtooltip.h
 
   \brief The QToolTip class provides tool tips (sometimes called
   balloon help) for any widget or rectangular part of a widget.
 
+  \ingroup application
+
   The tip is a short, one-line text reminding the user of the widget's
   or rectangle's function.  It is drawn immediately below the region,
   in a distinctive black on yellow combination.  In Motif style, Qt's
@@ -492,16 +540,16 @@ void QTipManager::hideTip()
   a key, lets the mouse rest for five seconds, or moves the mouse
   outside \e all tip-equpped regions for at least a second.
 
-  There are no less than three APIs for QToolTip: <ol> <li> Adding a
-  tip to an entire widget. <li> Adding a tip to a fixed rectangle
-  within a widget. <li> Adding a tip to a dynamic rectangle within a
-  widget. </ol>
+  The QToolTip class can be used in three different ways: <ol> <li>
+  Adding a tip to an entire widget. <li> Adding a tip to a fixed
+  rectangle within a widget. <li> Adding a tip to a dynamic rectangle
+  within a widget. </ol>
 
   To add a tip to a widget, call the static function QToolTip::add()
   with the widget and tip as arguments:
 
   \code
-  QToolTip::add( quitButton, "Leave the application" );
+    QToolTip::add( quitButton, "Leave the application" );
   \endcode
 
   This is the simplest and most common use of QToolTip.  The tip will
@@ -509,45 +557,133 @@ void QTipManager::hideTip()
   remove it yourself, too:
 
   \code
-  QToolTip::remove( quitButton );
+    QToolTip::remove( quitButton );
   \endcode
 
-  You can also display another text (typically in a status bar),
-  courtesy of QToolTipGroup.  This example assumes that \e g is a
-  <code>QToolTipGroup *</code> and already connected to the
-  appropriate status bar:
+  You can also display another text (typically in a \link QStatusBar
+  status bar),\endlink courtesy of QToolTipGroup.  This example
+  assumes that \e g is a <code>QToolTipGroup *</code> and already
+  connected to the appropriate status bar:
 
   \code
-  QToolTip::add( quitButton, "Leave the application", g,
-                 "Leave the application, without asking for confirmation" );
-  QToolTip::add( closeButton, "Close this window", g,
-                 "Close this window, without asking for confirmation" );
+    QToolTip::add( quitButton, "Leave the application", g,
+                   "Leave the application, without asking for confirmation" );
+    QToolTip::add( closeButton, "Close this window", g,
+                   "Close this window, without asking for confirmation" );
   \endcode
 
   To add a tip to a fixed rectangle within a widget, call the static
   function QToolTip::add() with the widget, rectangle and tip as
-  arguments.  (See the tellme/tellme.cpp example.)  Again, you can supply a
+  arguments.  (See the tooltip/tooltip.cpp example.)  Again, you can supply a
   QToolTipGroup * and another text if you want.
 
   Both of the above are one-liners and cover the vast majority of
-  cases.  The third and most general API uses a pure virtual function
-  to decide whether to pop up a tool tip.  The tellme/tellme.cpp
-  example demonstrates this too.  This mode can be used to implement
-  e.g. tips for text that can move as the user scrolls.
+  cases.  The third and most general way to use QToolTip uses a pure
+  virtual function to decide whether to pop up a tool tip.  The
+  tooltip/tooltip.cpp example demonstrates this too.  This mode can be
+  used to implement e.g. tips for text that can move as the user
+  scrolls.
 
-  To use this API, you need to subclass QToolTip and reimplement
-  maybeTip().  maybeTip() will be called when there's a chance that a
-  tip should pop up.  It must decide whether to show a tip, and
-  possibly call add() with the rectangle the tip applies to, the tip's
-  text and optionally the QToolTipGroup details.  The tip will
-  disappear once the mouse moves outside the rectangle you supply, and
-  \e not \e reappear - maybeTip() will be called again if the user
-  lets the mouse rest within the same rectangle again.  You can
-  forcibly remove the tip by calling remove() with no arguments.  This
-  is handy if the widget scrolls.
+  To use QToolTip like this, you need to subclass QToolTip and
+  reimplement maybeTip().  maybeTip() will be called when there's a
+  chance that a tip should pop up.  It must decide whether to show a
+  tip, and possibly call add() with the rectangle the tip applies to,
+  the tip's text and optionally the QToolTipGroup details.  The tip
+  will disappear once the mouse moves outside the rectangle you
+  supply, and \e not \e reappear - maybeTip() will be called again if
+  the user lets the mouse rest within the same rectangle again.  You
+  can forcibly remove the tip by calling remove() with no arguments.
+  This is handy if the widget scrolls.
+
+  \sa QStatusBar QWhatsThis QToolTipGroup
+  <a href="guibooks.html#fowler">GUI Design Handbook: Tool Tip</a>
 */
 
 
+/*
+  Global settings for tool tips.
+*/
+
+QFont	    *QToolTip::ttFont    = 0;
+QPalette    *QToolTip::ttPalette = 0;
+
+
+void QToolTip::initialize()
+{
+    if ( ttFont )				// already initialized
+	return;
+    qAddPostRoutine( cleanup );
+    ttFont = new QFont;
+    CHECK_PTR( ttFont );
+    QColorGroup cg( black, QColor(255,255,220),
+		    QColor(96,96,96), black, black,
+		    black, QColor(255,255,220) );
+    ttPalette = new QPalette( cg, cg, cg );
+    CHECK_PTR( ttPalette );
+}
+
+
+void QToolTip::cleanup()
+{
+    delete ttFont;
+    ttFont = 0;
+    delete ttPalette;
+    ttPalette = 0;
+}
+
+
+/*!
+  Returns the font common to all tool tips.
+  \sa setFont()
+*/
+
+QFont QToolTip::font()
+{
+    if ( !ttFont )
+	initialize();
+    return *ttFont;
+}
+
+
+/*!
+  Sets the font for all tool tips to \a font.
+  \sa font()
+*/
+
+void QToolTip::setFont( const QFont &font )
+{
+    if ( !ttFont )
+	initialize();
+    *ttFont = font;
+}
+
+
+/*!
+  Returns the palette common to all tool tips.
+  \sa setPalette()
+*/
+
+QPalette QToolTip::palette()
+{
+    if ( !ttPalette )
+	initialize();
+    return *ttPalette;
+}
+
+
+/*!
+  Sets the palette for all tool tips to \a palette.
+  \sa palette()
+*/
+
+void QToolTip::setPalette( const QPalette &palette )
+{
+    if ( !ttPalette )
+	initialize();
+    *ttPalette = palette;
+}
+
+
 /*!
   Creates a tool tip object.  This is necessary only if you need tool
   tips on regions that can move within the widget (most often because
@@ -563,8 +699,7 @@ QToolTip::QToolTip( QWidget * parent, QToolTipGroup * group )
 {
     p = parent;
     g = group;
-    if ( !tipManager )
-	tipManager = new QTipManager();
+    initTipManager();
     tipManager->add( p, entireWidget(), 0, g, 0, this, FALSE );
 }
 
@@ -577,12 +712,10 @@ QToolTip::QToolTip( QWidget * parent, QToolTipGroup * group )
   suitable for adding tool tips to buttons, check boxes, combo boxes
   and so on.
 */
-void QToolTip::add( QWidget * widget, const char * text )
+
+void QToolTip::add( QWidget *widget, const char *text )
 {
-    if ( !tipManager ) {
-	tipManager = new QTipManager;
-	qAddPostRoutine( cleanupTipManager );
-    }
+    initTipManager();
     tipManager->add( widget, entireWidget(), text, 0, 0, 0, FALSE );
 }
 
@@ -597,14 +730,14 @@ void QToolTip::add( QWidget * widget, const char * text )
   Normally, \a longText is shown in a status bar or similar.
 */
 
-void QToolTip::add( QWidget * widget, const char * text,
-		    QToolTipGroup * group, const char * longText )
+void QToolTip::add( QWidget *widget, const char *text,
+		    QToolTipGroup *group, const char *longText )
 {
-    if ( !tipManager )
-	tipManager = new QTipManager();
+    initTipManager();
     tipManager->add( widget, entireWidget(), text, group, longText, 0, FALSE );
 }
 
+
 /*!
   Remove the tool tip from \e widget.
 
@@ -618,7 +751,7 @@ void QToolTip::remove( QWidget * widget )
 	tipManager->remove( widget, entireWidget() );
 }
 
-/*!  
+/*!
   Adds a tool tip to a fixed rectangle within \a widget.  \a text is
   the text shown in the tool tip.  QToolTip makes a deep copy of this
   string.
@@ -626,8 +759,7 @@ void QToolTip::remove( QWidget * widget )
 
 void QToolTip::add( QWidget * widget, const QRect & rect, const char * text )
 {
-    if ( !tipManager )
-	tipManager = new QTipManager();
+    initTipManager();
     tipManager->add( widget, rect, text, 0, 0, 0, FALSE );
 }
 
@@ -643,15 +775,15 @@ void QToolTip::add( QWidget * widget, const QRect & rect, const char * text )
   Normally, \a longText is shown in a status bar or similar.
 */
 
-void QToolTip::add( QWidget * widget, const QRect & rect,
-		    const char * text,
-		    QToolTipGroup * group, const char * groupText )
+void QToolTip::add( QWidget *widget, const QRect &rect,
+		    const char *text,
+		    QToolTipGroup *group, const char *groupText )
 {
-    if ( !tipManager )
-	tipManager = new QTipManager();
+    initTipManager();
     tipManager->add( widget, rect, text, group, groupText, 0, FALSE );
 }
 
+
 /*!
   Remove the tool tip for \e rect from \e widget.
 
@@ -666,19 +798,36 @@ void QToolTip::remove( QWidget * widget, const QRect & rect )
 }
 
 
-/*! \fn virtual void QToolTip::maybeTip( const QPoint & p);
+/*!
+  \fn virtual void QToolTip::maybeTip( const QPoint & p);
 
   This pure virtual function is half of the most versatile interface
   QToolTip offers.
 
   It is called when there is a chance that a tool tip should be shown,
-  and must decide whether there is a tool tip for the point \a p and
-  what rectangle
+  and must decide whether there is a tool tip for the point \a p in
+  the widget this QToolTip object relates to.
+
+  \p is given in that widget's local coordinates.  Most maybeTip()
+  implementation will be of the form:
+  
+  \code
+    if ( \<something\> ) {
+	tip( \<something\>, \<something\> );
+    }
+  \endcode
+
+  The first argument to tip() (a rectangle) should include the \a p,
+  or QToolTip, the user or both can be confused.
+
+  \sa tip()
 */
 
 
-/*! Pop up a tip saying \a text right now, and remove that tip once
-  the cursor moves out of rectangle \a rect.
+/*!
+  Pop up a tip saying \a text right now, and remove that tip once the
+  cursor moves out of rectangle \a rect (which is given in the
+  coordinate system of the widget this QToolTip relates to).
 
   The tip will not come back if the cursor moves back; your maybeTip()
   has to reinstate it each time.
@@ -686,12 +835,13 @@ void QToolTip::remove( QWidget * widget, const QRect & rect )
 
 void QToolTip::tip( const QRect & rect, const char * text )
 {
-    if ( !tipManager )
-	tipManager = new QTipManager();
+    initTipManager();
     tipManager->add( parentWidget(), rect, text, 0, 0, 0, TRUE );
 }
 
-/*! Pop up a tip saying \a text right now, and remove that tip once
+
+/*!
+  Pop up a tip saying \a text right now, and remove that tip once
   the cursor moves out of rectangle \a rect.
 
   The tip will not come back if the cursor moves back; your maybeTip()
@@ -701,13 +851,14 @@ void QToolTip::tip( const QRect & rect, const char * text )
 void QToolTip::tip( const QRect & rect, const char * text,
 		    const char * groupText )
 {
-    if ( !tipManager )
-	tipManager = new QTipManager();
+    initTipManager();
     tipManager->add( parentWidget(), rect, text, group(), groupText, 0, TRUE );
 }
 
 
-/*! Remove all tool tips for this widget immediately. */
+/*!
+  Remove all tool tips for this widget immediately.
+*/
 
 void QToolTip::clear()
 {
@@ -716,7 +867,8 @@ void QToolTip::clear()
 }
 
 
-/*! \fn QWidget * QToolTip::parentWidget() const
+/*!
+  \fn QWidget * QToolTip::parentWidget() const
 
   Returns the widget this QToolTip applies to.
 
@@ -727,28 +879,28 @@ void QToolTip::clear()
 */
 
 
-/*! \fn QToolTipGroup * QToolTip::group() const
+/*!
+  \fn QToolTipGroup * QToolTip::group() const
 
   Returns the tool tip group this QToolTip is a member of, of 0 if it
   isn't a member of any group.
 
-  The tool tip group is the object responsible relaying contact
+  The tool tip group is the object responsible for relaying contact
   betweeen tool tips and a status bar or something else which can show
   a longer help text.
 
-  \sa parentWidget() QToolTipGroup
+  \sa parentWidget(), QToolTipGroup
 */
 
 
+/*!
+  \class QToolTipGroup qtooltip.h
 
-/*! \class QToolTipGroup qtooltip.h
+  \brief The QToolTipGroup class collects tool tips into natural groups.
 
-  \brief The QToolTipGroup class provides a way to group tool tips
-  into natural groups.
-
-  Tool tips can display \e two texts, the one in the tip an optionally
-  another one, typically in a status bar.  QTooTipGroup provides a way
-  to link tool tips to this status bar.
+  Tool tips can display \e two texts, the one in the tip and
+  optionally another one, typically in a status bar.  QToolTipGroup
+  provides a way to link tool tips to this status bar.
 
   QToolTipGroup has practically no API, it is only used as an argument
   to QToolTip's member functions, for example like this:
@@ -771,7 +923,23 @@ void QToolTip::clear()
   a meal" while the relevant tool tips are being displayed.
 
   Deleting a tool tip group removes the tool tips in it.
+*/
 
+/*! \fn void QToolTipGroup::showTip (const char* longText)
+
+  This signal is emitted when one of the tool tips in the group is
+  displayed.  \a longText is the supplementary text for the displayed
+  tool tip.
+
+  \sa removeTip()
+*/
+
+/*! \fn void QToolTipGroup::removeTip ()
+
+  This signal is emitted when a tool tip in this group is hidden.  See
+  the QToolTipGroup documentation for an example of use.
+
+  \sa showTip()
 */
 
 
@@ -779,14 +947,13 @@ void QToolTip::clear()
   Constructs a tool tip group.
 */
 
-QToolTipGroup::QToolTipGroup( QObject * parent, const char * name )
+QToolTipGroup::QToolTipGroup( QObject *parent, const char *name )
     : QObject( parent, name )
 {
-    initMetaObject();
+    // nothing
 }
 
 
-
 /*!
   Destroy this tool tip groups and all tool tips in it.
 */
