@@ -4,8 +4,8 @@
 **
 ****************************************************************/
 
-#include <qapp.h>
-#include <qpushbt.h>
+#include <qapplication.h>
+#include <qpushbutton.h>
 #include <qfont.h>
 
 
